;************************************************************************
;*									*
;* Multi-6809 Emulation Engine						*
;*									*
;* Copyright 1997, Pat Lawrence						*
;* Version 0.01								*
;*									*
;* This code is free to distribute and use in unmodified form for your	*
;* own projects requiring a 6809 emulator. This emulator is designed to	*
;* give the best emulation performance on a PC as possible.		*
;*									*
;* This code, and no part of this code, may not be used in any		*
;* commercial or for-profit venture without the express written		*
;* permission of Pat Lawrence. Credit must be given within the program	*
;* that uses this emulator OR in accompanying documentation.		*
;*									*
;* In case you hadn't noticed, this code is utilizing the structure	*
;* Neil Bradley created with his m6502 CPU emulator.  This is a der-	*
;* ivative work.  If there are problems, they are mine.  Don't bug Neil *
;* about this code!							*
;*									*
;* Contact information:							*
;*									*
;* Pat Lawrence		- pjl@ns.net - Author				*
;*									*
;************************************************************************
;*									*
;* Release history:							*
;*									*
;*									*
;************************************************************************

.486p
.model flat

;************************************************************************
;*									*
;* Philosophy of operation						*
;*									*
;* M6809 Is very macro based. This is to reduce the number of calls &	*
;* jumps that is critical to its operation. One could further modulize	*
;* this emulator by providing a very large call table for all adressing *
;* modes in addition to each call, but that would take significantly	*
;* longer to execute. The goal of this emulator is to be as fast as	*
;* possible for the 486 and higher CPUs.				*
;*									*
;* All of the most commonly used 6809 registers are kept in x86		*
;* registers for increased speed. They are:				*
;*									*
;* The 8 bit registers are:						*
;*	A -- Accumulator						*
;*	B -- Accumulator						*
;*	CC -- Flag Register						*
;*	DP -- Direct Page Register					*
;* The 16 bit registers are:						*
;*	X -- Index Register						*
;*	Y -- Index Register						*
;*	U -- Stack Pointer						*
;*	S -- Stack Pointer						*
;*	PC -- Program Counter						*
;*									*
;* EAX - Instruction fetch, general purpose, etc... be sure to ensure	*
;*	 that the upper 24 bits of this register are clear when		*
;*	 returning from an emulated function. It uses this to compute	*
;*	 an offset to where to jump for the next instruction.		*
;*									*
;* EBX - BH=The 6809's A register, BL=The 6809's B register.  Leave	*
;*	 the upper 16 bits clear.					*
;*									*
;* ECX - CX=The 6809's X index register.  Leave the upper 16 bits	*
;*	 clear as well.							*
;*									*
;* EDX - DH=The 6809's flags, DL=The 6809's direct page register.	*
;*	 The upper 16 bits are not used. Leave at zero.			*
;*									*
;* ESI - The 6809's program counter + emulation address space base addr.*
;*	 Used in instruction fetching. All 32 bits are used as the	*
;*	 currently executing address.					*
;*									*
;* EDI - Temporary address computation register.			*
;*									*
;* EBP - The base address of the virtual 64K 6809 machine. 32 Bits!	*
;*									*
;* All registers persist while executing, and are saved when execution	*
;* is complete.								*
;*									*
;* To use:								*
;*									*
;* * From your C program, allocate 64K of memory space for the 6809	*
;*   emulator to run it. Set _m6809Memory to that address.		*
;*									*
;* * Call _m6809Reset to cause the 6809 to reset itself			*
;*									*
;* * Set _m6809MemoryRead and _m6809MemoryWrite to your structures	*
;*   dealing  with read/writes to various places in memory. (See	*
;*   m6809.H for more info						*
;*									*
;* * Set _m6809Base to the 6809's logical address of 0 (64K buffer)	*
;*									*
;* * Call _exec6809() to begin emulation execution. See the _exec6809() *
;*   comment header for more information on this call.			*
;*									*
;* Optional operational calls:						*
;*									*
;* * _m6809int() - Call this when you wish to create a virtual 6809	*
;*   interrupt.								*
;*									*
;* * m6809nmi_() - Call this when you wish to create a virtual 6809 non-*
;*   maskable interrupt.						*
;*									*
;* All variables within the CONTEXT comments need to be swapped out	*
;* when doing multi-6809 processing.					*
;*									*
;* To assemble:								*
;*									*
;* For MASM	    :	 ML /c /Cx /DMASM m6809.asm			*
;*									*
;************************************************************************

;************************************************************************
;*									*
;* Todo:								*
;*									*
;* W.R.T. EXG/TFR -- Other operand values result in the constant $FF	*
;*        being transferred; this value is also used when there is an	*
;*	  operation mixing 8 and 16 bit registers (these effects are	*
;*	  undocumented).  This is not done yet...			*
;*									*
;* Implement CWAI and RESET						*
;*									*
;************************************************************************


;************************************************************************
;*									*
;* Equates, publics and options						*
;*									*
;************************************************************************

ifdef MASM
		public	_m6809Base
		public	_m6809MemoryRead
		public	_m6809MemoryWrite
		public	_m6809pc
		public	_m6809x
		public	_m6809y
		public	_m6809u
		public	_m6809s
		public	_m6809dp
		public	_m6809b
		public	_m6809a
		public	_m6809cc

		public	m6809nmi_, _m6809nmi
		public	m6809int_, _m6809int
		public	m6809exec_, _m6809exec
		public	m6809reset_, _m6809reset
		public	m6809GetContextSize_, _m6809GetContextSize
		public	m6809SetContext_, _m6809SetContext
		public	m6809GetContext_, _m6809GetContext

		extern	_cyclesRemaining:dword
		extern	_dwElapsedTicks:dword
endif

MAJOR_REV	equ	0	; Major revision
MINOR_REV	equ	1	; Minor revision
MULTI_CPU	equ	1	; Enable multi-CPU support
INST_COUNT	equ	0	; We're counting down by instructions
CYCLE_COUNT	equ	1	; We're counting down by virtual clock time

ifdef MULTI_CPU

ifdef MASM
		public	m6809GetContextSize_
		public	m6809SetContext_
		public	m6809GetContext_
endif

endif

; M6809.ASM OPTIONS:

; Comment this out to not include timing information in emulation

TICKS		equ	1	; Uncomment if we want a clock count

; Comment this out when you have released your code. This does a check to
; make sure that each instruction is fetched within the 64K segment in
; question. It will slow execution significantly, so comment it back out when
; you're ready to release your emulation.

;BOUNDS_CHECK	 equ	 1	; Uncomment if you want bounds checking for
				; address space. Comment this back out when
				; your emulation is working properly

; This controls what the value passed to m6809exec actually means. If this
; is set to the INST_COUNT method, then it will consider the value input to be
; the number of virtual instructions left to execute. If this is set to
; CYCLE_COUNT then it is considered to be the number of virtual cycles left
; to execute.
;
; Be warned that changing this from INST_COUNT to CYCLE_COUNT will give
; approximately a 9% performance hit. If the image being emulated doesn't
; need exact timing, then keep it at INST_COUNT and approximate the instructions
; that pass between each cycle.

CYCLE_METHOD	equ	CYCLE_COUNT ; Count instructions

; ** DO NOT MUCK WITH THESE LABELS **

; FLAG REGISTER BIT FIELD NAMES
;
CARRY		equ	01h	; Carry flag
OVERFLOW	equ	02h	; Overflow flag
ZERO		equ	04h	; Zero flag
NEGATIVE	equ	08h	; Negaitve flag
IRQ		equ	10h	; IRQ mask
HALFCARRY	equ	20h	; Half carry flag
FIRQ		equ	40h	; Fast IRQ mask
ENTIRE		equ	80h	; Entire flag

;************************************************************************
;*									*
;* Macros								*
;*									*
;************************************************************************

; Used when fetching instructions. It fetches the byte into AL from what is
; pointed to by ESI and leaves all other registers undisturbed

FetchInstructionByte	macro

		mov	al, [esi]
		inc	esi

		endm

; Used when fetching part of instructions (addresses, offsets, etc...). It
; fetches the two byte data into AX from ESI and leaves the other registers
; undisturbed.	The data is read from memory in Big Endian format and it 
; is converted to Little Endian format so we can actually use it...

FetchInstructionWord	macro
		mov	ax, [esi]
		add	esi, 2
		xchg	al, ah		; Swap AL and AH
		endm

; Used when fetching an address in direct mode.	 The address is calculated
; by:
;	(DP << 8) | operand
;
; The address to fetch from is stored in AX.

FetchDirectWord		macro
		mov	al, [esi]	; Get our direct word
		mov	ah, dl		; Get the DP value
		inc	esi

		endm

; Used when attempting to do a memory read from any address. The read address
; is expected to be in AX - the 6809's 64K memory space. Any fixups are done
; by routines that may have it covered. AL=Byte that just got read.

ReadMemoryByte	macro
		local	checkLoop, nextAddr, memoryRead, readMacroExit, callRoutine

ifdef MASM
		mov	edi, [_m6809MemoryRead] ; Point to the read array
endif

checkLoop:
		cmp	[edi], word ptr -1 ; Is it the end of the list?
		je	memoryRead	; Yes, go read it

		cmp	ax, [edi]	; Are we smaller?
		jb	nextAddr	; Yes... go to the next address

		cmp	ax, [edi+4]	; Are we bigger?
		jbe	callRoutine

nextAddr:
		add	edi, 16		; To the next structure
		jmp	short checkLoop ; Keep looping until we're done

callRoutine:
		push	esi
		push	ebx
		push	ecx
		push	edx
		push	ebp

ifdef USE_STACK
		push	edi		; Our ReadMemoryByte structure
		push	eax		; The address
endif
		sub		esi, ebp	; Get the PC into SI
ifdef MASM
		mov	_m6809pc, si	; Save it.
endif

		mov	edx, edi	; Our parameter passing
		and	eax, 0ffffh	; Only the first 16 bits
		call	dword ptr [edi + 8]	; Go call our handler!
ifdef USE_STACK
		add	esp,8
endif

		pop	ebp
		pop	edx
		pop	ecx
		pop	ebx
		pop	esi

		jmp	short readMacroExit

memoryRead:
		mov	al, [ebp + eax] ; Go get it!

readMacroExit:
		and	eax, 0ffh	; Only the lower byte
		xor	edi, edi	; Zero EDI
		endm

; Used when attempting to do a memory read of a word from any address. The read
; address is expected to be in AX - the 6809's 64K memory space. Any fixups are
; done by routines that may have it covered. AX=Word that just got read.  The
; macro just calls ReadMemoryByte twice since the routines that ReadMemoryByte
; may call are only returning a byte.  Upper 16 bits of EBX and ECX are trashed!!!

ReadMemoryWord	macro
		push	ecx		; Save CX
		push	ax		; Save AX for later...

		ReadMemoryByte		; Read our second byte first
		mov	ch, al		; It's the MSB, so put it in CH

		pop	ax		; Restore our address
		inc	ax		; Add 1

		ReadMemoryByte		; Read our first byte second
		mov	cl, al		; It's the LSB, so put it in CL

		mov	ax, cx		; Copy it to AX
		pop	ecx		; Restore CX
		endm

; Used when writing a memory byte to an address space. The address is
; expected to be in AX, and the byte to write in DL. Any fixups are done by
; the routines that have covered it.


WriteMemoryByte macro
		local	checkLoop, nextAddr, callRoutine, memoryWrite, writeMacroExit, noReloadPc

ifdef MASM
		mov	edi, [_m6809MemoryWrite] ; Point to the write array
endif

checkLoop:
		cmp	[edi], word ptr -1 ; Is it the end of the list?
		je	memoryWrite	; Yes, go write it

		cmp	ax, [edi]	; Are we smaller?
		jb	nextAddr	; Yes... go to the next address

		cmp	ax, [edi+4]	; Are we bigger?
		jbe	callRoutine

nextAddr:
		add	edi, 16		; To the next structure
		jmp	short checkLoop ; Keep looping until we're done

callRoutine:
		push	ebx
		push	ecx
		push	edx
		push	esi
		push	ebp

ifdef USE_STACK
		push	edi		; Our WriteMemoryByte structure
		push	edx		; The byte value
		push	eax		; The address
endif
		mov	ecx, edi	; Our parameter passing
		and	eax, 0ffffh	; Only the first 16 bits
		call	dword ptr [edi + 8]	; Go call our handler!
ifdef USE_STACK
		add	esp,12
endif

		pop	ebp
		pop	esi
		pop	edx
		pop	ecx
		pop	ebx

		and	eax, 0ffffh
		jmp	short writeMacroExit

memoryWrite:
		mov	[ebp + eax], dl ; Go get it!

writeMacroExit:
		xor	edi, edi	; Zero EDI
		endm

; Used when writing a memory word to an address space. The address is
; expected to be in AX, and the word to write in DX. Any fixups are done by
; the routines that have covered it.

WriteMemoryWord	macro
		push	ebx		; Save EBX
		push	edx		; Save DX
		push	ax		; Save AX for later

		mov	bx, dx		; Save our data to write
		xor	dh, dh		; Clear out DH just to be safe

		mov	dl, bh		; Move the MSB into DL
		WriteMemoryByte		; Write our byte

		pop	ax		; Restore AX
		inc	ax		; Add 1

		mov	dl, bl		; Move the LSB into DL
		WriteMemoryByte		; Write our byte

		pop	edx		; Restore DX
		pop	ebx		; Restore BX

		endm

; This is the macro to use to add something to our clock.

Clocktime	macro	addedTime

ifdef TICKS

ifdef MASM
endif

endif
		endm

; This macro is used at the end of an emulated function. It will return to the
; main execution loop to get the next instruction. IT IS SUPPOSED TO BE A
; JUMP AND NOT A CALL!

ExecuteNextInstruction	macro	clkTiming
		local	noMoreStuff

; If we're doing a countdown of the # of instructions, then use this code

if CYCLE_METHOD EQ INST_COUNT

ifdef MASM
		dec	_cyclesRemaining	; Next!
endif

		jz	noMoreStuff		; If we have nothing left...
endif

; If we're doing a countdown of virtual CPU clock cycles, then use THIS code

if CYCLE_METHOD EQ CYCLE_COUNT

ifdef MASM
		add	_dwElapsedTicks, clkTiming
		sub	_cyclesRemaining, clkTiming ; Subtract the virtual clock ticks
endif

		jc	noMoreStuff		; If we have underflowed...
endif


ifdef BOUNDS_CHECK
		mov	eax, esi		; Get our naturalized PC
		sub	eax, ebp		; Subtract base memory address
		test	eax, 0ffff0000h		; Are we out of range?
		jnz	OutOfRange

		xor	eax, eax		; Zero here for index lookup
endif

		mov	al, [esi]
		inc	esi
		jmp	dword ptr m6809zero[eax*4] ; Jump to our instruction!

noMoreStuff:

; MASM Is brain-dead about automatically extending shorts to longs, so it
; needs everything as a long

ifdef MASM
		jmp	instReturn
endif

		endm

; This macro is used to execute a page one instruction.  It will return to the
; main execution loop to get the next instruction. IT IS SUPPOSED TO BE A
; JUMP AND NOT A CALL!

ExecutePageOne	macro
		local	noMoreStuff

ifdef BOUNDS_CHECK
		mov	eax, esi		; Get our naturalized PC
		sub	eax, ebp		; Subtract base memory address
		test	eax, 0ffff0000h		; Are we out of range?
		jnz	OutOfRange

		xor	eax, eax		; Zero here for index lookup
endif

		mov	al, [esi]

		inc	esi
		jmp	dword ptr m6809one[eax*4] ; Jump to our instruction!

		endm

; This macro is used to execute a page two instruction.  It will return to the
; main execution loop to get the next instruction. IT IS SUPPOSED TO BE A
; JUMP AND NOT A CALL!

ExecutePageTwo	macro
		local	noMoreStuff

ifdef BOUNDS_CHECK
		mov	eax, esi		; Get our naturalized PC
		sub	eax, ebp		; Subtract base memory address
		test	eax, 0ffff0000h		; Are we out of range?
		jnz	OutOfRange

		xor	eax, eax		; Zero here for index lookup
endif

		mov	al, [esi]
		inc	esi
		jmp	dword ptr m6809two[eax*4] ; Jump to our instruction!

		endm

; This macro is used at the beginning of every emulated function, and is
; different between assemblers

Alignment	macro

ifdef MASM
align 
endif

		endm

; Macro used when all flags need to be set (value in AL)

FlagSet		macro
		and	dh, 0F3h	; Knock out zero and SIGN
		xor	ah, ah		; Zero this just in case
		or	dh, byte ptr flagTable[eax] ; Go get our flag

		endm

; Macro used when all flags need to be set for a word (value in AX)

FlagSetWord	macro
		local	noFlagSetWordNeg, noFlagSetWordZero
		and	dh, 0F3h	; Knock out zero and SIGN
		or	ax, ax		; Is it signed?
		jns	noFlagSetWordNeg
		or	dh, NEGATIVE	; Yes, it is

noFlagSetWordNeg:
		or	ax, ax		; Is AX zero?
		jnz	noFlagSetWordZero	; Nope
		or	dh, ZERO	; Yes, it is

noFlagSetWordZero:
		endm

; Macro used when all flags need to be set for D (value in BX)

FlagSetD	macro
		local	noFlagSetDNeg, noFlagSetDZero
		and	dh, 0F3h	; Knock out zero and SIGN
		or	bx, bx		; Is it signed?
		jns	noFlagSetDNeg
		or	dh, NEGATIVE	; Yes, it is

noFlagSetDNeg:
		or	bx, bx		; BX Zero?
		jnz	noFlagSetDZero	; Nope
		or	dh, ZERO	; Yes, it is

noFlagSetDZero:
		endm

; Macro used when all flags need to be set for X (value in CX)

FlagSetX	macro
		local	noFlagSetXNeg, noFlagSetXZero
		and	dh, 0F3h	; Knock out zero and SIGN
		or	cx, cx		; Is it signed?
		jns	noFlagSetXNeg
		or	dh, NEGATIVE	; Yes, it is
noFlagSetXNeg:
		or	cx, cx		; CX Zero?
		jnz	noFlagSetXZero	; Nope
		or	dh, ZERO	; Yes, it is

noFlagSetXZero:
		endm

; Used to load stack pointer S from our data stucture into AX.

LoadStackS	macro
ifdef MASM
		mov	ax, _m6809s	; Copy the S stack pointer
endif
		endm

; Used to unload stack pointer S from AX into our data stucture.

UnloadStackS	macro
ifdef MASM
		mov	_m6809s, ax	; Save S
endif
		endm

; Used to load stack pointer U from our data stucture into AX.

LoadStackU	macro
ifdef MASM
		mov	ax, _m6809u	; Copy the U stack pointer
endif
		endm

; Used to unload stack pointer U from AX into our data stucture.

UnloadStackU	macro
ifdef MASM
		mov	_m6809u, ax	; Save U
endif
		endm

; Used when pushing a word onto the stack.  The address of the stack is in AX.
; The data to write is in DX (in Little Endian format.)

PushStackWord	macro
		dec	ax		; Decrement AX
		push	ax		; Save AX for later

		WriteMemoryByte		; Write out the LO byte first
		xchg	dl, dh		; Swap DX around

		pop	ax		; Restore AX
		dec	ax		; Decrement AX
		push	ax		; Save AX for later

		WriteMemoryByte		; Write out the LO byte again...
		xchg	dl, dh		; Get DX right again

		pop	ax		; Restore AX

		endm

; Used when pushing a byte onto the stack.  The address of the stack is in AX.
; The data to write is in DL.

PushStackByte	macro
		dec	ax		; Decrement the stack pointer
		WriteMemoryByte		; Write out the LO byte first

		endm

; Used when popping a word off of the stack.  The address of the stack is in AX.
; The data is read into DX (in Little Endian format.)

PopStackWord	macro
		push	ax		; Save AX since we need it twice
		push	ax		; Save AX since we need it thrice

		ReadMemoryByte		; Read the HO byte first
		mov	dh, al		; Save it in DH

		pop	ax		; Get AX back
		inc	ax		; Increment the stack pointer

		ReadMemoryByte		; Read the LO byte
		mov	dl, al		; Save it to DL

		pop	ax		; Get AX back
		add	ax, 2		; Increment the stack pointer

		endm

; Used when popping a byte off of the stack.  The address of the stack is in AX.
; The data is read into DL.

PopStackByte	macro
		push	ax		; Save AX since we need it twice

		ReadMemoryByte		; Read the LO byte first
		mov	dl, al		; Save it in DL

		pop	ax		; Get AX back
		inc	ax		; Increment the stack pointer

		endm

; Used for normal branches.  The amount to branch is stored in AL and it
; is a signed value.

Branch		macro

		cbw			; Convert AL to AX
		sub	esi, ebp	; Get our PC in SI
		add	si, ax		; Add in our offset
		and	esi, 00000FFFFh	; Clear out the upper 16 bits
		add	esi, ebp	; Add in the base pointer again
		xor	ah, ah		; Clear out AH

		endm

; Used for l branches.	The amount to branch is stored in AX and it
; is a signed value.

LBranch		macro

		sub	esi, ebp	; Get our PC in SI
		add	si, ax		; Add in our offset
		and	esi, 00000FFFFh	; Clear out the upper 16 bits
		add	esi, ebp	; Add in the base pointer again
		xor	ah, ah		; Clear out AH

		endm

;************************************************************************
;*									*
;* Emulation data							*
;*									*
;************************************************************************

ifdef MASM
_DATA		segment		public 'DATA'
endif

Alignment

; CONTEXT CONTEXT CONTEXT CONTEXT CONTEXT CONTEXT CONTEXT CONTEXT CONTEXT

ifdef MASM
contextBegin::
endif

ifdef MASM
_m6809Base	dd	0	; Base address for 6809 stuff
_m6809MemoryRead	dd	0	; Offset of memory read structure array
_m6809MemoryWrite	dd	0	; Offset of memory write structure array
_m6809pc	dw	0	; PC register
_m6809u		dw	0	; U stack pointer
_m6809s		dw	0	; S stack pointer
_m6809x		dw	0	; X index register
_m6809y		dw	0	; Y index register
_m6809b		db	0	; Accumulator B
_m6809a		db	0	; Accumulator A
_m6809cc	db	0	; Status flags
_m6809dp	db	0	; Direct page register
_inNmi		db	0	; Non-zero if we're in an NMI
endif

ifdef MASM
contextEnd::
endif

; CONTEXT CONTEXT CONTEXT CONTEXT CONTEXT CONTEXT CONTEXT CONTEXT CONTEXT

; This is general purpose non-persistent data

Alignment

; This is the jump table for the page zero instructions

m6809zero	dd	inst00			; 00h - NEG	Direct
		dd	invalidInsByte		; 01h -		Invalid
		dd	invalidInsByte		; 02h -		Invalid
		dd	inst03			; 03h - COM	Direct
		dd	inst04			; 04h - LSR	Direct
		dd	invalidInsByte		; 05h -		Invalid
		dd	inst06			; 06h - ROR	Direct
		dd	inst07			; 07h - ASR	Direct
		dd	inst08			; 08h - LSL/ASL	Direct
		dd	inst09			; 09h - ROL	Direct
		dd	inst0A			; 0Ah - DEC	Direct
		dd	invalidInsByte		; 0Bh -		Invalid
		dd	inst0C			; 0Ch - INC	Direct
		dd	inst0D			; 0Dh - TST	Direct
		dd	inst0E			; 0Eh - JMP	Direct
		dd	inst0F			; 0Fh - CLR	Direct
		dd	inst10			; 10h - PAGE1+	Variant
		dd	inst11			; 11h - PAGE2+	Variant
		dd	inst12			; 12h - NOP	Inherent
		dd	inst13			; 13h - SYNC	Inherent
		dd	invalidInsByte		; 14h -		Invalid
		dd	invalidInsByte		; 15h -		Invalid
		dd	inst16			; 16h - LBRA	Relative
		dd	inst17			; 17h - LBSR	Relative
		dd	invalidInsByte		; 18h -		Invalid
		dd	inst19			; 19h - DAA	Inherent
		dd	inst1A			; 1Ah - ORCC	Immediate
		dd	invalidInsByte		; 1Bh -		Invalid
		dd	inst1C			; 1Ch - ANDCC	Immediate
		dd	inst1D			; 1Dh - SEX	Inherent
		dd	inst1E			; 1Eh - EXG	Inherent
		dd	inst1F			; 1Fh - TFR	Inherent
		dd	inst20			; 20h - BRA	Relative
		dd	inst21			; 21h - BRN	Relative
		dd	inst22			; 22h - BHI	Relative
		dd	inst23			; 23h - BLS	Relative
		dd	inst24			; 24h - BCC	Relative
		dd	inst25			; 25h - BCS	Relative
		dd	inst26			; 26h - BNE	Relative
		dd	inst27			; 27h - BEQ	Relative
		dd	inst28			; 28h - BVC	Relative
		dd	inst29			; 29h - BVS	Relative
		dd	inst2A			; 2Ah - BPL	Relative
		dd	inst2B			; 2Bh - BMI	Relative
		dd	inst2C			; 2Ch - BGE	Relative
		dd	inst2D			; 2Dh - BLT	Relative
		dd	inst2E			; 2Eh - BGT	Relative
		dd	inst2F			; 2Fh - BLE	Relative
		dd	inst30			; 30h - LEAX	Indexed
		dd	inst31			; 31h - LEAY	Indexed
		dd	inst32			; 32h - LEAS	Indexed
		dd	inst33			; 33h - LEAU	Indexed
		dd	inst34			; 34h - PSHS	Inherent
		dd	inst35			; 35h - PULS	Inherent
		dd	inst36			; 36h - PSHU	Inherent
		dd	inst37			; 37h - PULI	Inherent
		dd	invalidInsByte		; 38h -		Invalid
		dd	inst39			; 39h - RTS	Inherent
		dd	inst3A			; 3Ah - ABX	Inherent
		dd	inst3B			; 3Bh - RTI	Inherent
;		dd	inst3C			; 3Ch - CWAI	Inherent
		dd	invalidInsByte		; 3Ch -		Invalid
		dd	inst3D			; 3Dh - MUL	Inherent
;		dd	inst3E			; 3Eh - RESET	Inherent
		dd	invalidInsByte		; 3Eh -		Invalid
		dd	inst3F			; 3Fh - SWI	Inherent
		dd	inst40			; 40h - NEGA	Inherent
		dd	invalidInsByte		; 41h -		Invalid
		dd	invalidInsByte		; 42h -		Invalid
		dd	inst43			; 43h - COMA	Inherent
		dd	inst44			; 44h - LSRA	Inherent
		dd	invalidInsByte		; 45h -		Invalid
		dd	inst46			; 46h - RORA	Inherent
		dd	inst47			; 47h - ASRA	Inherent
		dd	inst48			; 48h - LSLA/ASLA	Inherent
		dd	inst49			; 49h - ROLA	Inherent
		dd	inst4A			; 4Ah - DECA	Inherent
		dd	invalidInsByte		; 4Bh -		Invalid
		dd	inst4C			; 4Ch - INCA	Inherent
		dd	inst4D			; 4Dh - TSTA	Inherent
		dd	invalidInsByte		; 4Eh -		Inherent
		dd	inst4F			; 4Fh - CLRA	Inherent
		dd	inst50			; 50h - NEGB	Inherent
		dd	invalidInsByte		; 51h -		Invalid
		dd	invalidInsByte		; 52h -		Invalid
		dd	inst53			; 53h - COMB	Inherent
		dd	inst54			; 54h - LSRB	Inherent
		dd	invalidInsByte		; 55h -		Invalid
		dd	inst56			; 56h - RORB	Inherent
		dd	inst57			; 57h - ASRB	Inherent
		dd	inst58			; 58h - LSLB/ASLB	Inherent
		dd	inst59			; 59h - ROLB	Inherent
		dd	inst5A			; 5Ah - DECB	Inherent
		dd	invalidInsByte		; 5Bh -		Invalid
		dd	inst5C			; 5Ch - INCB	Inherent
		dd	inst5D			; 5Dh - TSTB	Inherent
		dd	invalidInsByte		; 5Eh -		Inherent
		dd	inst5F			; 5Fh - CLRB	Inherent
		dd	inst60			; 60h - NEG	Indexed
		dd	invalidInsByte		; 61h -		Invalid
		dd	invalidInsByte		; 62h -		Invalid
		dd	inst63			; 63h - COM	Indexed
		dd	inst64			; 64h - LSR	Indexed
		dd	invalidInsByte		; 65h -		Invalid
		dd	inst66			; 66h - ROR	Indexed
		dd	inst67			; 67h - ASR	Indexed
		dd	inst68			; 68h - LSL/ASL	Indexed
		dd	inst69			; 69h - ROL	Indexed
		dd	inst6A			; 6Ah - DEC	Indexed
		dd	invalidInsByte		; 6Bh -		Invalid
		dd	inst6C			; 6Ch - INC	Indexed
		dd	inst6D			; 6Dh - TST	Indexed
		dd	inst6E			; 6Eh - JMP	Indexed
		dd	inst6F			; 6Fh - CLR	Indexed
		dd	inst70			; 70h - NEG	Extended
		dd	invalidInsByte		; 71h -		Invalid
		dd	invalidInsByte		; 72h -		Invalid
		dd	inst73			; 73h - COM	Extended
		dd	inst74			; 74h - LSR	Extended
		dd	invalidInsByte		; 75h -		Invalid
		dd	inst76			; 76h - ROR	Extended
		dd	inst77			; 77h - ASR	Extended
		dd	inst78			; 78h - LSL/ASL	Extended
		dd	inst79			; 79h - ROL	Extended
		dd	inst7A			; 7Ah - DEC	Extended
		dd	invalidInsByte		; 7Bh -		Invalid
		dd	inst7C			; 7Ch - INC	Extended
		dd	inst7D			; 7Dh - TST	Extended
		dd	inst7E			; 7Eh - JMP	Extended
		dd	inst7F			; 7Fh - CLR	Extended
		dd	inst80			; 80h - SUBA	Immediate
		dd	inst81			; 81h - CMPA	Immediate
		dd	inst82			; 82h - SBCA	Immediate
		dd	inst83			; 83h - SUBD	Immediate
		dd	inst84			; 84h - ANDA	Immediate
		dd	inst85			; 85h - BITA	Immediate
		dd	inst86			; 86h - LDA	Immediate
		dd	invalidInsByte		; 87h -		Invalid
		dd	inst88			; 88h - EORA	Immediate
		dd	inst89			; 89h - ADCA	Immediate
		dd	inst8A			; 8Ah - ORA	Immediate
		dd	inst8B			; 8Bh - ADDA	Immediate
		dd	inst8C			; 8Ch - CMPX	Immediate
		dd	inst8D			; 8Dh - BSR	Relative
		dd	inst8E			; 8Eh - LDX	Immediate
		dd	invalidInsByte		; 8Fh -		Invalid
		dd	inst90			; 90h - SUBA	Direct
		dd	inst91			; 91h - CMPA	Direct
		dd	inst92			; 92h - SBCA	Direct
		dd	inst93			; 93h - SUBD	Direct
		dd	inst94			; 94h - ANDA	Direct
		dd	inst95			; 95h - BITA	Direct
		dd	inst96			; 96h - LDA	Direct
		dd	inst97			; 97h -	STA	Direct
		dd	inst98			; 98h - EORA	Direct
		dd	inst99			; 99h - ADCA	Direct
		dd	inst9A			; 9Ah - ORA	Direct
		dd	inst9B			; 9Bh - ADDA	Direct
		dd	inst9C			; 9Ch - CMPX	Direct
		dd	inst9D			; 9Dh - JSR	Direct
		dd	inst9E			; 9Eh - LDX	Direct
		dd	inst9F			; 9Fh -	STX	Direct
		dd	instA0			; A0h - SUBA	Indexed
		dd	instA1			; A1h - CMPA	Indexed
		dd	instA2			; A2h - SBCA	Indexed
		dd	instA3			; A3h - SUBD	Indexed
		dd	instA4			; A4h - ANDA	Indexed
		dd	instA5			; A5h - BITA	Indexed
		dd	instA6			; A6h - LDA	Indexed
		dd	instA7			; A7h -	STA	Indexed
		dd	instA8			; A8h - EORA	Indexed
		dd	instA9			; A9h - ADCA	Indexed
		dd	instAA			; AAh - ORA	Indexed
		dd	instAB			; ABh - ADDA	Indexed
		dd	instAC			; ACh - CMPX	Indexed
		dd	instAD			; ADh - JSR	Indexed
		dd	instAE			; AEh - LDX	Indexed
		dd	instAF			; AFh -	STX	Indexed
		dd	instB0			; B0h - SUBA	Extended
		dd	instB1			; B1h - CMPA	Extended
		dd	instB2			; B2h - SBCA	Extended
		dd	instB3			; B3h - SUBD	Extended
		dd	instB4			; B4h - ANDA	Extended
		dd	instB5			; B5h - BITA	Extended
		dd	instB6			; B6h - LDA	Extended
		dd	instB7			; B7h -	STA	Extended
		dd	instB8			; B8h - EORA	Extended
		dd	instB9			; B9h - ADCA	Extended
		dd	instBA			; BAh - ORA	Extended
		dd	instBB			; BBh - ADDA	Extended
		dd	instBC			; BCh - CMPX	Extended
		dd	instBD			; BDh - JSR	Extended
		dd	instBE			; BEh - LDX	Extended
		dd	instBF			; BFh -	STX	Extended
		dd	instC0			; C0h - SUBB	Immediate
		dd	instC1			; C1h - CMPB	Immediate
		dd	instC2			; C2h - SBCB	Immediate
		dd	instC3			; C3h - ADDD	Immediate
		dd	instC4			; C4h - ANDB	Immediate
		dd	instC5			; C5h - BITB	Immediate
		dd	instC6			; C6h - LDB	Immediate
		dd	invalidInsByte		; C7h -		Invalid
		dd	instC8			; C8h - EORB	Immediate
		dd	instC9			; C9h - ADCB	Immediate
		dd	instCA			; CAh - ORB	Immediate
		dd	instCB			; CBh - ADDB	Immediate
		dd	instCC			; CCh - LDD 	Immediate
		dd	invalidInsByte		; CDh -     	Invalid
		dd	instCE			; CEh - LDU	Immediate
		dd	invalidInsByte		; CFh -		Invalid
		dd	instD0			; D0h - SUBB	Direct
		dd	instD1			; D1h - CMPB	Direct
		dd	instD2			; D2h - SBCB	Direct
		dd	instD3			; D3h - ADDD	Direct
		dd	instD4			; D4h - ANDB	Direct
		dd	instD5			; D5h - BITB	Direct
		dd	instD6			; D6h - LDB	Direct
		dd	instD7			; D7h -	STB	Direct
		dd	instD8			; D8h - EORB	Direct
		dd	instD9			; D9h - ADCB	Direct
		dd	instDA			; DAh - ORB	Direct
		dd	instDB			; DBh - ADDB	Direct
		dd	instDC			; DCh - LDD 	Direct
		dd	instDD			; DDh - STD 	Direct
		dd	instDE			; DEh - LDU	Direct
		dd	instDF			; DFh -	STU	Direct
		dd	instE0			; E0h - SUBB	Indexed
		dd	instE1			; E1h - CMPB	Indexed
		dd	instE2			; E2h - SBCB	Indexed
		dd	instE3			; E3h - ADDD	Indexed
		dd	instE4			; E4h - ANDB	Indexed
		dd	instE5			; E5h - BITB	Indexed
		dd	instE6			; E6h - LDB	Indexed
		dd	instE7			; E7h -	STB	Indexed
		dd	instE8			; E8h - EORB	Indexed
		dd	instE9			; E9h - ADCB	Indexed
		dd	instEA			; EAh - ORB	Indexed
		dd	instEB			; EBh - ADDB	Indexed
		dd	instEC			; ECh - LDD 	Indexed
		dd	instED			; EDh - STD 	Indexed
		dd	instEE			; EEh - LDU	Indexed
		dd	instEF			; EFh -	STU	Indexed
		dd	instF0			; F0h - SUBB	Extended
		dd	instF1			; F1h - CMPB	Extended
		dd	instF2			; F2h - SBCB	Extended
		dd	instF3			; F3h - ADDD	Extended
		dd	instF4			; F4h - ANDB	Extended
		dd	instF5			; F5h - BITB	Extended
		dd	instF6			; F6h - LDB	Extended
		dd	instF7			; F7h -	STB	Extended
		dd	instF8			; F8h - EORB	Extended
		dd	instF9			; F9h - ADCB	Extended
		dd	instFA			; FAh - ORB	Extended
		dd	instFB			; FBh - ADDB	Extended
		dd	instFC			; FCh - LDD 	Extended
		dd	instFD			; FDh - STD 	Extended
		dd	instFE			; FEh - LDU	Extended
		dd	instFF			; FFh -	STU	Extended

Alignment

; This is the jump table for the page one instructions

m6809one	dd	invalidPageInsByte	; 1000h -	Invalid
		dd	invalidPageInsByte	; 1001h -	Invalid
		dd	invalidPageInsByte	; 1002h -	Invalid
		dd	invalidPageInsByte	; 1003h -	Invalid
		dd	invalidPageInsByte	; 1004h -	Invalid
		dd	invalidPageInsByte	; 1005h -	Invalid
		dd	invalidPageInsByte	; 1006h -	Invalid
		dd	invalidPageInsByte	; 1007h -	Invalid
		dd	invalidPageInsByte	; 1008h -	Invalid
		dd	invalidPageInsByte	; 1009h -	Invalid
		dd	invalidPageInsByte	; 100Ah -	Invalid
		dd	invalidPageInsByte	; 100Bh -	Invalid
		dd	invalidPageInsByte	; 100Ch -	Invalid
		dd	invalidPageInsByte	; 100Dh -	Invalid
		dd	invalidPageInsByte	; 100Eh -	Invalid
		dd	invalidPageInsByte	; 100Fh -	Invalid
		dd	invalidPageInsByte	; 1010h -	Invalid
		dd	invalidPageInsByte	; 1011h -	Invalid
		dd	invalidPageInsByte	; 1012h -	Invalid
		dd	invalidPageInsByte	; 1013h -	Invalid
		dd	invalidPageInsByte	; 1014h -	Invalid
		dd	invalidPageInsByte	; 1015h -	Invalid
		dd	invalidPageInsByte	; 1016h -	Invalid
		dd	invalidPageInsByte	; 1017h -	Invalid
		dd	invalidPageInsByte	; 1018h -	Invalid
		dd	invalidPageInsByte	; 1019h -	Invalid
		dd	invalidPageInsByte	; 101Ah -	Invalid
		dd	invalidPageInsByte	; 101Bh -	Invalid
		dd	invalidPageInsByte	; 101Ch -	Invalid
		dd	invalidPageInsByte	; 101Dh -	Invalid
		dd	invalidPageInsByte	; 101Eh -	Invalid
		dd	invalidPageInsByte	; 101Fh -	Invalid
		dd	invalidPageInsByte	; 1020h -	Invalid
		dd	inst121			; 1021h - LBRN	Relative
		dd	inst122			; 1022h - LBHI	Relative
		dd	inst123			; 1023h - LBLS	Relative
		dd	inst124			; 1024h - LBCC	Relative
		dd	inst125			; 1025h - LBCS	Relative
		dd	inst126			; 1026h - LBNE	Relative
		dd	inst127			; 1027h - LBEQ	Relative
		dd	inst128			; 1028h - LBVC	Relative
		dd	inst129			; 1029h - LBVS	Relative
		dd	inst12A			; 102Ah - LBPL	Relative
		dd	inst12B			; 102Bh - LBMI	Relative
		dd	inst12C			; 102Ch - LBGE	Relative
		dd	inst12D			; 102Dh - LBLT	Relative
		dd	inst12E			; 102Eh - LBGT	Relative
		dd	inst12F			; 102Fh - LBLE	Relative
		dd	invalidPageInsByte	; 1030h -	Invalid
		dd	invalidPageInsByte	; 1031h -	Invalid
		dd	invalidPageInsByte	; 1032h -	Invalid
		dd	invalidPageInsByte	; 1033h -	Invalid
		dd	invalidPageInsByte	; 1034h -	Invalid
		dd	invalidPageInsByte	; 1035h -	Invalid
		dd	invalidPageInsByte	; 1036h -	Invalid
		dd	invalidPageInsByte	; 1037h -	Invalid
		dd	invalidPageInsByte	; 1038h -	Invalid
		dd	invalidPageInsByte	; 1039h -	Invalid
		dd	invalidPageInsByte	; 103Ah -	Invalid
		dd	invalidPageInsByte	; 103Bh -	Invalid
		dd	invalidPageInsByte	; 103Ch -	Invalid
		dd	invalidPageInsByte	; 103Dh -	Invalid
		dd	invalidPageInsByte	; 103Eh -	Invalid
		dd	inst13F			; 103Fh - SWI	Inherent
		dd	invalidPageInsByte	; 1040h -	Invalid
		dd	invalidPageInsByte	; 1041h -	Invalid
		dd	invalidPageInsByte	; 1042h -	Invalid
		dd	invalidPageInsByte	; 1043h -	Invalid
		dd	invalidPageInsByte	; 1044h -	Invalid
		dd	invalidPageInsByte	; 1045h -	Invalid
		dd	invalidPageInsByte	; 1046h -	Invalid
		dd	invalidPageInsByte	; 1047h -	Invalid
		dd	invalidPageInsByte	; 1048h -	Invalid
		dd	invalidPageInsByte	; 1049h -	Invalid
		dd	invalidPageInsByte	; 104Ah -	Invalid
		dd	invalidPageInsByte	; 104Bh -	Invalid
		dd	invalidPageInsByte	; 104Ch -	Invalid
		dd	invalidPageInsByte	; 104Dh -	Invalid
		dd	invalidPageInsByte	; 104Eh -	Invalid
		dd	invalidPageInsByte	; 104Fh -	Invalid
		dd	invalidPageInsByte	; 1050h -	Invalid
		dd	invalidPageInsByte	; 1051h -	Invalid
		dd	invalidPageInsByte	; 1052h -	Invalid
		dd	invalidPageInsByte	; 1053h -	Invalid
		dd	invalidPageInsByte	; 1054h -	Invalid
		dd	invalidPageInsByte	; 1055h -	Invalid
		dd	invalidPageInsByte	; 1056h -	Invalid
		dd	invalidPageInsByte	; 1057h -	Invalid
		dd	invalidPageInsByte	; 1058h -	Invalid
		dd	invalidPageInsByte	; 1059h -	Invalid
		dd	invalidPageInsByte	; 105Ah -	Invalid
		dd	invalidPageInsByte	; 105Bh -	Invalid
		dd	invalidPageInsByte	; 105Ch -	Invalid
		dd	invalidPageInsByte	; 105Dh -	Invalid
		dd	invalidPageInsByte	; 105Eh -	Invalid
		dd	invalidPageInsByte	; 105Fh -	Invalid
		dd	invalidPageInsByte	; 1060h -	Invalid
		dd	invalidPageInsByte	; 1061h -	Invalid
		dd	invalidPageInsByte	; 1062h -	Invalid
		dd	invalidPageInsByte	; 1063h -	Invalid
		dd	invalidPageInsByte	; 1064h -	Invalid
		dd	invalidPageInsByte	; 1065h -	Invalid
		dd	invalidPageInsByte	; 1066h -	Invalid
		dd	invalidPageInsByte	; 1067h -	Invalid
		dd	invalidPageInsByte	; 1068h -	Invalid
		dd	invalidPageInsByte	; 1069h -	Invalid
		dd	invalidPageInsByte	; 106Ah -	Invalid
		dd	invalidPageInsByte	; 106Bh -	Invalid
		dd	invalidPageInsByte	; 106Ch -	Invalid
		dd	invalidPageInsByte	; 106Dh -	Invalid
		dd	invalidPageInsByte	; 106Eh -	Invalid
		dd	invalidPageInsByte	; 106Fh -	Invalid
		dd	invalidPageInsByte	; 1070h -	Invalid
		dd	invalidPageInsByte	; 1071h -	Invalid
		dd	invalidPageInsByte	; 1072h -	Invalid
		dd	invalidPageInsByte	; 1073h -	Invalid
		dd	invalidPageInsByte	; 1074h -	Invalid
		dd	invalidPageInsByte	; 1075h -	Invalid
		dd	invalidPageInsByte	; 1076h -	Invalid
		dd	invalidPageInsByte	; 1077h -	Invalid
		dd	invalidPageInsByte	; 1078h -	Invalid
		dd	invalidPageInsByte	; 1079h -	Invalid
		dd	invalidPageInsByte	; 107Ah -	Invalid
		dd	invalidPageInsByte	; 107Bh -	Invalid
		dd	invalidPageInsByte	; 107Ch -	Invalid
		dd	invalidPageInsByte	; 107Dh -	Invalid
		dd	invalidPageInsByte	; 107Eh -	Invalid
		dd	invalidPageInsByte	; 107Fh -	Invalid
		dd	invalidPageInsByte	; 1080h -	Invalid
		dd	invalidPageInsByte	; 1081h -	Invalid
		dd	invalidPageInsByte	; 1082h -	Invalid
		dd	inst183			; 1083h - CMPD	Immediate
		dd	invalidPageInsByte	; 1084h -	Invalid
		dd	invalidPageInsByte	; 1085h -	Invalid
		dd	invalidPageInsByte	; 1086h -	Invalid
		dd	invalidPageInsByte	; 1087h -	Invalid
		dd	invalidPageInsByte	; 1088h -	Invalid
		dd	invalidPageInsByte	; 1089h -	Invalid
		dd	invalidPageInsByte	; 108Ah -	Invalid
		dd	invalidPageInsByte	; 108Bh -	Invalid
		dd	inst18C			; 108Ch - CMPY	Immediate
		dd	invalidPageInsByte	; 108Dh -	Invalid
		dd	inst18E			; 108Eh - LDY	Immediate
		dd	invalidPageInsByte	; 108Fh -	Invalid
		dd	invalidPageInsByte	; 1090h -	Invalid
		dd	invalidPageInsByte	; 1091h -	Invalid
		dd	invalidPageInsByte	; 1092h -	Invalid
		dd	inst193			; 1093h - CMPD	Direct
		dd	invalidPageInsByte	; 1094h -	Invalid
		dd	invalidPageInsByte	; 1095h -	Invalid
		dd	invalidPageInsByte	; 1096h -	Invalid
		dd	invalidPageInsByte	; 1097h -	Invalid
		dd	invalidPageInsByte	; 1098h -	Invalid
		dd	invalidPageInsByte	; 1099h -	Invalid
		dd	invalidPageInsByte	; 109Ah -	Invalid
		dd	invalidPageInsByte	; 109Bh -	Invalid
		dd	inst19C			; 109Ch - CMPY	Direct
		dd	invalidPageInsByte	; 109Dh -	Invalid
		dd	inst19E			; 109Eh - LDY	Direct
		dd	inst19F			; 109Fh - STY	Direct
		dd	invalidPageInsByte	; 10A0h -	Invalid
		dd	invalidPageInsByte	; 10A1h -	Invalid
		dd	invalidPageInsByte	; 10A2h -	Invalid
		dd	inst1A3			; 10A3h - CMPD	Indexed
		dd	invalidPageInsByte	; 10A4h -	Invalid
		dd	invalidPageInsByte	; 10A5h -	Invalid
		dd	invalidPageInsByte	; 10A6h -	Invalid
		dd	invalidPageInsByte	; 10A7h -	Invalid
		dd	invalidPageInsByte	; 10A8h -	Invalid
		dd	invalidPageInsByte	; 10A9h -	Invalid
		dd	invalidPageInsByte	; 10AAh -	Invalid
		dd	invalidPageInsByte	; 10ABh -	Invalid
		dd	inst1AC			; 10ACh - CMPY	Indexed
		dd	invalidPageInsByte	; 10ADh -	Invalid
		dd	inst1AE			; 10AEh - LDY	Indexed
		dd	inst1AF			; 10AFh - STY	Indexed
		dd	invalidPageInsByte	; 10B0h -	Invalid
		dd	invalidPageInsByte	; 10B1h -	Invalid
		dd	invalidPageInsByte	; 10B2h -	Invalid
		dd	inst1B3			; 10B3h - CMPD	Extended
		dd	invalidPageInsByte	; 10B4h -	Invalid
		dd	invalidPageInsByte	; 10B5h -	Invalid
		dd	invalidPageInsByte	; 10B6h -	Invalid
		dd	invalidPageInsByte	; 10B7h -	Invalid
		dd	invalidPageInsByte	; 10B8h -	Invalid
		dd	invalidPageInsByte	; 10B9h -	Invalid
		dd	invalidPageInsByte	; 10BAh -	Invalid
		dd	invalidPageInsByte	; 10BBh -	Invalid
		dd	inst1BC			; 10BCh - CMPY	Extended
		dd	invalidPageInsByte	; 10BDh -	Invalid
		dd	inst1BE			; 10BEh - LDY	Extended
		dd	inst1BF			; 10BFh - STY	Extended
		dd	invalidPageInsByte	; 10C0h -	Invalid
		dd	invalidPageInsByte	; 10C1h -	Invalid
		dd	invalidPageInsByte	; 10C2h -	Invalid
		dd	invalidPageInsByte	; 10C3h -	Invalid
		dd	invalidPageInsByte	; 10C4h -	Invalid
		dd	invalidPageInsByte	; 10C5h -	Invalid
		dd	invalidPageInsByte	; 10C6h -	Invalid
		dd	invalidPageInsByte	; 10C7h -	Invalid
		dd	invalidPageInsByte	; 10C8h -	Invalid
		dd	invalidPageInsByte	; 10C9h -	Invalid
		dd	invalidPageInsByte	; 10CAh -	Invalid
		dd	invalidPageInsByte	; 10CBh -	Invalid
		dd	invalidPageInsByte	; 10CCh -	Invalid
		dd	invalidPageInsByte	; 10CDh -	Invalid
		dd	inst1CE			; 10CEh - LDS	Immediate
		dd	invalidPageInsByte	; 10CFh -	Invalid
		dd	invalidPageInsByte	; 10D0h -	Invalid
		dd	invalidPageInsByte	; 10D1h -	Invalid
		dd	invalidPageInsByte	; 10D2h -	Invalid
		dd	invalidPageInsByte	; 10D3h -	Invalid
		dd	invalidPageInsByte	; 10D4h -	Invalid
		dd	invalidPageInsByte	; 10D5h -	Invalid
		dd	invalidPageInsByte	; 10D6h -	Invalid
		dd	invalidPageInsByte	; 10D7h -	Invalid
		dd	invalidPageInsByte	; 10D8h -	Invalid
		dd	invalidPageInsByte	; 10D9h -	Invalid
		dd	invalidPageInsByte	; 10DAh -	Invalid
		dd	invalidPageInsByte	; 10DBh -	Invalid
		dd	invalidPageInsByte	; 10DCh -	Invalid
		dd	invalidPageInsByte	; 10DDh -	Invalid
		dd	inst1DE			; 10DEh - LDS	Direct
		dd	inst1DF			; 10DFh - STS	Direct
		dd	invalidPageInsByte	; 10E0h -	Invalid
		dd	invalidPageInsByte	; 10E1h -	Invalid
		dd	invalidPageInsByte	; 10E2h -	Invalid
		dd	invalidPageInsByte	; 10E3h -	Invalid
		dd	invalidPageInsByte	; 10E4h -	Invalid
		dd	invalidPageInsByte	; 10E5h -	Invalid
		dd	invalidPageInsByte	; 10E6h -	Invalid
		dd	invalidPageInsByte	; 10E7h -	Invalid
		dd	invalidPageInsByte	; 10E8h -	Invalid
		dd	invalidPageInsByte	; 10E9h -	Invalid
		dd	invalidPageInsByte	; 10EAh -	Invalid
		dd	invalidPageInsByte	; 10EBh -	Invalid
		dd	invalidPageInsByte	; 10ECh -	Invalid
		dd	invalidPageInsByte	; 10EDh -	Invalid
		dd	inst1EE			; 10EEh - LDS	Indexed
		dd	inst1EF			; 10EFh - STS	Indexed
		dd	invalidPageInsByte	; 10F0h -	Invalid
		dd	invalidPageInsByte	; 10F1h -	Invalid
		dd	invalidPageInsByte	; 10F2h -	Invalid
		dd	invalidPageInsByte	; 10F3h -	Invalid
		dd	invalidPageInsByte	; 10F4h -	Invalid
		dd	invalidPageInsByte	; 10F5h -	Invalid
		dd	invalidPageInsByte	; 10F6h -	Invalid
		dd	invalidPageInsByte	; 10F7h -	Invalid
		dd	invalidPageInsByte	; 10F8h -	Invalid
		dd	invalidPageInsByte	; 10F9h -	Invalid
		dd	invalidPageInsByte	; 10FAh -	Invalid
		dd	invalidPageInsByte	; 10FBh -	Invalid
		dd	invalidPageInsByte	; 10FCh -	Invalid
		dd	invalidPageInsByte	; 10FDh -	Invalid
		dd	inst1FE			; 10FEh - LDS	Extended
		dd	inst1FF			; 10FFh - STS	Extended

Alignment

; This is the jump table for the page two instructions

m6809two	dd	invalidPageInsByte	; 1100h -	Invalid
		dd	invalidPageInsByte	; 1101h -	Invalid
		dd	invalidPageInsByte	; 1102h -	Invalid
		dd	invalidPageInsByte	; 1103h -	Invalid
		dd	invalidPageInsByte	; 1104h -	Invalid
		dd	invalidPageInsByte	; 1105h -	Invalid
		dd	invalidPageInsByte	; 1106h -	Invalid
		dd	invalidPageInsByte	; 1107h -	Invalid
		dd	invalidPageInsByte	; 1108h -	Invalid
		dd	invalidPageInsByte	; 1109h -	Invalid
		dd	invalidPageInsByte	; 110Ah -	Invalid
		dd	invalidPageInsByte	; 110Bh -	Invalid
		dd	invalidPageInsByte	; 110Ch -	Invalid
		dd	invalidPageInsByte	; 110Dh -	Invalid
		dd	invalidPageInsByte	; 110Eh -	Invalid
		dd	invalidPageInsByte	; 110Fh -	Invalid
		dd	invalidPageInsByte	; 1110h -	Invalid
		dd	invalidPageInsByte	; 1111h -	Invalid
		dd	invalidPageInsByte	; 1112h -	Invalid
		dd	invalidPageInsByte	; 1113h -	Invalid
		dd	invalidPageInsByte	; 1114h -	Invalid
		dd	invalidPageInsByte	; 1115h -	Invalid
		dd	invalidPageInsByte	; 1116h -	Invalid
		dd	invalidPageInsByte	; 1117h -	Invalid
		dd	invalidPageInsByte	; 1118h -	Invalid
		dd	invalidPageInsByte	; 1119h -	Invalid
		dd	invalidPageInsByte	; 111Ah -	Invalid
		dd	invalidPageInsByte	; 111Bh -	Invalid
		dd	invalidPageInsByte	; 111Ch -	Invalid
		dd	invalidPageInsByte	; 111Dh -	Invalid
		dd	invalidPageInsByte	; 111Eh -	Invalid
		dd	invalidPageInsByte	; 111Fh -	Invalid
		dd	invalidPageInsByte	; 1120h -	Invalid
		dd	invalidPageInsByte	; 1121h -	Invalid
		dd	invalidPageInsByte	; 1122h -	Invalid
		dd	invalidPageInsByte	; 1123h -	Invalid
		dd	invalidPageInsByte	; 1124h -	Invalid
		dd	invalidPageInsByte	; 1125h -	Invalid
		dd	invalidPageInsByte	; 1126h -	Invalid
		dd	invalidPageInsByte	; 1127h -	Invalid
		dd	invalidPageInsByte	; 1128h -	Invalid
		dd	invalidPageInsByte	; 1129h -	Invalid
		dd	invalidPageInsByte	; 112Ah -	Invalid
		dd	invalidPageInsByte	; 112Bh -	Invalid
		dd	invalidPageInsByte	; 112Ch -	Invalid
		dd	invalidPageInsByte	; 112Dh -	Invalid
		dd	invalidPageInsByte	; 112Eh -	Invalid
		dd	invalidPageInsByte	; 112Fh -	Invalid
		dd	invalidPageInsByte	; 1130h -	Invalid
		dd	invalidPageInsByte	; 1131h -	Invalid
		dd	invalidPageInsByte	; 1132h -	Invalid
		dd	invalidPageInsByte	; 1133h -	Invalid
		dd	invalidPageInsByte	; 1134h -	Invalid
		dd	invalidPageInsByte	; 1135h -	Invalid
		dd	invalidPageInsByte	; 1136h -	Invalid
		dd	invalidPageInsByte	; 1137h -	Invalid
		dd	invalidPageInsByte	; 1138h -	Invalid
		dd	invalidPageInsByte	; 1139h -	Invalid
		dd	invalidPageInsByte	; 113Ah -	Invalid
		dd	invalidPageInsByte	; 113Bh -	Invalid
		dd	invalidPageInsByte	; 113Ch -	Invalid
		dd	invalidPageInsByte	; 113Dh -	Invalid
		dd	invalidPageInsByte	; 113Eh -	Invalid
		dd	inst23F			; 113Fh - SWI3	Inherent
		dd	invalidPageInsByte	; 1140h -	Invalid
		dd	invalidPageInsByte	; 1141h -	Invalid
		dd	invalidPageInsByte	; 1142h -	Invalid
		dd	invalidPageInsByte	; 1143h -	Invalid
		dd	invalidPageInsByte	; 1144h -	Invalid
		dd	invalidPageInsByte	; 1145h -	Invalid
		dd	invalidPageInsByte	; 1146h -	Invalid
		dd	invalidPageInsByte	; 1147h -	Invalid
		dd	invalidPageInsByte	; 1148h -	Invalid
		dd	invalidPageInsByte	; 1149h -	Invalid
		dd	invalidPageInsByte	; 114Ah -	Invalid
		dd	invalidPageInsByte	; 114Bh -	Invalid
		dd	invalidPageInsByte	; 114Ch -	Invalid
		dd	invalidPageInsByte	; 114Dh -	Invalid
		dd	invalidPageInsByte	; 114Eh -	Invalid
		dd	invalidPageInsByte	; 114Fh -	Invalid
		dd	invalidPageInsByte	; 1150h -	Invalid
		dd	invalidPageInsByte	; 1151h -	Invalid
		dd	invalidPageInsByte	; 1152h -	Invalid
		dd	invalidPageInsByte	; 1153h -	Invalid
		dd	invalidPageInsByte	; 1154h -	Invalid
		dd	invalidPageInsByte	; 1155h -	Invalid
		dd	invalidPageInsByte	; 1156h -	Invalid
		dd	invalidPageInsByte	; 1157h -	Invalid
		dd	invalidPageInsByte	; 1158h -	Invalid
		dd	invalidPageInsByte	; 1159h -	Invalid
		dd	invalidPageInsByte	; 115Ah -	Invalid
		dd	invalidPageInsByte	; 115Bh -	Invalid
		dd	invalidPageInsByte	; 115Ch -	Invalid
		dd	invalidPageInsByte	; 115Dh -	Invalid
		dd	invalidPageInsByte	; 115Eh -	Invalid
		dd	invalidPageInsByte	; 115Fh -	Invalid
		dd	invalidPageInsByte	; 1160h -	Invalid
		dd	invalidPageInsByte	; 1161h -	Invalid
		dd	invalidPageInsByte	; 1162h -	Invalid
		dd	invalidPageInsByte	; 1163h -	Invalid
		dd	invalidPageInsByte	; 1164h -	Invalid
		dd	invalidPageInsByte	; 1165h -	Invalid
		dd	invalidPageInsByte	; 1166h -	Invalid
		dd	invalidPageInsByte	; 1167h -	Invalid
		dd	invalidPageInsByte	; 1168h -	Invalid
		dd	invalidPageInsByte	; 1169h -	Invalid
		dd	invalidPageInsByte	; 116Ah -	Invalid
		dd	invalidPageInsByte	; 116Bh -	Invalid
		dd	invalidPageInsByte	; 116Ch -	Invalid
		dd	invalidPageInsByte	; 116Dh -	Invalid
		dd	invalidPageInsByte	; 116Eh -	Invalid
		dd	invalidPageInsByte	; 116Fh -	Invalid
		dd	invalidPageInsByte	; 1170h -	Invalid
		dd	invalidPageInsByte	; 1171h -	Invalid
		dd	invalidPageInsByte	; 1172h -	Invalid
		dd	invalidPageInsByte	; 1173h -	Invalid
		dd	invalidPageInsByte	; 1174h -	Invalid
		dd	invalidPageInsByte	; 1175h -	Invalid
		dd	invalidPageInsByte	; 1176h -	Invalid
		dd	invalidPageInsByte	; 1177h -	Invalid
		dd	invalidPageInsByte	; 1178h -	Invalid
		dd	invalidPageInsByte	; 1179h -	Invalid
		dd	invalidPageInsByte	; 117Ah -	Invalid
		dd	invalidPageInsByte	; 117Bh -	Invalid
		dd	invalidPageInsByte	; 117Ch -	Invalid
		dd	invalidPageInsByte	; 117Dh -	Invalid
		dd	invalidPageInsByte	; 117Eh -	Invalid
		dd	invalidPageInsByte	; 117Fh -	Invalid
		dd	invalidPageInsByte	; 1180h -	Invalid
		dd	invalidPageInsByte	; 1181h -	Invalid
		dd	invalidPageInsByte	; 1182h -	Invalid
 		dd	inst283			; 1183h - CMPU	Immediate
		dd	invalidPageInsByte	; 1184h -	Invalid
		dd	invalidPageInsByte	; 1185h -	Invalid
		dd	invalidPageInsByte	; 1186h -	Invalid
		dd	invalidPageInsByte	; 1187h -	Invalid
		dd	invalidPageInsByte	; 1188h -	Invalid
		dd	invalidPageInsByte	; 1189h -	Invalid
		dd	invalidPageInsByte	; 118Ah -	Invalid
		dd	invalidPageInsByte	; 118Bh -	Invalid
		dd	inst28C			; 118Ch - CMPS	Immediate
		dd	invalidPageInsByte	; 118Dh -	Invalid
		dd	invalidPageInsByte	; 118Eh -	Invalid
		dd	invalidPageInsByte	; 118Fh -	Invalid
		dd	invalidPageInsByte	; 1190h -	Invalid
		dd	invalidPageInsByte	; 1191h -	Invalid
		dd	invalidPageInsByte	; 1192h -	Invalid
		dd	inst293			; 1193h - CMPU	Direct
		dd	invalidPageInsByte	; 1194h -	Invalid
		dd	invalidPageInsByte	; 1195h -	Invalid
		dd	invalidPageInsByte	; 1196h -	Invalid
		dd	invalidPageInsByte	; 1197h -	Invalid
		dd	invalidPageInsByte	; 1198h -	Invalid
		dd	invalidPageInsByte	; 1199h -	Invalid
		dd	invalidPageInsByte	; 119Ah -	Invalid
		dd	invalidPageInsByte	; 119Bh -	Invalid
		dd	inst29C			; 119Ch - CMPS	Direct
		dd	invalidPageInsByte	; 119Dh -	Invalid
		dd	invalidPageInsByte	; 119Eh -	Invalid
		dd	invalidPageInsByte	; 119Fh -	Invalid
		dd	invalidPageInsByte	; 11A0h -	Invalid
		dd	invalidPageInsByte	; 11A1h -	Invalid
		dd	invalidPageInsByte	; 11A2h -	Invalid
		dd	inst2A3			; 11A3h - CMPU	Indexed
		dd	invalidPageInsByte	; 11A4h -	Invalid
		dd	invalidPageInsByte	; 11A5h -	Invalid
		dd	invalidPageInsByte	; 11A6h -	Invalid
		dd	invalidPageInsByte	; 11A7h -	Invalid
		dd	invalidPageInsByte	; 11A8h -	Invalid
		dd	invalidPageInsByte	; 11A9h -	Invalid
		dd	invalidPageInsByte	; 11AAh -	Invalid
		dd	invalidPageInsByte	; 11ABh -	Invalid
		dd	inst2AC			; 11ACh - CMPS	Indexed
		dd	invalidPageInsByte	; 11ADh -	Invalid
		dd	invalidPageInsByte	; 11AEh -	Invalid
		dd	invalidPageInsByte	; 11AFh -	Invalid
		dd	invalidPageInsByte	; 11B0h -	Invalid
		dd	invalidPageInsByte	; 11B1h -	Invalid
		dd	invalidPageInsByte	; 11B2h -	Invalid
		dd	inst2B3			; 11B3h - CMPU	Extended
		dd	invalidPageInsByte	; 11B4h -	Invalid
		dd	invalidPageInsByte	; 11B5h -	Invalid
		dd	invalidPageInsByte	; 11B6h -	Invalid
		dd	invalidPageInsByte	; 11B7h -	Invalid
		dd	invalidPageInsByte	; 11B8h -	Invalid
		dd	invalidPageInsByte	; 11B9h -	Invalid
		dd	invalidPageInsByte	; 11BAh -	Invalid
		dd	invalidPageInsByte	; 11BBh -	Invalid
		dd	inst2BC			; 11BCh - CMPS	Extended
		dd	invalidPageInsByte	; 11BDh -	Invalid
		dd	invalidPageInsByte	; 11BEh -	Invalid
		dd	invalidPageInsByte	; 11BFh -	Invalid
		dd	invalidPageInsByte	; 11C0h -	Invalid
		dd	invalidPageInsByte	; 11C1h -	Invalid
		dd	invalidPageInsByte	; 11C2h -	Invalid
		dd	invalidPageInsByte	; 11C3h -	Invalid
		dd	invalidPageInsByte	; 11C4h -	Invalid
		dd	invalidPageInsByte	; 11C5h -	Invalid
		dd	invalidPageInsByte	; 11C6h -	Invalid
		dd	invalidPageInsByte	; 11C7h -	Invalid
		dd	invalidPageInsByte	; 11C8h -	Invalid
		dd	invalidPageInsByte	; 11C9h -	Invalid
		dd	invalidPageInsByte	; 11CAh -	Invalid
		dd	invalidPageInsByte	; 11CBh -	Invalid
		dd	invalidPageInsByte	; 11CCh -	Invalid
		dd	invalidPageInsByte	; 11CDh -	Invalid
		dd	invalidPageInsByte	; 11CEh -	Invalid
		dd	invalidPageInsByte	; 11CFh -	Invalid
		dd	invalidPageInsByte	; 11D0h -	Invalid
		dd	invalidPageInsByte	; 11D1h -	Invalid
		dd	invalidPageInsByte	; 11D2h -	Invalid
		dd	invalidPageInsByte	; 11D3h -	Invalid
		dd	invalidPageInsByte	; 11D4h -	Invalid
		dd	invalidPageInsByte	; 11D5h -	Invalid
		dd	invalidPageInsByte	; 11D6h -	Invalid
		dd	invalidPageInsByte	; 11D7h -	Invalid
		dd	invalidPageInsByte	; 11D8h -	Invalid
		dd	invalidPageInsByte	; 11D9h -	Invalid
		dd	invalidPageInsByte	; 11DAh -	Invalid
		dd	invalidPageInsByte	; 11DBh -	Invalid
		dd	invalidPageInsByte	; 11DCh -	Invalid
		dd	invalidPageInsByte	; 11DDh -	Invalid
		dd	invalidPageInsByte	; 11DEh -	Invalid
		dd	invalidPageInsByte	; 11DFh -	Invalid
		dd	invalidPageInsByte	; 11E0h -	Invalid
		dd	invalidPageInsByte	; 11E1h -	Invalid
		dd	invalidPageInsByte	; 11E2h -	Invalid
		dd	invalidPageInsByte	; 11E3h -	Invalid
		dd	invalidPageInsByte	; 11E4h -	Invalid
		dd	invalidPageInsByte	; 11E5h -	Invalid
		dd	invalidPageInsByte	; 11E6h -	Invalid
		dd	invalidPageInsByte	; 11E7h -	Invalid
		dd	invalidPageInsByte	; 11E8h -	Invalid
		dd	invalidPageInsByte	; 11E9h -	Invalid
		dd	invalidPageInsByte	; 11EAh -	Invalid
		dd	invalidPageInsByte	; 11EBh -	Invalid
		dd	invalidPageInsByte	; 11ECh -	Invalid
		dd	invalidPageInsByte	; 11EDh -	Invalid
		dd	invalidPageInsByte	; 11EEh -	Invalid
		dd	invalidPageInsByte	; 11EFh -	Invalid
		dd	invalidPageInsByte	; 11F0h -	Invalid
		dd	invalidPageInsByte	; 11F1h -	Invalid
		dd	invalidPageInsByte	; 11F2h -	Invalid
		dd	invalidPageInsByte	; 11F3h -	Invalid
		dd	invalidPageInsByte	; 11F4h -	Invalid
		dd	invalidPageInsByte	; 11F5h -	Invalid
		dd	invalidPageInsByte	; 11F6h -	Invalid
		dd	invalidPageInsByte	; 11F7h -	Invalid
		dd	invalidPageInsByte	; 11F8h -	Invalid
		dd	invalidPageInsByte	; 11F9h -	Invalid
		dd	invalidPageInsByte	; 11FAh -	Invalid
		dd	invalidPageInsByte	; 11FBh -	Invalid
		dd	invalidPageInsByte	; 11FCh -	Invalid
		dd	invalidPageInsByte	; 11FDh -	Invalid
		dd	invalidPageInsByte	; 11FEh -	Invalid
		dd	invalidPageInsByte	; 11FFh -	Invalid

; This is the flag table for all 255 bytes

flagTable	db	04h, 00h, 00h, 00h, 00h, 00h, 00h, 00h
		db	00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h
		db	00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h
		db	00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h

		db	00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h
		db	00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h
		db	00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h
		db	00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h

		db	00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h
		db	00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h
		db	00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h
		db	00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h

		db	00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h
		db	00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h
		db	00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h
		db	00h, 00h, 00h, 00h, 00h, 00h, 00h, 00h

		db	08h, 08h, 08h, 08h, 08h, 08h, 08h, 08h
		db	08h, 08h, 08h, 08h, 08h, 08h, 08h, 08h
		db	08h, 08h, 08h, 08h, 08h, 08h, 08h, 08h
		db	08h, 08h, 08h, 08h, 08h, 08h, 08h, 08h

		db	08h, 08h, 08h, 08h, 08h, 08h, 08h, 08h
		db	08h, 08h, 08h, 08h, 08h, 08h, 08h, 08h
		db	08h, 08h, 08h, 08h, 08h, 08h, 08h, 08h
		db	08h, 08h, 08h, 08h, 08h, 08h, 08h, 08h

		db	08h, 08h, 08h, 08h, 08h, 08h, 08h, 08h
		db	08h, 08h, 08h, 08h, 08h, 08h, 08h, 08h
		db	08h, 08h, 08h, 08h, 08h, 08h, 08h, 08h
		db	08h, 08h, 08h, 08h, 08h, 08h, 08h, 08h

		db	08h, 08h, 08h, 08h, 08h, 08h, 08h, 08h
		db	08h, 08h, 08h, 08h, 08h, 08h, 08h, 08h
		db	08h, 08h, 08h, 08h, 08h, 08h, 08h, 08h
		db	08h, 08h, 08h, 08h, 08h, 08h, 08h, 08h

; Set up a 16 entry table for the register size

Alignment

regSizeTable	db	2			; D
		db	2			; X
		db	2			; Y
		db	2			; U
		db	2			; S
		db	2			; PC
		db	0			; None
		db	0			; None
		db	1			; A
		db	1			; B
		db	1			; CC
		db	1			; DP
		db	0			; None
		db	0			; None
		db	0			; None
		db	0			; None

; Set up a 16 entry jump table for the m6809GetReg functions

Alignment

getRegJumpTable	dd	getReg_D
		dd	getReg_X
		dd	getReg_Y
		dd	getReg_U
		dd	getReg_S
		dd	getReg_PC
		dd	noGetReg
		dd	noGetReg
		dd	getReg_A
		dd	getReg_B
		dd	getReg_CC
		dd	getReg_DP
		dd	noGetReg
		dd	noGetReg
		dd	noGetReg
		dd	noGetReg
		
; Set up a 16 entry jump table for the m6809SetReg functions

Alignment

setRegJumpTable	dd	setReg_D
		dd	setReg_X
		dd	setReg_Y
		dd	setReg_U
		dd	setReg_S
		dd	setReg_PC
		dd	noSetReg
		dd	noSetReg
		dd	setReg_A
		dd	setReg_B
		dd	setReg_CC
		dd	setReg_DP
		dd	noSetReg
		dd	noSetReg
		dd	noSetReg
		dd	noSetReg

_DATA		ends

; Support routines go here

;************************************************************************
;*									*
;* Execution points for instructions					*
;*									*
;************************************************************************

ifdef MASM
_TEXT		segment		public 'CODE'
endif

		assume		CS:_text

;************************************************************************
;*									*
;* Subroutines								*
;*									*
;************************************************************************

; The postbyte is in AL.  The return value is in AX.  This funtion is
; fairly hideous, looking, but it should be fast...

Alignment

m6809GetIndexWord	proc

		xor	ah, ah		; Clear out AH just in case
		jmp	dword ptr getIndexWordJumpTable[eax*4]	; Go to it...

Alignment

IllegalInd:
		mov	ax, 0
		ret

					; Address = X + [0..15]
Alignment
getIndex00:
getIndex01:
getIndex02:
getIndex03:
getIndex04:
getIndex05:
getIndex06:
getIndex07:
getIndex08:
getIndex09:
getIndex0A:
getIndex0B:
getIndex0C:
getIndex0D:
getIndex0E:
getIndex0F:
		add	ax, cx		; Add X into the postbyte
		ret

					; Address = X - [16..1]
Alignment
getIndex10:
getIndex11:
getIndex12:
getIndex13:
getIndex14:
getIndex15:
getIndex16:
getIndex17:
getIndex18:
getIndex19:
getIndex1A:
getIndex1B:
getIndex1C:
getIndex1D:
getIndex1E:
getIndex1F:
		sub	ax, 020h	; Subtract 0x20 from the postbyte
		add	ax, cx		; Add X into the new postbyte
		ret

					; Address = Y + [0..15]
Alignment
getIndex20:
getIndex21:
getIndex22:
getIndex23:
getIndex24:
getIndex25:
getIndex26:
getIndex27:
getIndex28:
getIndex29:
getIndex2A:
getIndex2B:
getIndex2C:
getIndex2D:
getIndex2E:
getIndex2F:
		sub	ax, 020h	; Subtract 0x20 from AL
		add	ax, [_m6809y]	; Add Y into the postbyte
		ret

					; Address = Y - [16..1]
Alignment
getIndex30:
getIndex31:
getIndex32:
getIndex33:
getIndex34:
getIndex35:
getIndex36:
getIndex37:
getIndex38:
getIndex39:
getIndex3A:
getIndex3B:
getIndex3C:
getIndex3D:
getIndex3E:
getIndex3F:
		sub	ax, 040h	; Subtract 0x40 from the postbyte
		add	ax, [_m6809y]	; Add X into the new postbyte

		ret

					; Address = U + [0..15]
Alignment
getIndex40:
getIndex41:
getIndex42:
getIndex43:
getIndex44:
getIndex45:
getIndex46:
getIndex47:
getIndex48:
getIndex49:
getIndex4A:
getIndex4B:
getIndex4C:
getIndex4D:
getIndex4E:
getIndex4F:
		sub	ax, 040h	; Subtract 0x40 from AL
		add	ax, [_m6809u]	; Add Y into the postbyte
		ret

					; Address = U - [16..1]
Alignment
getIndex50:
getIndex51:
getIndex52:
getIndex53:
getIndex54:
getIndex55:
getIndex56:
getIndex57:
getIndex58:
getIndex59:
getIndex5A:
getIndex5B:
getIndex5C:
getIndex5D:
getIndex5E:
getIndex5F:
		sub	ax, 060h	; Subtract 0x60 from the postbyte
		add	ax, [_m6809u]	; Add X into the new postbyte

		ret

					; Address = S + [0..15]
Alignment
getIndex60:
getIndex61:
getIndex62:
getIndex63:
getIndex64:
getIndex65:
getIndex66:
getIndex67:
getIndex68:
getIndex69:
getIndex6A:
getIndex6B:
getIndex6C:
getIndex6D:
getIndex6E:
getIndex6F:
		sub	ax, 060h	; Subtract 0x60 from AL
		add	ax, [_m6809s]	; Add Y into the postbyte

		ret

					; Address = S - [16..1]
Alignment
getIndex70:
getIndex71:
getIndex72:
getIndex73:
getIndex74:
getIndex75:
getIndex76:
getIndex77:
getIndex78:
getIndex79:
getIndex7A:
getIndex7B:
getIndex7C:
getIndex7D:
getIndex7E:
getIndex7F:
		sub	ax, 080h	; Subtract 0x80 from the postbyte
		add	ax, [_m6809s]	; Add X into the new postbyte

		ret

					; Address = X; X++
Alignment
getIndex80:
		mov	ax, cx		; Copy X into AX
		inc	cx		; Increment CX

		ret

					; Address = X; X+=2
Alignment
getIndex81:
		mov	ax, cx		; Copy X into AX
		add	cx, 2		; Increment CX

		ret

					; X--; Address = X
Alignment
getIndex82:
		dec	cx		; Increment CX
		mov	ax, cx		; Copy X into AX

		ret

					; X-=2; Address = X
Alignment
getIndex83:
		sub	cx, 2		; Increment CX
		mov	ax, cx		; Copy X into AX

		ret

					; Address = X
Alignment
getIndex84:
		mov	ax, cx		; Copy X into AX

		ret

					; Address = X + B
Alignment
getIndex85:
		mov	al, bl		; Copy B into AL
		cbw			; Convert AL into AX
		add	ax, cx		; Add X into AX

		ret

					; Address = X + A
Alignment
getIndex86:
		mov	al, bh		; Copy A into AL
		cbw			; Convert AL into AX
		add	ax, cx		; Add X into AX

		ret

					; Address = X + InstructionByte
Alignment
getIndex88:
		FetchInstructionByte	; Get the next instruction byte
		cbw			; Convert AL into AX
		add	ax, cx		; Add X to it...

		ret

					; Address = X + InstructionWord
Alignment
getIndex89:
		FetchInstructionWord	; Get the next instruction word
		add	ax, cx		; Add X to it...

		ret

					; Address = X + D
Alignment
getIndex8B:
		mov	ax, cx		; Copy X to AX
		add	ax, bx		; Add D to AX

		ret

					; Address = PC + InstructionByte
Alignment
getIndex8C:
		FetchInstructionByte	; Get the next instruction byte
		shl	ecx, 16		; Save X

		cbw			; Convert AL into AX
		mov	cx, ax		; Save AX in CX
		mov	eax, esi	; Save ESI in AX
		sub	eax, ebp	; Get PC in AX

		add	ax, cx		; Add CX to the PC...

		shr	ecx, 16		; Restore X

		ret

					; Address = PC + InstructionWord
Alignment
getIndex8D:
		FetchInstructionWord	; Get the next instruction word
		shl	ecx, 16		; Save X

		mov	cx, ax		; Save AX in CX
		mov	eax, esi	; Save ESI in AX
		sub	eax, ebp	; Get PC in AX

		add	ax, cx		; Add CX to the PC...

		shr	ecx, 16		; Restore X

		ret

					; Address = InstructionWord
Alignment
getIndex8F:
		FetchInstructionWord	; Get the next instruction word

		ret

					; Address = X; X++; Addess = ReadMemoryWord(Address)
Alignment
getIndex90:
		mov	ax, cx		; Copy X into AX
		inc	cx		; Increment CX
		ReadMemoryWord		; Read the word at AX

		ret

					; Address = X; X+=2; Addess = ReadMemoryWord(Address)
Alignment
getIndex91:
		mov	ax, cx		; Copy X into AX
		add	cx, 2		; Increment CX
		ReadMemoryWord		; Read the word at AX

		ret

					; X--; Address = X; Addess = ReadMemoryWord(Address)
Alignment
getIndex92:
		dec	cx		; Increment CX
		mov	ax, cx		; Copy X into AX
		ReadMemoryWord		; Read the word at AX

		ret

					; X-=2; Address = X; Addess = ReadMemoryWord(Address)
Alignment
getIndex93:
		sub	cx, 2		; Increment CX
		mov	ax, cx		; Copy X into AX
		ReadMemoryWord		; Read the word at AX

		ret

					; Address = X; Addess = ReadMemoryWord(Address)
Alignment
getIndex94:
		mov	ax, cx		; Copy X into AX
		ReadMemoryWord		; Read the word at AX

		ret

					; Address = X + B; Addess = ReadMemoryWord(Address)
Alignment
getIndex95:
		mov	al, bl		; Copy B into AL
		cbw			; Convert AL into AX
		add	ax, cx		; Add X into AX
		ReadMemoryWord		; Read the word at AX

		ret

					; Address = X + A; Addess = ReadMemoryWord(Address)
Alignment
getIndex96:
		mov	al, bh		; Copy A into AL
		cbw			; Convert AL into AX
		add	ax, cx		; Add X into AX
		ReadMemoryWord		; Read the word at AX

		ret

					; Address = X + InstructionByte; Addess = ReadMemoryWord(Address)
Alignment
getIndex98:
		FetchInstructionByte	; Get the next instruction byte
		cbw			; Convert AL into AX
		add	ax, cx		; Add X to it...
		ReadMemoryWord		; Read the word at AX

		ret

					; Address = X + InstructionWord; Addess = ReadMemoryWord(Address)
Alignment
getIndex99:
		FetchInstructionWord	; Get the next instruction word
		add	ax, cx		; Add X to it...
		ReadMemoryWord		; Read the word at AX

		ret

					; Address = X + D; Addess = ReadMemoryWord(Address)
Alignment
getIndex9B:
		mov	ax, cx		; Copy X into AX
		add	ax, bx		; Add D to AX
		ReadMemoryWord		; Read the word at AX

		ret

					; Address = PC + InstructionByte; Addess = ReadMemoryWord(Address)
Alignment
getIndex9C:
		FetchInstructionByte	; Get the next instruction byte
		shl	ecx, 16		; Save CX

		cbw			; Convert AL into AX
		mov	cx, ax		; Save AX in CX
		mov	eax, esi	; Save ESI in AX
		sub	eax, ebp	; Get PC in AX

		add	ax, cx		; Add CX to the PC...

		shr	ecx, 16		; Restore CX
		ReadMemoryWord		; Read the word at AX

		ret

					; Address = PC + InstructionWord; Addess = ReadMemoryWord(Address)
Alignment
getIndex9D:
		FetchInstructionWord	; Get the next instruction word
		shl	ecx, 16		; Save CX

		mov	cx, ax		; Save AX in CX
		mov	eax, esi	; Save ESI in AX
		sub	eax, ebp	; Get PC in AX

		add	ax, cx		; Add CX to the PC...

		shr	ecx, 16		; Restore CX
		ReadMemoryWord		; Read the word at AX

		ret

					; Address = InstructionWord; Addess = ReadMemoryWord(Address)
Alignment
getIndex9F:
		FetchInstructionWord	; Get the next instruction word
		ReadMemoryWord		; Read the word at AX

		ret

					; Address = Y; Y++
Alignment
getIndexA0:
		mov	ax, [_m6809y]		; Copy Y in AX
		inc	[_m6809y]

		ret

					; Address = Y; Y+=2
Alignment
getIndexA1:
		mov	ax, [_m6809y]	; Get Y in AX
		add	[_m6809y], 2

		ret

					; y--; Address = Y
Alignment
getIndexA2:
		dec	[_m6809y]
		mov	ax, [_m6809y]
		ret

					; Y-=2; Address = Y
Alignment
getIndexA3:
		sub	[_m6809y], 2
		mov	ax, [_m6809y]
		ret

					; Address = X
Alignment
getIndexA4:
		mov	ax, [_m6809y]
		ret

					; Address = Y + B
Alignment
getIndexA5:
		mov	al, bl		; Copy B into AL
		cbw			; Convert AL into AX
		add	ax, [_m6809y]	; Add Y into AX
		ret

					; Address = Y + A
Alignment
getIndexA6:
		mov	al, bh		; Copy A into AL
		cbw			; Convert AL into AX
		add	ax, [_m6809y]	; Add X into AX
		ret

					; Address = Y + InstructionByte
Alignment
getIndexA8:
		FetchInstructionByte	; Get the next instruction byte
		cbw			; Convert AL into AX
		add	ax, [_m6809y]	; Add Y to it...
		ret

					; Address = Y + InstructionWord
Alignment
getIndexA9:
		FetchInstructionWord	; Get the next instruction word
		add	ax, [_m6809y]	; Add Y to it...
		ret

					; Address = Y + D
Alignment
getIndexAB:
		mov	ax, [_m6809y]	; Copy Y to AX
		add	ax, bx		; Add D to AX
		ret

					; Address = PC + InstructionByte
Alignment
getIndexAC:
		FetchInstructionByte	; Get the next instruction byte
		shl	ecx, 16		; Save CX

		cbw			; Convert AL into AX
		mov	cx, ax		; Save AX in CX
		mov	eax, esi	; Save ESI in AX
		sub	eax, ebp	; Get PC in AX

		add	ax, cx		; Add CX to the PC...

		shr	ecx, 16		; Restore CX

		ret

					; Address = PC + InstructionWord
Alignment
getIndexAD:
		FetchInstructionWord	; Get the next instruction word
		shl	ecx, 16		; Save CX

		mov	cx, ax		; Save AX in CX
		mov	eax, esi	; Save ESI in AX
		sub	eax, ebp	; Get PC in AX

		add	ax, cx		; Add CX to the PC...

		shr	ecx, 16		; Restore CX

		ret

					; Address = InstructionWord
Alignment
getIndexAF:
		FetchInstructionWord	; Get the next instruction word

		ret

					; Address = Y; Y++; Addess = ReadMemoryWord(Address)
Alignment
getIndexB0:
		mov	ax, [_m6809y]	; Copy Y into AX
		inc	[_m6809y]	; Increment CX
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = Y; Y+=2; Addess = ReadMemoryWord(Address)
Alignment
getIndexB1:
		mov	ax, [_m6809y]	; Copy Y into AX
		add	[_m6809y], 2	; Increment CX
		ReadMemoryWord		; Read the word at AX
		ret

					; Y--; Address = Y; Addess = ReadMemoryWord(Address)
Alignment
getIndexB2:
		dec	[_m6809y]    	; Increment CX
		mov	ax, [_m6809y]	; Copy X into AX
		ReadMemoryWord		; Read the word at AX
		ret

					; Y-=2; Address = X; Addess = ReadMemoryWord(Address)
Alignment
getIndexB3:
		sub	[_m6809y], 2	; Decrement Y by 2
		mov	ax, [_m6809y]	; Copy Y into AX
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = Y; Addess = ReadMemoryWord(Address)
Alignment
getIndexB4:
		mov	ax, [_m6809y]	; Copy X into AX
		ReadMemoryWord		; Read the word at AX

		ret

					; Address = Y + B; Addess = ReadMemoryWord(Address)
Alignment
getIndexB5:
		mov	al, bl		; Copy B into AL
		cbw			; Convert AL into AX
		add	ax, [_m6809y]	; Add Y into AX
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = Y + A; Addess = ReadMemoryWord(Address)
Alignment
getIndexB6:
		mov	al, bh		; Copy A into AL
		cbw			; Convert AL into AX
		add	ax, [_m6809y]	; Add Y into AX
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = Y + InstructionByte; Addess = ReadMemoryWord(Address)
Alignment
getIndexB8:
		FetchInstructionByte	; Get the next instruction byte
		cbw			; Convert AL into AX
		add	ax, [_m6809y]	; Add X to it...
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = Y + InstructionWord; Addess = ReadMemoryWord(Address)
Alignment
getIndexB9:
		FetchInstructionWord	; Get the next instruction word
		add	ax, [_m6809y]	; Add X to it...
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = Y + D; Addess = ReadMemoryWord(Address)
Alignment
getIndexBB:
		mov	ax, [_m6809y]	; Copy X to AX
		add	ax, bx		; Add D to AX
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = PC + InstructionByte; Addess = ReadMemoryWord(Address)
Alignment
getIndexBC:
		FetchInstructionByte	; Get the next instruction byte
		shl	ecx, 16		; Save CX

		cbw			; Convert AL into AX
		mov	cx, ax		; Save AX in CX
		mov	eax, esi	; Save ESI in AX
		sub	eax, ebp	; Get PC in AX

		add	ax, cx		; Add CX to the PC...

		shr	ecx, 16		; Restore CX
		ReadMemoryWord		; Read the word at AX

		ret

					; Address = PC + InstructionWord; Addess = ReadMemoryWord(Address)
Alignment
getIndexBD:
		FetchInstructionWord	; Get the next instruction word
		shl	ecx, 16		; Save CX

		mov	cx, ax		; Save AX in CX
		mov	eax, esi	; Save ESI in AX
		sub	eax, ebp	; Get PC in AX

		add	ax, cx		; Add CX to the PC...

		shr	ecx, 16		; Restore CX
		ReadMemoryWord		; Read the word at AX

		ret

					; Address = InstructionWord; Addess = ReadMemoryWord(Address)
Alignment
getIndexBF:
		FetchInstructionWord	; Get the next instruction word
		ReadMemoryWord		; Read the word at AX

		ret

					; Address = U; U++
Alignment
getIndexC0:
		mov	ax, [_m6809u]	; Copy U in AX
		inc	[_m6809u]	; Increment CX
		ret

					; Address = U; U+=2
Alignment
getIndexC1:
		mov	ax, [_m6809u]	; Copy U in AX
		add	[_m6809u], 2	; Increment CX
		ret

					; U--; Address = U
Alignment
getIndexC2:
		dec	[_m6809u]	; Decrement CX
		mov	ax, [_m6809u]	; Copy U in AX
		ret

					; U-=2; Address = U
Alignment
getIndexC3:
		sub	[_m6809u], 2	; Increment [_m6809u]
		mov	ax, [_m6809u]		; Copy U in AX
		ret

					; Address = X
Alignment
getIndexC4:
		mov	ax, [_m6809u]	; Copy U into AX
		ret

					; Address = U + B
Alignment
getIndexC5:
		mov	al, bl		; Copy B into AL
		cbw			; Convert AL into AX
		add	ax, [_m6809u]	; Add U into AX
		ret

					; Address = U + A
Alignment
getIndexC6:
		mov	al, bh		; Copy A into AL
		cbw			; Convert AL into AX
		add	ax, [_m6809u]	; Add X into AX
		ret

					; Address = U + InstructionByte
Alignment
getIndexC8:
		FetchInstructionByte	; Get the next instruction byte
		cbw			; Convert AL into AX
		add	ax, [_m6809u]	; Add U to it...
		ret

					; Address = U + InstructionWord
Alignment
getIndexC9:
		FetchInstructionWord	; Get the next instruction word
		add	ax, [_m6809u]	; Add U to it...
		ret

					; Address = U + D
Alignment
getIndexCB:
		mov	ax, [_m6809u]	; Copy U to AX
		add	ax, bx		; Add D to AX
		ret

					; Address = PC + InstructionByte
Alignment
getIndexCC:
		FetchInstructionByte	; Get the next instruction byte
		shl	ecx, 16		; Save CX

		cbw			; Convert AL into AX
		mov	cx, ax		; Save AX in CX
		mov	eax, esi	; Save ESI in AX
		sub	eax, ebp	; Get PC in AX

		add	ax, cx		; Add CX to the PC...

		shr	ecx, 16		; Restore CX

		ret

					; Address = PC + InstructionWord
Alignment
getIndexCD:
		FetchInstructionWord	; Get the next instruction word
		shl	ecx, 16		; Save CX

		mov	cx, ax		; Save AX in CX
		mov	eax, esi	; Save ESI in AX
		sub	eax, ebp	; Get PC in AX

		add	ax, cx		; Add CX to the PC...

		shr	ecx, 16		; Restore CX

		ret

					; Address = InstructionWord
Alignment
getIndexCF:
		FetchInstructionWord	; Get the next instruction word

		ret

					; Address = U; U++; Addess = ReadMemoryWord(Address)
Alignment
getIndexD0:
		mov	ax, [_m6809u]	; Copy U into AX
		inc	[_m6809u]	; Increment CX
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = U; U+=2; Addess = ReadMemoryWord(Address)
Alignment
getIndexD1:
		mov	ax, [_m6809u]	; Copy U into AX
		add	[_m6809u], 2	; Increment CX
		ReadMemoryWord		; Read the word at AX
		ret

					; U-=2; Address = U; Addess = ReadMemoryWord(Address)
Alignment
getIndexD2:
		dec	[_m6809u]	; Increment CX
		mov	ax, [_m6809u]	; Copy U into AX
		ReadMemoryWord		; Read the word at AX
		ret

					; U-=2; Address = U; Addess = ReadMemoryWord(Address)
Alignment
getIndexD3:
		sub	[_m6809u], 2 	; Increment CX
		mov	ax, [_m6809u]	; Copy U into AX
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = U; Addess = ReadMemoryWord(Address)
Alignment
getIndexD4:
		mov	ax, [_m6809u]	; Copy X into AX
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = U + B; Addess = ReadMemoryWord(Address)
Alignment
getIndexD5:
		mov	al, bl		; Copy B into AL
		cbw			; Convert AL into AX
		add	ax, [_m6809u]	; Add U into AX
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = U + A; Addess = ReadMemoryWord(Address)
Alignment
getIndexD6:
		mov	al, bh		; Copy A into AL
		cbw			; Convert AL into AX
		add	ax, [_m6809u]	; Add U into AX
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = U + InstructionByte; Addess = ReadMemoryWord(Address)
Alignment
getIndexD8:
		FetchInstructionByte	; Get the next instruction byte
		cbw			; Convert AL into AX
		add	ax, [_m6809u]	; Add X to it...
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = U + InstructionWord; Addess = ReadMemoryWord(Address)
Alignment
getIndexD9:
		FetchInstructionWord	; Get the next instruction word
		add	ax, [_m6809u]	; Add X to it...
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = U + D; Addess = ReadMemoryWord(Address)
Alignment
getIndexDB:
		mov	ax, [_m6809u]	; Copy X to AX
		add	ax, bx		; Add D to AX
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = PC + InstructionByte; Addess = ReadMemoryWord(Address)
Alignment
getIndexDC:
		FetchInstructionByte	; Get the next instruction byte
		shl	ecx, 16		; Save CX

		cbw			; Convert AL into AX
		mov	cx, ax		; Save AX in CX
		mov	eax, esi	; Save ESI in AX
		sub	eax, ebp	; Get PC in AX

		add	ax, cx		; Add CX to the PC...

		shr	ecx, 16		; Restore CX
		ReadMemoryWord		; Read the word at AX

		ret

					; Address = PC + InstructionWord; Addess = ReadMemoryWord(Address)
Alignment
getIndexDD:
		FetchInstructionWord	; Get the next instruction word
		shl	ecx, 16		; Save CX

		mov	cx, ax		; Save AX in CX
		mov	eax, esi	; Save ESI in AX
		sub	eax, ebp	; Get PC in AX

		add	ax, cx		; Add CX to the PC...

		shr	ecx, 16		; Restore CX
		ReadMemoryWord		; Read the word at AX

		ret

					; Address = InstructionWord; Addess = ReadMemoryWord(Address)
Alignment
getIndexDF:
		FetchInstructionWord	; Get the next instruction word
		ReadMemoryWord		; Read the word at AX

		ret

					; Address = S; S++
Alignment
getIndexE0:
		mov	ax, [_m6809s]		; Copy S in AX
		inc	[_m6809s]		; Increment CX
		ret

					; Address = S; S+=2
Alignment
getIndexE1:
		mov	ax, [_m6809s]		; Copy S in AX
		add	[_m6809s], 2		; Increment CX
		ret

					; S--; Address = S
Alignment
getIndexE2:
		dec	[_m6809s]		; Increment [_m6809s]
		mov	ax, [_m6809s]		; Copy S in AX
		ret

					; S-=2; Address = S
Alignment
getIndexE3:
		sub	[_m6809s], 2		; Increment [_m6809s]
		mov	ax, [_m6809s]		; Copy S in AX
		ret

					; Address = X
Alignment
getIndexE4:
		mov	ax, [_m6809s]		; Copy S into AX
		ret

					; Address = S + B
Alignment
getIndexE5:
		mov	al, bl		; Copy B into AL
		cbw			; Convert AL into AX
		add	ax, [_m6809s]		; Add S into AX
		ret

					; Address = S + A
Alignment
getIndexE6:
		mov	al, bh		; Copy A into AL
		cbw			; Convert AL into AX
		add	ax, [_m6809s]		; Add X into AX

		ret

					; Address = S + InstructionByte
Alignment
getIndexE8:
		FetchInstructionByte	; Get the next instruction byte
		cbw			; Convert AL into AX
		add	ax, [_m6809s]		; Add S to it...
		ret

					; Address = S + InstructionWord
Alignment
getIndexE9:
		FetchInstructionWord	; Get the next instruction word
		add	ax, [_m6809s]	; Add S to it...
		ret

					; Address = S + D
Alignment
getIndexEB:
		mov	ax, [_m6809s]	; Copy S to AX
		add	ax, bx		; Add D to AX
		ret

					; Address = PC + InstructionByte
Alignment
getIndexEC:
		FetchInstructionByte	; Get the next instruction byte
		shl	ecx, 16		; Save CX

		cbw			; Convert AL into AX
		mov	cx, ax		; Save AX in CX
		mov	eax, esi	; Save ESI in AX
		sub	eax, ebp	; Get PC in AX

		add	ax, cx		; Add CX to the PC...

		shr	ecx, 16		; Restore CX

		ret

					; Address = PC + InstructionWord
Alignment
getIndexED:
		FetchInstructionWord	; Get the next instruction word
		shl	ecx, 16		; Save CX

		mov	cx, ax		; Save AX in CX
		mov	eax, esi	; Save ESI in AX
		sub	eax, ebp	; Get PC in AX

		add	ax, cx		; Add CX to the PC...

		shr	ecx, 16		; Restore CX

		ret

					; Address = InstructionWord
Alignment
getIndexEF:
		FetchInstructionWord	; Get the next instruction word

		ret

					; Address = S; S++; Addess = ReadMemoryWord(Address)
Alignment
getIndexF0:
		mov	ax, [_m6809s]		; Copy S into AX
		inc	[_m6809s]		; Increment CX
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = S; S+=2; Addess = ReadMemoryWord(Address)
Alignment
getIndexF1:
		mov	ax, [_m6809s]		; Copy S into AX
		add	[_m6809s], 2		; Increment CX
		ReadMemoryWord		; Read the word at AX
		ret

					; S--; Address = S; Addess = ReadMemoryWord(Address)
Alignment
getIndexF2:
		dec	[_m6809s]		; Increment [_m6809s]
		mov	ax, [_m6809s]		; Copy X into AX
		ReadMemoryWord		; Read the word at AX
		ret

					; S-=2; Address = S; Addess = ReadMemoryWord(Address)
Alignment
getIndexF3:
		sub	[_m6809s], 2		; Increment [_m6809s]
		mov	ax, [_m6809s]		; Copy S into AX
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = S; Addess = ReadMemoryWord(Address)
Alignment
getIndexF4:
		mov	ax, [_m6809s]		; Copy X into AX
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = S + B; Addess = ReadMemoryWord(Address)
Alignment
getIndexF5:
		mov	al, bl		; Copy B into AL
		cbw			; Convert AL into AX
		add	ax, [_m6809s]		; Add S into AX
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = S + A; Addess = ReadMemoryWord(Address)
Alignment
getIndexF6:
		mov	al, bh		; Copy A into AL
		cbw			; Convert AL into AX
		add	ax, [_m6809s]		; Add S into AX
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = S + InstructionByte; Addess = ReadMemoryWord(Address)
Alignment
getIndexF8:
		FetchInstructionByte	; Get the next instruction byte
		cbw			; Convert AL into AX
		add	ax, [_m6809s]	; Add X to it...
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = S + InstructionWord; Addess = ReadMemoryWord(Address)
Alignment
getIndexF9:
		FetchInstructionWord	; Get the next instruction word
		add	ax, [_m6809s]	; Add X to it...
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = S + D; Addess = ReadMemoryWord(Address)
Alignment
getIndexFB:
		mov	ax, [_m6809s]	; Copy X to AX
		add	ax, bx		; Add D to AX
		ReadMemoryWord		; Read the word at AX
		ret

					; Address = PC + InstructionByte; Addess = ReadMemoryWord(Address)
Alignment
getIndexFC:
		FetchInstructionByte	; Get the next instruction byte
		shl	ecx, 16		; Save CX

		cbw			; Convert AL into AX
		mov	cx, ax		; Save AX in CX
		mov	eax, esi	; Save ESI in AX
		sub	eax, ebp	; Get PC in AX

		add	ax, cx		; Add CX to the PC...

		shr	ecx, 16		; Restore CX
		ReadMemoryWord		; Read the word at AX

		ret

					; Address = PC + InstructionWord; Addess = ReadMemoryWord(Address)
Alignment
getIndexFD:
		FetchInstructionWord	; Get the next instruction word
		shl	ecx, 16		; Save CX

		mov	cx, ax		; Save AX in CX
		mov	eax, esi	; Save ESI in AX
		sub	eax, ebp	; Get PC in AX

		add	ax, cx		; Add CX to the PC...

		shr	ecx, 16		; Restore CX
		ReadMemoryWord		; Read the word at AX

		ret

					; Address = InstructionWord; Addess = ReadMemoryWord(Address)
Alignment
getIndexFF:
		FetchInstructionWord	; Get the next instruction word
		ReadMemoryWord		; Read the word at AX

		ret

; Set up a little jump table for m6809GetIndexWord :-)

Alignment

getIndexWordJumpTable	dd	getIndex00, getIndex01, getIndex02, getIndex03
		dd	getIndex04, getIndex05, getIndex06, getIndex07
		dd	getIndex08, getIndex09, getIndex0A, getIndex0B
		dd	getIndex0C, getIndex0D, getIndex0E, getIndex0F
		dd	getIndex10, getIndex11, getIndex12, getIndex13
		dd	getIndex14, getIndex15, getIndex16, getIndex17
		dd	getIndex18, getIndex19, getIndex1A, getIndex1B
		dd	getIndex1C, getIndex1D, getIndex1E, getIndex1F
		dd	getIndex20, getIndex21, getIndex22, getIndex23
		dd	getIndex24, getIndex25, getIndex26, getIndex27
		dd	getIndex28, getIndex29, getIndex2A, getIndex2B
		dd	getIndex2C, getIndex2D, getIndex2E, getIndex2F
		dd	getIndex30, getIndex31, getIndex32, getIndex33
		dd	getIndex34, getIndex35, getIndex36, getIndex37
		dd	getIndex38, getIndex39, getIndex3A, getIndex3B
		dd	getIndex3C, getIndex3D, getIndex3E, getIndex3F
		dd	getIndex40, getIndex41, getIndex42, getIndex43
		dd	getIndex44, getIndex45, getIndex46, getIndex47
		dd	getIndex48, getIndex49, getIndex4A, getIndex4B
		dd	getIndex4C, getIndex4D, getIndex4E, getIndex4F
		dd	getIndex50, getIndex51, getIndex52, getIndex53
		dd	getIndex54, getIndex55, getIndex56, getIndex57
		dd	getIndex58, getIndex59, getIndex5A, getIndex5B
		dd	getIndex5C, getIndex5D, getIndex5E, getIndex5F
		dd	getIndex60, getIndex61, getIndex62, getIndex63
		dd	getIndex64, getIndex65, getIndex66, getIndex67
		dd	getIndex68, getIndex69, getIndex6A, getIndex6B
		dd	getIndex6C, getIndex6D, getIndex6E, getIndex6F
		dd	getIndex70, getIndex71, getIndex72, getIndex73
		dd	getIndex74, getIndex75, getIndex76, getIndex77
		dd	getIndex78, getIndex79, getIndex7A, getIndex7B
		dd	getIndex7C, getIndex7D, getIndex7E, getIndex7F
		dd	getIndex80, getIndex81, getIndex82, getIndex83
		dd	getIndex84, getIndex85, getIndex86, IllegalInd
		dd	getIndex88, getIndex89, IllegalInd, getIndex8B
		dd	getIndex8C, getIndex8D, IllegalInd, getIndex8F
		dd	getIndex90, getIndex91, getIndex92, getIndex93
		dd	getIndex94, getIndex95, getIndex96, IllegalInd
		dd	getIndex98, getIndex99, IllegalInd, getIndex9B
		dd	getIndex9C, getIndex9D, IllegalInd, getIndex9F
		dd	getIndexA0, getIndexA1, getIndexA2, getIndexA3
		dd	getIndexA4, getIndexA5, getIndexA6, IllegalInd
		dd	getIndexA8, getIndexA9, IllegalInd, getIndexAB
		dd	getIndexAC, getIndexAD, IllegalInd, getIndexAF
		dd	getIndexB0, getIndexB1, getIndexB2, getIndexB3
		dd	getIndexB4, getIndexB5, getIndexB6, IllegalInd
		dd	getIndexB8, getIndexB9, IllegalInd, getIndexBB
		dd	getIndexBC, getIndexBD, IllegalInd, getIndexBF
		dd	getIndexC0, getIndexC1, getIndexC2, getIndexC3
		dd	getIndexC4, getIndexC5, getIndexC6, IllegalInd
		dd	getIndexC8, getIndexC9, IllegalInd, getIndexCB
		dd	getIndexCC, getIndexCD, IllegalInd, getIndexCF
		dd	getIndexD0, getIndexD1, getIndexD2, getIndexD3
		dd	getIndexD4, getIndexD5, getIndexD6, IllegalInd
		dd	getIndexD8, getIndexD9, IllegalInd, getIndexDB
		dd	getIndexDC, getIndexDD, IllegalInd, getIndexDF
		dd	getIndexE0, getIndexE1, getIndexE2, getIndexE3
		dd	getIndexE4, getIndexE5, getIndexE6, IllegalInd
		dd	getIndexE8, getIndexE9, IllegalInd, getIndexEB
		dd	getIndexEC, getIndexED, IllegalInd, getIndexEF
		dd	getIndexF0, getIndexF1, getIndexF2, getIndexF3
		dd	getIndexF4, getIndexF5, getIndexF6, IllegalInd
		dd	getIndexF8, getIndexF9, IllegalInd, getIndexFB
		dd	getIndexFC, getIndexFD, IllegalInd, getIndexFF

m6809GetIndexWord	endp		
		

Alignment

; The register to fetch is in AL.  The value of the register is returned
; in AX.

m6809GetReg	proc

		xor	ah, ah			; Clear out AH just in case
		jmp	dword ptr getRegJumpTable[eax*4]	; Go to it...
Alignment

getReg_D	proc

ifdef MASM
		mov	ah, _m6809a		; Put A into AH
		mov	al, _m6809b		; Put B into AL
endif

		ret				; We're done
getReg_D	endp

Alignment

getReg_X	proc
ifdef MASM
		mov	ax, _m6809x		; Put X into AX
endif
		ret				; We're done
getReg_X	endp

Alignment

getReg_Y	proc
ifdef MASM
		mov	ax, _m6809y		; Put Y into AX
endif
		ret				; We're done
getReg_Y	endp

Alignment

getReg_U	proc

ifdef MASM
		mov	ax, _m6809u		; Put U into AX
endif
		ret				; We're done
getReg_U	endp

Alignment

getReg_S	proc

ifdef MASM
		mov	ax, _m6809s		; Put S into AX
endif
		ret				; We're done
getReg_S	endp

Alignment

getReg_PC	proc

ifdef MASM
		mov	ax, _m6809pc		; Put PC into AX
endif
		ret				; We're done
getReg_PC	endp

Alignment

getReg_A	proc

ifdef MASM
		mov	al, _m6809a		; Put A in AL
endif
		ret				; We're done
getReg_A	endp

Alignment

getReg_B	proc

ifdef MASM
		mov	al, _m6809b		; Put X in AL
endif
		ret				; We're done
getReg_B	endp

Alignment

getReg_CC	proc

ifdef MASM
		mov	al, _m6809cc		; Put CC in AL
endif
		ret				; We're done
getReg_CC	endp

Alignment

getReg_DP	proc

ifdef MASM
		mov	al, _m6809dp		; Put DP in AL
endif
		ret				; We're done
getReg_DP	endp

Alignment

noGetReg	proc
		mov	ax, 0ffffh		; Undocumented stuff...
noGetReg	endp

m6809GetReg	endp

Alignment

; The register to set is in AL.	 The value to set the register with is
; in BX.

m6809SetReg	proc

		xor	ah, ah		; Clear out AH just in case
		jmp	dword ptr setRegJumpTable[eax*4]	; Go to it...
Alignment

setReg_D	proc
ifdef MASM
		mov	_m6809a, bh	; Put BH in A
		mov	_m6809b, bl	; Put BL in B
endif
		ret			; We're done
setReg_D	endp

Alignment

setReg_X	proc
ifdef MASM
		mov	_m6809x, bx	; Put BX in X
endif
		ret			; We're done
setReg_X	endp

Alignment

setReg_Y	proc

ifdef MASM
		mov	_m6809y, bx	; Put BX in Y
endif
		ret			; We're done
setReg_Y	endp

Alignment

setReg_U	proc

ifdef MASM
		mov	_m6809u, bx	; Put BX in U
endif
		ret			; We're done
setReg_U	endp

Alignment

setReg_S	proc

ifdef MASM
		mov	_m6809s, bx	; Put BX in S
endif
		ret			; We're done
setReg_S	endp

Alignment

setReg_PC	proc
ifdef MASM
		mov	_m6809pc, bx	; Put BX in PC
endif
		ret			; We're done
setReg_PC	endp

Alignment

setReg_A	proc
ifdef MASM
		mov	_m6809a, bl	; Put BL in A
endif
		ret			; We're done
setReg_A	endp

Alignment

setReg_B	proc

ifdef MASM
		mov	_m6809b, bl	; Put BL in X
endif
		ret			; We're done
setReg_B	endp

Alignment

setReg_CC	proc

ifdef MASM
		mov	_m6809cc, bl	; Put BL in CC
endif
		ret			; We're done
setReg_CC	endp

Alignment

setReg_DP	proc

ifdef MASM
		mov	_m6809dp, bl	; Put BL in DP
endif
		ret			; We're done
setReg_DP	endp

Alignment

noSetReg	proc
		nop			; Do nothing
noSetReg	endp

m6809SetReg	endp

;************************************************************************
;*									*
;* Main routines							*
;*									*
;************************************************************************

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; NEG		00	Direct		6	2	uaaaa

Alignment

inst00		proc	private

		Clocktime	6

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchDirectWord			; Get our address for the data fetch
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		cmp	al, 80h			; Are we gonna overflow?
		jne	noOverflow00		; No, skip down...
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflow00:
		neg	al			; Negate our data

		adc	dh, 0			; If it carried, set the bit
		FlagSet				; Set the remaining flags

		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		ExecuteNextInstruction 6	; Return to the main loop

inst00		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; COM		03	Direct		6	2	-aa01

Alignment

inst03		proc	private

		Clocktime	6

		FetchDirectWord			; Get our address for the data fetch
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		xor	al, 0ffh		; Take the 1's complement

		or	dh, CARRY		; Set the carry bit
		and	dh, 0ffh - OVERFLOW	; There's no overflow
		FlagSet				; Set the remaining flags.

		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		ExecuteNextInstruction 6	; Return to the main loop

inst03		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LSR		04	Direct		6	2	-0a-s

Alignment

inst04		proc	private

		Clocktime 6

		FetchDirectWord			; Get our address for the data fetch
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		and	dh, 0ffh - (NEGATIVE or CARRY)	; There's no negative
		shr	al, 1			; Shift to the right 1 bit

		adc	dh, 0			; If it carried, set the bit
		FlagSet				; Set the remaining flags.

		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX
		xor	eax, eax		; Zero EAX

		ExecuteNextInstruction 6	; Return to the main loop

inst04		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ROR		06	Direct		6	2	-aa-s

Alignment

inst06		proc	private

		Clocktime 6

		FetchDirectWord			; Get our address for the data fetch
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		mov	ah, dh			; Copy our flags
		and	dh, 0FFh - CARRY	; Unset carry
		shr	al, 1			; Shift to the right 1 bit

		adc	dh, 0			; If it carried, set the bit

		and	ah, CARRY		; Check to see if carry was previously set
		jz	noPrevCarry06		; Was it set prior to the ROR?
		or	al, 080h		; Ok, set bit #7 then...

noPrevCarry06:
		FlagSet				; Set the remaining flags.

		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst06		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ASR		07	Direct		6	2	uaa-s

Alignment

inst07		proc	private

		Clocktime 6

		FetchDirectWord			; Get our address for the data fetch
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		and	dh, 0FFh - CARRY	; Unset carry
		FlagSet				; Set the negative before the shift

		shr	al, 1			; Shift to the right 1 bit

		adc	dh, 0			; If it carried, set the bit

		mov	ah, dh			; Copy our flags
		and	ah, NEGATIVE		; Is negative set?
		jz	noNegative07		; No, we're not negative...
		or	al, 080h		; Yes we are, set bit #7 then...

noNegative07:
		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst07		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LSL/ASL	08	Direct		6	2	naaas

Alignment

inst08		proc	private

		Clocktime 6

		FetchDirectWord			; Get our address for the data fetch
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		and	dh, 0FFh - ( OVERFLOW or CARRY )	; Unset overflow and carry

		mov	ah, al			; Make a copy

		shl	al, 1			; Shift to the left 1 bit
		adc	dh, 0			; If it carried, set the bit

		shl	ah, 1			; Do the shift again
		jno	noOverflow08		; Did we overflow?
		or	dh, OVERFLOW		; Yes, set it...

noOverflow08:
		FlagSet				; Set the remaining flags

		shl	edx, 16			; Protect DX

		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst08		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ROL		09	Direct		6	2	-aaas

Alignment

inst09		proc	private

		Clocktime 6

		FetchDirectWord			; Get our address for the data fetch
		shl	ebx, 16			; Protect BX
		shl	ecx, 16			; Protect CX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		mov	cl, al			; Save our data...
		mov	ah, dh			; Copy our flags

		and	dh, 0FFh - ( OVERFLOW or CARRY )	; Unset overflow and carry

		shl	al, 1			; Shift to the right 1 bit
		adc	dh, 0			; If it carried, set the bit

		shl	cl, 1			; Check the data again...

		jno	noOverflow09		; Did we overflow?
		or	dh, OVERFLOW		; Yes, set it...

noOverflow09:
		and	ah, CARRY		; Was carry set prior to the ROL?
		jz	noPrevCarry09		; No, it wasn't
		or	al, 001h		; Yes, set bit 1

noPrevCarry09:
		FlagSet				; Set the remaining flags

		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ecx, 16			; Restore CX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst09		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; DEC		0A	Direct		6	2	-aaa-

Alignment

inst0A		proc	private

		Clocktime	6

		and	dh, 0ffh - (OVERFLOW)	; Zero out overflow

		FetchDirectWord			; Get our address for the data fetch
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		dec	al			; Subtract 1
		jno	noOverflow0A		; Did we overflow?
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflow0A:
		FlagSet				; Set the remaining flags

		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst0A		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; INC		0C	Direct		6	2	-aaa-

Alignment

inst0C		proc	private

		Clocktime	6

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord			; Get our address for the data fetch
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		cmp	al, 07Fh		; Are we gonna overflow?
		jne	noOverflow0C		; No, skip down...
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflow0C:
		inc	al			; Add 1

		FlagSet				; Set the remaining flags

		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst0C		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; TST		0D	Direct		6	2	-aa0-

Alignment

inst0D		proc	private

		Clocktime	6

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord			; Get our address for the data fetch
		ReadMemoryByte			; Go get our data

		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 6	; Return to the main loop

inst0D		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; JMP		0E	Direct		3	2	-----

Alignment

inst0E		proc	private

		Clocktime	3

		FetchDirectWord			; Get our address for the data fetch

		sub	esi, ebp		; Get SI
		mov	si, ax			; Move in our new address
		add	esi, ebp		; Add in the base offset

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 3	; Return to the main loop

inst0E		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CLR		0F	Direct		6	2	-0100

Alignment

inst0F		proc	private

		Clocktime	6

		and	dh, 0f0h		; Unset negative, zero, overflow and carry
		or	dh, ZERO		; Set zero

		FetchDirectWord			; Get our address for the data fetch
		shl	edx, 16			; Protect DX and clear DL

		WriteMemoryByte			; Write out zero

		shr	edx, 16			; Restore DX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst0F		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; PAGE 1	10	Variant		0	1	+++++

Alignment

inst10		proc	private

		ExecutePageOne		; Go do the Page 1 instruction

inst10		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; PAGE 2	11	Variant		0	1	+++++

Alignment

inst11		proc	private

		ExecutePageTwo		; Go do the Page 2 instruction

inst11		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; NOP		12	Inherent	2	1	-----

Alignment

inst12		proc	private

		Clocktime	2
		ExecuteNextInstruction 2	; Return to the main loop

inst12		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SYNC		13	Inherent	2	1	-----
;
; NOT CURRENTLY IMPLEMENTED PROPERLY!!!!!!

Alignment

inst13		proc	private

		Clocktime	2
		ExecuteNextInstruction 2	; Return to the main loop

inst13		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LBRA		16	Relative	5	3	-----

Alignment

inst16		proc	private

		Clocktime	5

		FetchInstructionWord		; Get our address to jump to

		LBranch				; Branch

		ExecuteNextInstruction 5	; Return to the main loop

inst16		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LBSR		17	Relative	9	3	-----

Alignment

inst17		proc	private

		Clocktime	9

		FetchInstructionWord		; Get out offset...
		push	ax			; Save AX

		sub	esi, ebp		; Get our PC into SI
		shl	edx, 16			; Save DX
		mov	dx, si			; Copy the PC into DX
		add	esi, ebp		; Add the base back in...

		LoadStackS			; Load stack pointer S into AX
		PushStackWord			; Push DX onto the stack...
		UnloadStackS			; Unload stack pointer from AX

		shr	edx, 16			; Restore DX

		pop	ax			; Restor AX
		sub	esi, ebp		; Get our PC into SI
		add	si, ax			; Add it in
		add	esi, ebp		; Add back in our base

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 9	; Return to the main loop

inst17		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; DAA		19	Inherent	2	1	-aa0a

Alignment

inst19		proc	private

		Clocktime	2

		and	dh, 0ffh - (OVERFLOW or CARRY )	; Zero out overflow and carry

		mov	al, bh			; Copy A to AL
		lahf				; Save the X86 flags in AH
		and	ah, 0efh		; Make sure aux carry is not set

		test	dh, 020h		; Check the halfcarry flag
		jz	noHalfCarry19		; Nope, it's not set

		or	ah, 010h		; Set the X86 aux. carry
noHalfCarry19:		
		test	dh, 01h			; Check the carry flag
		jz	noCarry19		; Nope, it's not set

		stc
noCarry19:
		sahf				; Save X86 flags

		daa				; Do the conversion

		jnc	noX86Carry19		; Carry was not set

		or	dh, CARRY		; Set carry
noX86Carry19:
		mov	bh, al			; Store A

		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

inst19		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ORCC		1A	Imediate	3	2	ddddd

Alignment

inst1A		proc	private

		Clocktime	3

		FetchInstructionByte		; Get the next byte
		or	dh, al			; OR the returned byte with the flags

		ExecuteNextInstruction 3	; Return to the main loop

inst1A		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ANDCC		1C	Immediate	3	2	ddddd

Alignment

inst1C		proc	private

		Clocktime	3

		FetchInstructionByte		; Get the next byte
		and	dh, al			; AND the returned byte with the flags

		ExecuteNextInstruction 3	; Return to the main loop

inst1C		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SEX		1D	Inherent	2	1	-aa0-

Alignment

inst1D		proc	private

		Clocktime	2

		and	dh, 0ffh - ( OVERFLOW )	; Unset overflow

		mov	al, bl			; Copy B into AL
		cbw				; Make AL into AX

		FlagSetWord			; Set the zero and negative flags

		mov	bx, ax			; Copy D into BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 2	; Return to the main loop

inst1D		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; EXG		1E	Inherent	8	2	ccccc

Alignment

inst1E		proc	private

		Clocktime	8

		FetchInstructionByte		; Get our registers to exchange...

						; Store virtual registers for the swap
ifdef MASM
		mov	_m6809a, bh		; Store A
		mov	_m6809b, bl		; Store B
		mov	_m6809x, cx		; Store X
		mov	_m6809cc, dh		; Store CC
		mov	_m6809dp, dl		; Store DP
		sub	esi, _m6809Base		; Knock off physical address
		mov	_m6809pc, si		; And store virtual address
endif
		xor	ebx, ebx		; Clear out EBX
		xor	ecx, ecx		; Clear out ECX
		xor	edx, edx		; Clear out EDX

		mov	dl, al			; Copy it to DL
		and	dl, 0f0h		; Get the first nibble
		shr	dl, 4			; Make it mean something to us

		mov	dh, al			; Copy it to DH
		and	dh, 0fh			; Get the second nibble

		mov	al, dl			; Copy R1 to AL
		mov	bl, byte ptr regSizeTable[eax]	; Set the size of our register
		
		mov	al, dh			; Copy R2 to AL
		mov	bh, byte ptr regSizeTable[eax]	; Set the size of our register
		
		cmp	bl, bh			; Are they the same size?
		jz	goodReg1E		; Yep, do the exchange...

		mov	bx, 0ffffh		; Do the undocumented thing :-)
		mov	cx, 0ffffh		; Do the undocumented thing :-)

		jmp	setReg1E

goodReg1E:
		mov	al, dl			; Put reg1 into AL
		call	m6809GetReg		; Get contents of reg1 in AX
		mov	cx, ax			; Store reg1 in CX

		mov	al, dh			; Put reg2 into AL
		call	m6809GetReg		; Get contents of reg2 in AX

		mov	bx, ax			; Put contents of reg2 in BX
setReg1E:
		mov	al, dl			; Put reg1 in AL
		call	m6809SetReg		; Write it to reg1

		mov	bx, cx			; Put contents of reg1 in BX
		mov	al, dh			; Put reg2 in AL
		call	m6809SetReg		; Write it to reg2

		xor	esi, esi		; Zero it out

ifdef MASM
		mov	bh, _m6809a		; Get our A register
		mov	bl, _m6809b		; Get our B register
		mov	cx, _m6809x		; Get our X register
		mov	dh, _m6809cc		; Get our flags
		mov	dl, _m6809dp		; Get our DP register
		mov	si, _m6809pc		; Get our program counter
endif

		add	esi, ebp		; Add in our base address

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 8	; Return to the main loop

inst1E		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; TFR		1F	Inherent	7	2	ccccc

Alignment

inst1F		proc	private

		Clocktime	7

		FetchInstructionByte		; Get our registers to exchange...

						; Store virtual registers for the swap
ifdef MASM
		mov	_m6809a, bh		; Store A
		mov	_m6809b, bl		; Store B
		mov	_m6809x, cx		; Store X
		mov	_m6809cc, dh		; Store CC
		mov	_m6809dp, dl		; Store DP
		sub	esi, _m6809Base		; Knock off physical address
		mov	_m6809pc, si		; And store virtual address
		add	esi, ebp		; Add the base back in
endif
		xor	ebx, ebx		; Clear out EBX
		xor	ecx, ecx		; Clear out ECX
		xor	edx, edx		; Clear out EDX

		mov	dl, al			; Copy it to DL
		and	dl, 0f0h		; Get the first nibble
		shr	dl, 4			; Make it mean something to us

		mov	dh, al			; Copy it to DH
		and	dh, 0fh			; Get the second nibble

		mov	al, dl			; Copy R1 to AL
		mov	bl, byte ptr regSizeTable[eax]	; Set the size of our register
		
		mov	al, dh			; Copy R2 to AL
		mov	bh, byte ptr regSizeTable[eax]	; Set the size of our register
		
		cmp	bl, bh			; Are they the same size or R2 is smaller?
		jle	goodReg1F		; Yep, do the exchange...

		mov	bx, 0ffffh		; Do the undocumented thing :-)

		jmp	setReg1F

goodReg1F:
		mov	al, dl			; Put reg1 into AL
		call	m6809GetReg		; Get contents of reg1 in AX
		mov	bx, ax			; Store reg1 in BX

setReg1F:
		mov	al, dh			; Put reg2 in AL
		call	m6809SetReg		; Write it to reg2

		xor	esi, esi		; Zero it out

ifdef MASM
		mov	bh, _m6809a		; Get our A register
		mov	bl, _m6809b		; Get our B register
		mov	cx, _m6809x		; Get our X register
		mov	dh, _m6809cc		; Get our flags
		mov	dl, _m6809dp		; Get our DP register
		mov	si, _m6809pc		; Get our program counter
endif

		add	esi, ebp		; Add in our base address
		xor	ah, ah

		ExecuteNextInstruction 7	; Return to the main loop

inst1F		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BRA		20	Relative	3	2	-----

Alignment

inst20		proc	private

		Clocktime	3

		FetchInstructionByte		; Get the offset

		Branch				; Branch

		ExecuteNextInstruction 3	; Return to the main loop

inst20		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BRN		21	Relative	3	2	-----

Alignment

inst21		proc	private

		Clocktime	3

		FetchInstructionByte		; Get the offset
		ExecuteNextInstruction 3	; Return to the main loop

inst21		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BHI		22	Relative	3	2	-----

Alignment

inst22		proc	private

		Clocktime	3

		FetchInstructionByte		; Get the offset

		mov	ah, dh			; Copy our flags
		and	ah, (ZERO or CARRY)	; Isolate zero and carry
		jnz	noBranch22		; One of the is set so no branch

		Branch				; Branch

noBranch22:
		xor	ah, ah
		ExecuteNextInstruction 3	; Return to the main loop

inst22		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BLS		23	Relative	3	2	-----

Alignment

inst23		proc	private

		Clocktime	3

		FetchInstructionByte		; Get the offset

		mov	ah, dh			; Copy our flags
		and	ah, (ZERO or CARRY)	; Isolate zero and carry
		jz	noBranch23		; Neither is set so no branch

		Branch				; Branch

noBranch23:
		xor	ah, ah
		ExecuteNextInstruction 3	; Return to the main loop

inst23		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BCC		24	Relative	3	2	-----

Alignment

inst24		proc	private

		Clocktime	3

		FetchInstructionByte		; Get the offset

		mov	ah, dh			; Copy our flags
		and	ah, (CARRY)		; Isolate carry
		jnz	noBranch24		; Carry is set so no branch

		Branch				; Branch

noBranch24:
		xor	ah, ah
		ExecuteNextInstruction 3	; Return to the main loop

inst24		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BCS		25	Relative	3	2	-----

Alignment

inst25		proc	private

		Clocktime	3

		FetchInstructionByte		; Get the offset

		mov	ah, dh			; Copy our flags
		and	ah, (CARRY)		; Isolate carry
		jz	noBranch25		; Carry is not set so no branch

		Branch				; Branch

noBranch25:
		xor	ah, ah
		ExecuteNextInstruction 3	; Return to the main loop

inst25		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BNE		26	Relative	3	2	-----

Alignment

inst26		proc	private

		Clocktime	3

		FetchInstructionByte		; Get the offset

		mov	ah, dh			; Copy our flags
		and	ah, (ZERO)		; Isolate zero
		jnz	noBranch26		; Zero is set so no branch

		Branch				; Branch

noBranch26:
		xor	ah, ah
		ExecuteNextInstruction 3	; Return to the main loop

inst26		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BEQ		27	Relative	3	2	-----

Alignment

inst27		proc	private

		Clocktime	3

		FetchInstructionByte		; Get the offset

		mov	ah, dh			; Copy our flags
		and	ah, (ZERO)		; Isolate zero
		jz	noBranch27		; Zero is not set so no branch

		Branch				; Branch

noBranch27:
		xor	ah, ah
		ExecuteNextInstruction 3	; Return to the main loop

inst27		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BVC		28	Relative	3	2	-----

Alignment

inst28		proc	private

		Clocktime	3

		FetchInstructionByte		; Get the offset

		mov	ah, dh			; Copy our flags
		and	ah, (OVERFLOW)		; Isolate overflow
		jnz	noBranch28		; Overflow is set so no branch

		Branch				; Branch

noBranch28:
		xor	ah, ah
		ExecuteNextInstruction 3	; Return to the main loop

inst28		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BVS		29	Relative	3	2	-----

Alignment

inst29		proc	private

		Clocktime	3

		FetchInstructionByte		; Get the offset

		mov	ah, dh			; Copy our flags
		and	ah, (OVERFLOW)		; Isolate overflow
		jz	noBranch29		; Overflow is not set so no branch

		Branch				; Branch

noBranch29:
		xor	ah, ah
		ExecuteNextInstruction 3	; Return to the main loop

inst29		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BPL		2A	Relative	3	2	-----

Alignment

inst2A		proc	private

		Clocktime	3

		FetchInstructionByte		; Get the offset

		mov	ah, dh			; Copy our flags
		and	ah, (NEGATIVE)		; Isolate negative
		jnz	noBranch2A		; Negative is set so no branch

		Branch				; Branch

noBranch2A:
		xor	ah, ah
		ExecuteNextInstruction 3	; Return to the main loop

inst2A		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BMI		2B	Relative	3	2	-----

Alignment

inst2B		proc	private

		Clocktime	3

		FetchInstructionByte		; Get the offset

		mov	ah, dh			; Copy our flags
		and	ah, (NEGATIVE)		; Isolate negative
		jz	noBranch2B		; Negative is not set so no branch

		Branch				; Branch

noBranch2B:
		xor	ah, ah
		ExecuteNextInstruction 3	; Return to the main loop

inst2B		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BGE		2C	Relative	3	2	-----

Alignment

inst2C		proc	private

		Clocktime	3

		FetchInstructionByte		; Get the offset

		shl	eax, 16			; Save our offset

		mov	ah, dh			; Copy our flags
		and	ah, (OVERFLOW)		; Isolate overflw
		shl	ah, 2			; Put overflow under negative
		mov	al, dh			; Copy our flags
		and	al, (NEGATIVE)		; Isolate negative
		xor	al, ah			; XOR negative and overflow
		jnz	noBranch2C		; Only one is not set so no branch

		shr	eax, 16			; Restore our offset
		
		Branch				; Branch

noBranch2C:
		xor	eax, eax
		ExecuteNextInstruction 3	; Return to the main loop

inst2C		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BLT		2D	Relative	3	2	-----

Alignment

inst2D		proc	private

		Clocktime	3

		FetchInstructionByte		; Get the offset


		shl	eax, 16			; Save our offset

		mov	ah, dh			; Copy our flags
		and	ah, (OVERFLOW)		; Isolate overflw
		shl	ah, 2			; Put overflow under negative
		mov	al, dh			; Copy our flags
		and	al, (NEGATIVE)		; Isolate negative
		xor	al, ah			; XOR negative and overflow
		jz	noBranch2D		; Only one is not set so no branch

		shr	eax, 16			; Restore our offset

		Branch				; Branch

noBranch2D:
		xor	eax, eax
		ExecuteNextInstruction 3	; Return to the main loop

inst2D		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BGT		2E	Relative	3	2	-----

Alignment

inst2E		proc	private

		Clocktime	3

		FetchInstructionByte		; Get the offset

		shl	eax, 16			; Save our offset

		mov	ah, dh			; Copy our flags
		and	ah, (OVERFLOW)		; Isolate overflw
		shl	ah, 2			; Put overflow under negative
		mov	al, dh			; Copy our flags
		and	al, (NEGATIVE)		; Isolate negative
		xor	al, ah			; XOR negative and overflow
		mov	ah, dh			; Copy our flags
		and	ah, (ZERO)		; Isolate zero
		or	al, ah			; OR zero and XOR from above
		jnz	noBranch2E		; Only one is not set so no branch

		shr	eax, 16			; Restore our offset

		Branch				; Branch

noBranch2E:
		xor	eax, eax
		ExecuteNextInstruction 3	; Return to the main loop

inst2E		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BLE		2F	Relative	3	2	-----

Alignment

inst2F		proc	private

		Clocktime	3

		FetchInstructionByte		; Get the offset


		shl	eax, 16			; Save our offset

		mov	ah, dh			; Copy our flags
		and	ah, (OVERFLOW)		; Isolate overflw
		shl	ah, 2			; Put overflow under negative
		mov	al, dh			; Copy our flags
		and	al, (NEGATIVE)		; Isolate negative
		xor	al, ah			; XOR negative and overflow
		mov	ah, dh			; Copy our flags
		and	ah, (ZERO)		; Isolate zero
		or	al, ah			; OR zero with XOR from above
		jz	noBranch2F		; Only one is not set so no branch

		shr	eax, 16			; Restore our offset

		Branch				; Branch

noBranch2F:
		xor	eax, eax
		ExecuteNextInstruction 3	; Return to the main loop

inst2F		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LEAX		30	Indexed		4	2	--a--

Alignment

inst30		proc	private

		Clocktime	4

		and	dh, 0ffh - (ZERO)	; Zero out carry and overflow

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from

		mov	cx, ax			; Copy the data into X
		or	cx, cx			; Is it zero?

		jnz	noZero30		; It's not zero
		or	dh, ZERO		; Yes it is, set zero

noZero30:
		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 4	; Return to the main loop

inst30		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LEAY		31	Indexed		4	2	--a--

Alignment

inst31		proc	private

		Clocktime	4

		and	dh, 0ffh - (ZERO)	; Zero out carry and overflow

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from

ifdef MASM
		mov	_m6809y, ax		; Copy AX into Y
endif
		or	ax, ax			; Is it zero?

		jnz	noZero31		; It's not zero
		or	dh, ZERO		; Yes it is, set zero

noZero31:
		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 4	; Return to the main loop

inst31		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LEAS		32	Indexed		4	2	-----

Alignment

inst32		proc	private

		Clocktime	4

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from

ifdef MASM
		mov	_m6809s, ax		; Copy CX into S
endif
		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 4	; Return to the main loop

inst32		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LEAU		33	Indexed		4	2	-----

Alignment

inst33		proc	private

		Clocktime	4

		FetchInstructionByte	; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from

ifdef MASM
		mov	_m6809u, ax		; Copy AX into U
endif
		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 4	; Return to the main loop

inst33		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; PSHS		34	Inherent	5	2	-----

Alignment

inst34		proc	private

		Clocktime	5

		FetchInstructionByte		; Get our postbyte

ifdef MASM
		mov	_m6809a, bh		; Store B
		mov	_m6809b, bl		; Store A
		mov	_m6809x, cx		; Store X
		mov	_m6809cc, dh		; Store CC
		mov	_m6809dp, dl		; Store DP
		sub	esi, _m6809Base		; Knock off physical address
		mov	_m6809pc, si		; And store virtual address
		add	esi, ebp
endif

		mov	cl, al			; Copy AL into CL

		LoadStackS			; Load stack pointer S into AX

		mov	ch, cl			; Copy CL into CH
		and	ch, 080h		; Is 0x80 set?
		jz	noPC34			; Nope

ifdef MASM
		mov	dx, _m6809pc		; Put the PC in DX
endif
		PushStackWord			; Push the word on the S stack

noPC34:
		mov	ch, cl			; Copy CL into CH
		and	ch, 040h		; Is 0x40 set?
		jz	noU34			; Nope

ifdef MASM
		mov	dx, _m6809u		; Put the U in DX
endif
		PushStackWord			; Push the word on the S stack

noU34:
		mov	ch, cl			; Copy CL into CH
		and	ch, 020h		; Is 0x20 set?
		jz	noY34			; Nope

ifdef MASM
		mov	dx, _m6809y		; Put the Y in DX
endif
		PushStackWord			; Push the word on the S stack

noY34:
		mov	ch, cl			; Copy CL into CH
		and	ch, 010h		; Is 0x10 set?
		jz	noX34			; Nope

ifdef MASM
		mov	dx, _m6809x		; Put the X in DX
endif
		PushStackWord			; Push the word on the S stack

noX34:
		mov	ch, cl			; Copy CL into CH
		and	ch, 008h		; Is 0x08 set?
		jz	noDP34			; Nope

ifdef MASM
		mov	dl, _m6809dp		; Put the DP in DL
endif
		PushStackByte			; Push the byte on the S stack

noDP34:
		mov	ch, cl			; Copy CL into CH
		and	ch, 004h		; Is 0x04 set?
		jz	noB34			; Nope

ifdef MASM
		mov	dl, _m6809b		; Put B in DL
endif
		PushStackByte			; Push the byte on the S stack

noB34:
		mov	ch, cl			; Copy CL into CH
		and	ch, 002h		; Is 0x02 set?
		jz	noA34			; Nope

ifdef MASM
		mov	dl, _m6809a		; Put A in DL
endif
		PushStackByte			; Push the byte on the S stack

noA34:
		mov	ch, cl			; Copy CL into CH
		and	ch, 001h		; Is 0x01 set?
		jz	noCC34			; Nope

ifdef MASM
		mov	dl, _m6809cc		; Put the CC in DL
endif
		PushStackByte			; Push the byte on the S stack

noCC34:
		UnloadStackS			; Unload stack pointer from AX

ifdef MASM
		mov	bh, _m6809a		; Restore A
		mov	bl, _m6809b		; Restore B
		mov	cx, _m6809x		; Restore X
		mov	dh, _m6809cc		; Restore CC
		mov	dl, _m6809dp		; Restore DP
		xor	esi, esi		; Clear out for new PC
		mov	si, _m6809pc		; Restore virtual address
		add	esi, ebp		; Add in the base
endif
		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 5	; Return to the main loop

inst34		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; PULS		35	Inherent	5	2	ccccc

Alignment

inst35		proc	private

		Clocktime	5

		FetchInstructionByte		; Get our postbyte

ifdef MASM
		mov	_m6809a, bh		; Store B
		mov	_m6809b, bl		; Store A
		mov	_m6809x, cx		; Store X
		mov	_m6809cc, dh		; Store CC
		mov	_m6809dp, dl		; Store DP
		sub	esi, _m6809Base		; Knock off physical address
		mov	_m6809pc, si		; And store virtual address
		add	esi, ebp
endif

		mov	cl, al			; Copy AL into CL

		LoadStackS			; Load stack pointer S into AX

		mov	ch, cl			; Copy CL into CH
		and	ch, 001h		; Is 0x01 set?
		jz	noCC35			; Nope

		PopStackByte			; Pop the byte from the S stack

ifdef MASM
		mov	_m6809cc, dl 		; Save CC
endif

noCC35:
		mov	ch, cl			; Copy CL into CH
		and	ch, 002h		; Is 0x02 set?
		jz	noA35			; Nope

		PopStackByte			; Pop the byte from the S stack
ifdef MASM
		mov	_m6809a, dl 		; Save A
endif

noA35:
		mov	ch, cl			; Copy CL into CH
		and	ch, 004h		; Is 0x04 set?
		jz	noB35			; Nope

		PopStackByte			; Pop the byte from the S stack
ifdef MASM
		mov	_m6809b, dl 		; Save B
endif

noB35:
		mov	ch, cl			; Copy CL into CH
		and	ch, 008h		; Is 0x08 set?
		jz	noDP35			; Nope

		PopStackByte			; Pop the byte from the S stack
ifdef MASM
		mov	_m6809dp, dl 		; Save DP
endif

noDP35:
		mov	ch, cl			; Copy CL into CH
		and	ch, 010h		; Is 0x10 set?
		jz	noX35			; Nope

		PopStackWord			; Pop the word from the S stack
ifdef MASM
		mov	_m6809x, dx 		; Save X
endif

noX35:
		mov	ch, cl			; Copy CL into CH
		and	ch, 020h		; Is 0x20 set?
		jz	noY35			; Nope

		PopStackWord			; Pop the word from the S stack
ifdef MASM
		mov	_m6809y, dx 		; Save Y
endif

noY35:
		mov	ch, cl			; Copy CL into CH
		and	ch, 040h		; Is 0x40 set?
		jz	noU35			; Nope

		PopStackWord			; Pop the word from the S stack
ifdef MASM
		mov	_m6809u, dx 		; Save U
endif

noU35:
		mov	ch, cl			; Copy CL into CH
		and	ch, 080h		; Is 0x80 set?
		jz	noPC35			; Nope

		PopStackWord			; Pop the word from the S stack
ifdef MASM
		mov	_m6809pc, dx 		; Save PC
endif

noPC35:
		UnloadStackS			; Unload stack pointer from AX

ifdef MASM
		mov	bh, _m6809a		; Restore A
		mov	bl, _m6809b		; Restore B
		mov	cx, _m6809x		; Restore X
		mov	dh, _m6809cc		; Restore CC
		mov	dl, _m6809dp		; Restore DP
		xor	esi, esi		; Clear out for new PC
		mov	si, _m6809pc		; Restore virtual address
		add	esi, ebp		; Add in the base
endif
		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 5	; Return to the main loop

inst35		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; PSHU		36	Inherent	5	2	-----

Alignment

inst36		proc	private

		Clocktime	5

		FetchInstructionByte		; Get our postbyte

ifdef MASM
		mov	_m6809a, bh		; Store B
		mov	_m6809b, bl		; Store A
		mov	_m6809x, cx		; Store X
		mov	_m6809cc, dh		; Store CC
		mov	_m6809dp, dl		; Store DP
		sub	esi, _m6809Base		; Knock off physical address
		mov	_m6809pc, si		; And store virtual address
		add	esi, ebp
endif
		mov	cl, al			; Copy AL into CL

		LoadStackU			; Load U into AX

		mov	ch, cl			; Copy CL into CH
		and	ch, 080h		; Is 0x40 set?
		jz	noPC36			; Nope

ifdef MASM
		mov	dx, _m6809pc		; Put the PC in DX
endif
		PushStackWord			; Push the word on the U stack

noPC36:
		mov	ch, cl			; Copy CL into CH
		and	ch, 040h		; Is 0x40 set?
		jz	noS36			; Nope

ifdef MASM
		mov	dx, _m6809s		; Put the U in DX
endif
		PushStackWord			; Push the word on the U stack

noS36:
		mov	ch, cl			; Copy CL into CH
		and	ch, 020h		; Is 0x20 set?
		jz	noY36			; Nope

ifdef MASM
		mov	dx, _m6809y		; Put the Y in DX
endif
		PushStackWord			; Push the word on the U stack

noY36:
		mov	ch, cl			; Copy CL into CH
		and	ch, 010h		; Is 0x10 set?
		jz	noX36			; Nope

ifdef MASM
		mov	dx, _m6809x		; Put the X in DX
endif
		PushStackWord			; Push the word on the U stack

noX36:
		mov	ch, cl			; Copy CL into CH
		and	ch, 008h		; Is 0x08 set?
		jz	noDP36			; Nope

ifdef MASM
		mov	dl, _m6809dp		; Put the DP in DL
endif
		PushStackByte			; Push the byte on the U stack

noDP36:
		mov	ch, cl			; Copy CL into CH
		and	ch, 004h		; Is 0x04 set?
		jz	noB36			; Nope

ifdef MASM
		mov	dl, _m6809b		; Put B in DX
endif
		PushStackByte			; Push the byte on the U stack

noB36:
		mov	ch, cl			; Copy CL into CH
		and	ch, 002h		; Is 0x02 set?
		jz	noA36			; Nope

ifdef MASM
		mov	dl, _m6809a		; Put A in DX
endif
		PushStackByte			; Push the byte on the U stack

noA36:
		mov	ch, cl			; Copy CL into CH
		and	ch, 001h		; Is 0x01 set?
		jz	noCC36			; Nope

ifdef MASM
		mov	dl, _m6809cc		; Put the CC in DL
endif
		PushStackByte			; Push the byte on the U stack

noCC36:
		UnloadStackU			; Save U from AX

ifdef MASM
		mov	bh, _m6809a		; Restore A
		mov	bl, _m6809b		; Restore B
		mov	cx, _m6809x		; Restore X
		mov	dh, _m6809cc		; Restore CC
		mov	dl, _m6809dp		; Restore DP
		xor	esi, esi		; Clear out for new PC
		mov	si, _m6809pc		; Restore virtual address
		add	esi, ebp		; Add in the base
endif
		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 5	; Return to the main loop

inst36		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; PULU		37	Inherent	5	2	ccccc

Alignment

inst37		proc	private

		Clocktime	5

		FetchInstructionByte		; Get our postbyte

ifdef MASM
		mov	_m6809a, bh		; Store B
		mov	_m6809b, bl		; Store A
		mov	_m6809x, cx		; Store X
		mov	_m6809cc, dh		; Store CC
		mov	_m6809dp, dl		; Store DP
		sub	esi, _m6809Base		; Knock off physical address
		mov	_m6809pc, si		; And store virtual address
		add	esi, ebp
endif
		mov	cl, al			; Copy AL into CL

		LoadStackU			; Copy U into AX

		mov	ch, cl			; Copy CL into CH
		and	ch, 001h		; Is 0x01 set?
		jz	noCC37			; Nope

		PopStackByte			; Pop the byte from the U stack

ifdef MASM
		mov	_m6809cc, dl 		; Save CC
endif

noCC37:
		mov	ch, cl			; Copy CL into CH
		and	ch, 002h		; Is 0x02 set?
		jz	noA37			; Nope

		PopStackByte			; Pop the byte from the U stack
ifdef MASM
		mov	_m6809a, dl 		; Save A
endif

noA37:
		mov	ch, cl			; Copy CL into CH
		and	ch, 004h		; Is 0x04 set?
		jz	noB37			; Nope

		PopStackByte			; Pop the byte from the U stack
ifdef MASM
		mov	_m6809b, dl 		; Save B
endif

noB37:
		mov	ch, cl			; Copy CL into CH
		and	ch, 008h		; Is 0x08 set?
		jz	noDP37			; Nope

		PopStackByte			; Pop the byte from the U stack
ifdef MASM
		mov	_m6809dp, dl 		; Save DP
endif

noDP37:
		mov	ch, cl			; Copy CL into CH
		and	ch, 010h		; Is 0x10 set?
		jz	noX37			; Nope

		PopStackWord			; Pop the word from the U stack
ifdef MASM
		mov	_m6809x, dx 		; Save X
endif

noX37:
		mov	ch, cl			; Copy CL into CH
		and	ch, 020h		; Is 0x20 set?
		jz	noY37			; Nope

		PopStackWord			; Pop the word from the U stack
ifdef MASM
		mov	_m6809y, dx 		; Save Y
endif

noY37:
		mov	ch, cl			; Copy CL into CH
		and	ch, 040h		; Is 0x40 set?
		jz	noS37			; Nope

		PopStackWord			; Pop the word from the U stack
ifdef MASM
		mov	_m6809s, dx 		; Save S
endif

noS37:
		mov	ch, cl			; Copy CL into CH
		and	ch, 080h		; Is 0x80 set?
		jz	noPC37			; Nope

		PopStackWord			; Pop the word from the U stack
ifdef MASM
		mov	_m6809pc, dx 		; Save PC
endif

noPC37:
		UnloadStackU			; Save AX to U

ifdef MASM
		mov	bh, _m6809a		; Restore A
		mov	bl, _m6809b		; Restore B
		mov	cx, _m6809x		; Restore X
		mov	dh, _m6809cc		; Restore CC
		mov	dl, _m6809dp		; Restore DP
		xor	esi, esi		; Clear out for new PC
		mov	si, _m6809pc		; Restore virtual address
		add	esi, ebp		; Add in the base
endif
		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 5	; Return to the main loop

inst37		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; RTS		39	Inherent	5	1	-----

Alignment

inst39		proc	private

		Clocktime	5

		shl	edx, 16			; Save DX

		LoadStackS			; Load stack pointer S into AX
		PopStackWord			; Pop the word from the S stack
		UnloadStackS			; Unload stack pointer from AX

		xor	esi, esi		; Zero out ESI
		mov	si, dx			; Put the PC in SI
		add	esi, ebp		; Add in our base address

		shr	edx, 16			; Restore DX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 5	; Return to the main loop

inst39		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ABX		3A	Inherent	3	1	-----

Alignment

inst3A		proc	private

		Clocktime	3

		mov	al, bl			; Put B into AL
		xor	ah, ah			; Just in case...
		add	cx, ax			; Add B to X

		ExecuteNextInstruction 3	; Return to the main loop

inst3A		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; RTI		3B	Inherent	6/15	1	-----

Alignment

inst3B		proc	private

		mov	al, dh			; Copy flags to AL
		and	al, ENTIRE		; Check to see what we need to Pop
		jnz	popAll3B		; Pop it all...

		Clocktime	6

		shl	edx, 16			; Save DX

		LoadStackS			; Load stack pointer S into AX

		PopStackByte			; Pop the byte from the S stack

		shl	eax, 16			; Save current AX
		mov	al, dl			; Save new CC into AL
		shr	edx, 16			; Restore DX
		mov	dh, al			; Copy in the new CC
		shr	eax, 16			; Restore AX

		shl	edx, 16			; Save DX

		PopStackWord			; Pop the word from the S stack

		xor	esi, esi		; Zero it out
		mov	si, dx			; Put the PC in SI
		add	esi, ebp		; Add in our base address

		shr	edx, 16			; Restore DX

		LoadStackS			; Load stack pointer S into AX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop
popAll3B:
		Clocktime	15

		shl	edx, 16			; Save DX

		LoadStackS			; Load stack pointer S into AX

		PopStackByte			; Pop CC from the S stack

		shl	eax, 16			; Save current AX
		mov	al, dl			; Save new CC into AL
		shr	edx, 16			; Restore DX
		mov	dh, al			; Copy in the new CC
		shr	eax, 16			; Restore AX

		shl	edx, 16			; Save DX

		PopStackByte			; Pop A from the S stack
		mov	bh, dl			; Put DL into A

		PopStackByte			; Pop B from the S stack
		mov	bl, dl			; Put DL into B

		PopStackByte			; Pop DP from the S stack

		shl	eax, 16			; Save current AX
		mov	al, dl			; Save new DP into AL
		shr	edx, 16			; Restore DX
		mov	dl, al			; Copy in the new DP
		shr	eax, 16			; Restore AX

		shl	edx, 16			; Save DX

		PopStackWord			; Pop X from the S stack

		mov	cx, dx			; Put the new X into CX

		PopStackWord			; Pop Y from the S stack
ifdef MASM
		mov	_m6809y, dx		; Put DX into Y
endif

		PopStackWord			; Pop U from the S stack
ifdef MASM
		mov	_m6809u, dx		; Put DX into U
endif

		PopStackWord			; Pop the word from the S stack

		xor	esi, esi		; Zero it out
		mov	si, dx			; Put the PC in SI
		add	esi, ebp		; Add in our base address

		shr	edx, 16			; Restore DX

		UnloadStackS			; Unload stack pointer from AX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 15	; Return to the main loop

inst3B		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; MUL		3D	Inherent	11	1	--a-a

Alignment

inst3D		proc	private

		Clocktime	11

		and	dh, 0ffh - (ZERO or CARRY)	; Zero out zero and carry

		shl	ecx, 16			; Save CX
		mov	al, bl			; Copy B into AL
		mov	cl, bh			; Copy A into CL

		mul	cl			; Multiply A*B

		bt	ax, 7			; Check bit 7 of AX
		jnc	noCarry3D		; Did we carry?

		or	dh, CARRY		; Yes, we carried

noCarry3D:
		or	ax, ax			; Is it zero?
		jnz	noZero3D		; Nope

		or	dh, ZERO		; Yes, set zero

noZero3D:
		mov	bx, ax			; Copy D into BX

		shr	ecx, 16			; Restore CX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 11	; Return to the main loop

inst3D		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SWI		3F	Inherent	19	1	-----

Alignment

inst3F		proc	private

		Clocktime	19

ifdef MASM
		mov	_m6809dp, dl		; Save DP
		mov	_m6809cc, dh		; Save CC
endif
		shl	edx, 16			; Save DX

		LoadStackS			; Load stack pointer S into AX

		sub	esi, ebp		; Get our PC in SI
		mov	dx, si			; Copy it to DX
		add	esi, ebp		; Add the base back in...
		PushStackWord			; Push it on the stack

ifdef MASM
		mov	dx, _m6809u		; Put U into DX
endif
		PushStackWord			; Push it on the stack

ifdef MASM
		mov	dx, _m6809y		; Put Y into DX
endif
		PushStackWord			; Push it on the stack

		mov	dx, cx			; Put X into DX
		PushStackWord			; Push it on the stack

ifdef MASM
		mov	dl, _m6809dp		; Put DP into DL
endif
		PushStackByte			; Push it on the stack

		mov	dl, bl			; Put B into DL
		PushStackByte			; Push it on the stack

		mov	dl, bh			; Put A into DL
		PushStackByte			; Push it on the stack

ifdef MASM
		mov	dl, _m6809cc		; Put CC into DL
endif
		PushStackByte			; Push it on the stack

		UnloadStackS			; Unload stack pointer from AX

		mov	ax, 0fffah		; The address of the SWI vector
		ReadMemoryWord			; Read it

		xor	esi, esi		; Clear out ESI
		mov	si, ax			; Put the new PC in SI
		add	esi, ebp		; Add in our base

		shr	edx, 16			; Restore CC and DP
		or	dh, ENTIRE		; Set entire flag

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 19	; Return to the main loop

inst3F		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; NEGA		40	Inherent	2	1	uaaaa

Alignment

inst40		proc	private

		Clocktime	2

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		cmp	bh, 80h			; Is A gonna overflow?
		jne	noOverflow40		; No, skip down...
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflow40:
		neg	bh			; Negate A

		adc	dh, 0			; If it carried, set the bit
		mov	al, bh			; Copy A to AL for setting the flags
		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

inst40		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; COMA		43	Inherent	2	1	-aa01

Alignment

inst43		proc	private

		Clocktime	2

		xor	bh, 0ffh		; Take the 1's complement of A

		or	dh, CARRY		; Set the carry bit
		and	dh, 0ffh - OVERFLOW	; There's no overflow
		mov	al, bh			; Copy A to AL for setting the flags
		FlagSet				; Set the remaining flags.

		ExecuteNextInstruction 2	; Return to the main loop

inst43		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LSRA		44	Inherent	2	1	-0a-s

Alignment

inst44		proc	private

		Clocktime 2

		and	dh, 0ffh - (NEGATIVE or CARRY)	; There's no negative
		shr	bh, 1			; Shift A to the right 1 bit

		adc	dh, 0			; If it carried, set the bit
		mov	al, bh			; Copy A to AL for setting the flags
		FlagSet				; Set the remaining flags.

		ExecuteNextInstruction 2	; Return to the main loop

inst44		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; RORA		46	Inherent	2	1	-aa-s

Alignment

inst46		proc	private

		Clocktime 2

		mov	ah, dh			; Copy our flags
		and	dh, 0FFh - CARRY	; Unset carry
		shr	bh, 1			; Shift A to the right 1 bit

		adc	dh, 0			; If it carried, set the bit

		and	ah, CARRY		; Check to see if carry was previously set
		jz	noPrevCarry46		; Was it set prior to the ROR?
		or	bh, 080h		; Ok, set bit #7 then...

noPrevCarry46:
		mov	al, bh			; Copy A to AL for setting the flags
		FlagSet				; Set the remaining flags.

		ExecuteNextInstruction 2	; Return to the main loop

inst46		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ASRA		47	Inherent	2	1	uaa-s

Alignment

inst47		proc	private

		Clocktime 2

		and	dh, 0FFh - CARRY	; Unset carry
		mov	al, bh			; Copy A to AL for setting the flags
		FlagSet				; Set the negative before the shift

		shr	bh, 1			; Shift A to the right 1 bit

		adc	dh, 0			; If it carried, set the bit

		mov	ah, dh			; Copy our flags
		and	ah, NEGATIVE		; Is negative set?
		jz	noNegative47		; No, we're not negative...
		or	bh, 080h		; Yes we are, set bit #7 then...

noNegative47:
		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 2	; Return to the main loop

inst47		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LSLA/ASLA	48	Inherent	2	1	naaas

Alignment

inst48		proc	private

		Clocktime 2

		and	dh, 0FFh - ( OVERFLOW or CARRY )	; Unset overflow and carry

		mov	al, bh			; Copy A to AL

		shl	al, 1			; Shift to the left 1 bit
		adc	dh, 0			; If it carried, set the bit

		shl	bh, 1			; Shift it for real
		jno	noOverflow48		; Did we overflow?
		or	dh, OVERFLOW		; Yes, set it...

noOverflow48:
		FlagSet				; Set the remaining flags.

		ExecuteNextInstruction 2	; Return to the main loop

inst48		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ROLA		49	Inherent	2	1	-aaas

Alignment

inst49		proc	private

		Clocktime 2

		mov	ah, dh			; Copy our flags

		and	dh, 0FFh - ( OVERFLOW or CARRY )	; Unset overflow and carry

		mov	al, bh			; Copy A to AL

		shl	al, 1			; Shift to the left 1 bit
		adc	dh, 0			; If it carried, set the bit

		shl	bh, 1			; Shift A to the left 1 bit

		jno	noOverflow49		; Did we overflow?
		or	dh, OVERFLOW		; Yes, set it...

noOverflow49:
		and	ah, CARRY		; Was carry set prior to the ROL?
		jz	noPrevCarry49		; No, it wasn't
		or	bh, 01h			; Yes, set A's bit 1

noPrevCarry49:
		mov	al, bh			; Copy A to AL for setting the flags

		FlagSet				; Set the remaining flags.

		ExecuteNextInstruction 2	; Return to the main loop

inst49		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; DECA		4A	Inherent	2	1	-aaa-

Alignment

inst4A		proc	private

		Clocktime	2

		and	dh, 0ffh - (OVERFLOW)	; Zero out overflow

		dec	bh			; Subtract 1 from A
		jno	noOverflow4A		; Did we overflow?
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflow4A:
		mov	al, bh			; Copy A to AL for setting the flags
		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

inst4A		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; INCA		4C	Inherent	2	1	-aaa-

Alignment

inst4C		proc	private

		Clocktime	2

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		inc	bh			; Add 1 to A
		jno	noOverflow4C		; Did we overflow?
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflow4C:
		mov	al, bh			; Copy A to AL for setting the flags
		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

inst4C		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; TSTA		4D	Inherent	2	1	-aa0-

Alignment

inst4D		proc	private

		Clocktime	2

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		mov	al, bh			; Copy A to AL for setting the flags
		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

inst4D		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CLRA		4F	Direct		2	1	-0100

Alignment

inst4F		proc	private

		Clocktime	2

		and	dh, 0f0h		; Unset negative, zero, overflow and carry
		or	dh, ZERO		; Set zero

		xor	bh, bh			; Clear out A

		ExecuteNextInstruction 2	; Return to the main loop

inst4F		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; NEGB		50	Inherent	2	1	uaaaa

Alignment

inst50		proc	private

		Clocktime	2

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		cmp	bl, 80h			; Is B gonna overflow?
		jne	noOverflow50		; No, skip down...
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflow50:
		neg	bl			; Negate B

		adc	dh, 0			; If it carried, set the bit
		mov	al, bl			; Copy B to AL for setting the flags
		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

inst50		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; COMB		53	Inherent	2	1	-aa01

Alignment

inst53		proc	private

		Clocktime	2

		xor	bl, 0ffh		; Take the 1's complement of B

		or	dh, CARRY		; Set the carry bit
		and	dh, 0ffh - OVERFLOW	; There's no overflow
		mov	al, bl			; Copy B to AL for setting the flags
		FlagSet				; Set the remaining flags.

		ExecuteNextInstruction 2	; Return to the main loop

inst53		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LSRB		54	Inherent	2	1	-0a-s

Alignment

inst54		proc	private

		Clocktime 2

		and	dh, 0ffh - (NEGATIVE or CARRY)	; There's no negative
		shr	bl, 1			; Shift B to the right 1 bit

		adc	dh, 0			; If it carried, set the bit
		mov	al, bl			; Copy B to AL for setting the flags
		FlagSet				; Set the remaining flags.

		ExecuteNextInstruction 2	; Return to the main loop

inst54		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; RORB		56	Inherent	2	1	-aa-s

Alignment

inst56		proc	private

		Clocktime 2

		mov	ah, dh			; Copy our flags
		and	dh, 0FFh - CARRY	; Unset carry
		shr	bl, 1			; Shift B to the right 1 bit

		adc	dh, 0			; If it carried, set the bit

		and	ah, CARRY		; Check to see if carry was previously set
		jz	noPrevCarry46a		; Was it set prior to the ROR?
		or	bl, 080h		; Ok, set bit #7 then...

noPrevCarry46a:
		mov	al, bl;			; Copy B to AL for setting the flags
		FlagSet				; Set the remaining flags.

		ExecuteNextInstruction 2	; Return to the main loop

inst56		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ASRB		57	Inherent	2	1	uaa-s

Alignment

inst57		proc	private

		Clocktime 2

		and	dh, 0FFh - CARRY	; Unset carry
		mov	al, bl			; Copy B to AL for setting the flags
		FlagSet				; Set the negative before the shift

		shr	bl, 1			; Shift B to the right 1 bit

		adc	dh, 0			; If it carried, set the bit

		mov	ah, dh			; Copy our flags
		and	ah, NEGATIVE		; Is negative set?
		jz	noNegative57		; No, we're not negative...
		or	bl, 080h		; Yes we are, set bit #7 then...

noNegative57:
		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 2	; Return to the main loop

inst57		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LSLB/ASLB	58	Inherent	2	1	naaas

Alignment

inst58		proc	private

		Clocktime 2

		and	dh, 0FFh - ( OVERFLOW or CARRY )	; Unset overflow and carry

		mov	al, bl			; Copy B to AL

		shl	al, 1			; Shift to the left 1 bit
		adc	dh, 0			; If it carried, set the bit

		shl	bl, 1			; Shift B to the left 1 bit
		jno	noOverflow58		; Did we overflow?
		or	dh, OVERFLOW		; Yes, set it...

noOverflow58:
		FlagSet				; Set the remaining flags.

		ExecuteNextInstruction 2	; Return to the main loop

inst58		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ROLB		59	Inherent	2	1	-aaas

Alignment

inst59		proc	private

		Clocktime 2

		mov	ah, dh			; Copy our flags

		and	dh, 0FFh - ( OVERFLOW or CARRY )	; Unset overflow and carry

		mov	al, bl			; Copy B to AL

		shl	al, 1			; Shift to the left 1 bit
		adc	dh, 0			; If it carried, set the bit

		shl	bl, 1			; Shift B to the left 1 bit

		jno	noOverflow59		; Did we overflow?
		or	dh, OVERFLOW		; Yes, set it...

noOverflow59:
		and	ah, CARRY		; Was carry set prior to the ROL?
		jz	noPrevCarry59		; No, it wasn't
		or	bl, 01h			; Yes, set B's bit 1

noPrevCarry59:
		mov	al, bl			; Copy B to AL for setting the flags
		FlagSet				; Set the remaining flags.

		ExecuteNextInstruction 2	; Return to the main loop

inst59		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; DECB		5A	Inherent	2	1	-aaa-

Alignment

inst5A		proc	private

		Clocktime	2

		and	dh, 0ffh - (OVERFLOW)	; Zero out overflow

		dec	bl			; Subtract 1 from B
		jno	noOverflow5A		; Did we overflow?
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflow5A:
		mov	al, bl			; Copy B to AL for setting the flags
		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

inst5A		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; INCB		5C	Inherent	2	1	-aaa-

Alignment

inst5C		proc	private

		Clocktime	2

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		inc	bl			; Add 1 to B
		jno	noOverflow5C		; Did we overflow?
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflow5C:
		mov	al, bl			; Copy B to AL for setting the flags
		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

inst5C		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; TSTB		5D	Inherent	2	1	-aa0-

Alignment

inst5D		proc	private

		Clocktime	2

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		mov	al, bl			; Copy B to AL for setting the flags
		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

inst5D		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CLRB		5F	Direct		2	1	-0100

Alignment

inst5F		proc	private

		Clocktime	2

		and	dh, 0f0h		; Unset negative, zero, overflow and carry
		or	dh, ZERO		; Set zero

		xor	bl, bl			; Clear out B

		ExecuteNextInstruction 2	; Return to the main loop

inst5F		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; NEG		60	Indexed		6	2	uaaaa

Alignment

inst60		proc	private

		Clocktime	6

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		cmp	al, 80h			; Are we gonna overflow?
		jne	noOverflow60		; No, skip down...
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflow60:
		neg	al			; Negate our data

		adc	dh, 0			; If it carried, set the bit
		FlagSet				; Set the remaining flags

		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst60		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; COM		63	Indexed		6	2	-aa01

Alignment

inst63		proc	private

		Clocktime	6

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		xor	al, 0ffh		; Take the 1's complement

		or	dh, CARRY		; Set the carry bit
		and	dh, 0ffh - OVERFLOW	; There's no overflow
		FlagSet				; Set the remaining flags.

		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst63		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LSR		64	Indexed		6	2	-0a-s

Alignment

inst64		proc	private

		Clocktime 6

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		and	dh, 0ffh - (NEGATIVE or CARRY)	; There's no negative
		shr	al, 1			; Shift to the right 1 bit

		adc	dh, 0			; If it carried, set the bit
		FlagSet				; Set the remaining flags.

		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst64		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ROR		66	Indexed		6	2	-aa-s

Alignment

inst66		proc	private

		Clocktime 6

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		mov	ah, dh			; Copy our flags
		and	dh, 0FFh - CARRY	; Unset carry
		shr	al, 1			; Shift to the right 1 bit

		adc	dh, 0			; If it carried, set the bit

		and	ah, CARRY		; Check to see if carry was previously set
		jz	noPrevCarry66		; Was it set prior to the ROR?
		or	al, 080h		; Ok, set bit #7 then...

noPrevCarry66:
		FlagSet				; Set the remaining flags.

		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst66		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ASR		67	Indexed		6	2	uaa-s

Alignment

inst67		proc	private

		Clocktime 6

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		and	dh, 0FFh - CARRY	; Unset carry
		FlagSet				; Set the negative before the shift

		shr	al, 1			; Shift to the right 1 bit

		adc	dh, 0			; If it carried, set the bit

		mov	ah, dh			; Copy our flags
		and	ah, NEGATIVE		; Is negative set?
		jz	noNegative67		; No, we're not negative...
		or	al, 080h		; Yes we are, set bit #7 then...

noNegative67:
		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst67		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LSL/ASL	68	Indexed		6	2	naaas

Alignment

inst68		proc	private

		Clocktime 6

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		and	dh, 0FFh - ( OVERFLOW or CARRY )	; Unset overflow and carry

		mov	ah, al			; Make a copy

		shl	al, 1			; Shift to the left 1 bit
		adc	dh, 0			; If it carried, set the bit

		shl	ah, 1			; Shift the copy
		jno	noOverflow68		; Did we overflow?
		or	dh, OVERFLOW		; Yes, set it...

noOverflow68:
		FlagSet				; Set the remaining flags

		shl	edx, 16			; Protect DX

		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst68		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ROL		69	Indexed		6	2	-aaas

Alignment

inst69		proc	private

		Clocktime 6

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		shl	ebx, 16			; Protect BX
		shl	ecx, 16			; Protect CX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		mov	cl, al			; Save our data
		mov	ah, dh			; Copy our flags

		and	dh, 0FFh - ( OVERFLOW or CARRY )	; Unset overflow and carry
		shl	al, 1			; Shift to the right 1 bit

		shl	cl, 1			; Check the data again...
		adc	dh, 0			; If it carried, set the bit

		jno	noOverflow69		; Did we overflow?
		or	dh, OVERFLOW		; Yes, set it...

noOverflow69:
		and	ah, CARRY		; Was carry set prior to the ROL?
		jz	noPrevCarry69		; No, it wasn't
		or	al, 001h		; Yes, set bit 1

noPrevCarry69:
		FlagSet				; Set the remaining flags

		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ecx, 16			; Restore CX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst69		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; DEC		6A	Indexed		6	2	-aaa-

Alignment

inst6A		proc	private

		Clocktime	6

		and	dh, 0ffh - (OVERFLOW)	; Zero out overflow

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		dec	al			; Subtract 1
		jno	noOverflow6A		; Did we overflow?
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflow6A:
		FlagSet				; Set the remaining flags

		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst6A		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; INC		6C	Indexed		6	2	-aaa-

Alignment

inst6C		proc	private

		Clocktime	6

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		cmp	al, 07Fh		; Are we gonna overflow?
		jne	noOverflow6C		; No, skip down...
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflow6C:
		inc	al			; Add 1

		FlagSet				; Set the remaining flags

		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst6C		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; TST		6D	Indexed		6	2	-aa0-

Alignment

inst6D		proc	private

		Clocktime	6

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryByte			; Go get our data

		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 6	; Return to the main loop

inst6D		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; JMP		6E	Indexed		3	2	-----

Alignment

inst6E		proc	private

		Clocktime	3

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from

		mov	esi, eax		; Move in our new address
		add	esi, ebp		; Add in the base offset

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 3	; Return to the main loop

inst6E		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CLR		6F	Indexed		6	2	-0100

Alignment

inst6F		proc	private

		Clocktime	6

		and	dh, 0f0h		; Unset negative, zero, overflow and carry
		or	dh, ZERO		; Set zero

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		shl	edx, 16			; Protect DX and clear DL

		WriteMemoryByte			; Write out zero

		shr	edx, 16			; Restore DX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst6F		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; NEG		70	Extended	7	3	uaaaa

Alignment

inst70		proc	private

		Clocktime	7

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionWord		; Get our address for the data fetch
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		cmp	al, 80h			; Are we gonna overflow?
		jne	noOverflow70		; No, skip down...
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflow70:
		neg	al			; Negate our data

		adc	dh, 0			; If it carried, set the bit
		FlagSet				; Set the remaining flags

		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

inst70		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; COM		73	Extended	7	3	-aa01

Alignment

inst73		proc	private

		Clocktime	7

		FetchInstructionWord		; Get our address for the data fetch
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		xor	al, 0ffh		; Take the 1's complement

		or	dh, CARRY		; Set the carry bit
		and	dh, 0ffh - OVERFLOW	; There's no overflow
		FlagSet				; Set the remaining flags.

		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

inst73		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LSR		74	Extended	7	3	-0a-s

Alignment

inst74		proc	private

		Clocktime 7

		FetchInstructionWord		; Get our address for the data fetch
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		and	dh, 0ffh - (NEGATIVE or CARRY)	; There's no negative
		shr	al, 1			; Shift to the right 1 bit

		adc	dh, 0			; If it carried, set the bit
		FlagSet				; Set the remaining flags.

		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

inst74		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ROR		76	Extended	7	3	-aa-s

Alignment

inst76		proc	private

		Clocktime 7

		FetchInstructionWord		; Get our address for the data fetch
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		mov	ah, dh			; Copy our flags
		and	dh, 0FFh - (CARRY)	; Unset carry
		shr	al, 1			; Shift to the right 1 bit

		adc	dh, 0			; If it carried, set the bit

		and	ah, CARRY		; Check to see if carry was previously set
		jz	noPrevCarry76		; Was it set prior to the ROR?
		or	al, 080h		; Ok, set bit #7 then...

noPrevCarry76:
		FlagSet				; Set the remaining flags.

		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

inst76		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ASR		77	Extended	7	3	uaa-s

Alignment

inst77		proc	private

		Clocktime 7

		FetchInstructionWord		; Get our address for the data fetch
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		and	dh, 0FFh - CARRY	; Unset carry
		FlagSet				; Set the negative before the shift

		shr	al, 1			; Shift to the right 1 bit

		adc	dh, 0			; If it carried, set the bit

		mov	ah, dh			; Copy our flags
		and	ah, NEGATIVE		; Is negative set?
		jz	noNegative77		; No, we're not negative...
		or	al, 080h		; Yes we are, set bit #7 then...

noNegative77:
		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

inst77		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LSL/ASL	78	Extended	7	3	naaas

Alignment

inst78		proc	private

		Clocktime 7

		FetchInstructionWord		; Get our address for the data fetch
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		and	dh, 0FFh - ( OVERFLOW or CARRY )	; Unset overflow and carry

		mov	ah, al			; Make a copy
		shl	al, 1			; Shift to the left 1 bit

		adc	dh, 0			; If it carried, set the bit

		shl	ah, 1			; Shift the copy
		jno	noOverflow78		; Did we overflow?
		or	dh, OVERFLOW		; Yes, set it...

noOverflow78:
		FlagSet				; Set the remaining flags

		shl	edx, 16			; Protect DX

		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

inst78		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ROL		79	Extended	7	3	-aaas

Alignment

inst79		proc	private

		Clocktime 7

		FetchInstructionWord		; Get our address for the data fetch
		shl	ebx, 16			; Protect BX
		shl	ecx, 16			; Protect CX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		mov	ah, dh			; Copy our flags
		mov	cl, al			; Save our data...

		and	dh, 0FFh - ( OVERFLOW or CARRY )	; Unset overflow and carry

		shl	al, 1			; Shift to the right 1 bit
		adc	dh, 0			; If it carried, set the bit

		shl	cl, 1			; Check the data again...
		jno	noOverflow79		; Did we overflow?
		or	dh, OVERFLOW		; Yes, set it...

noOverflow79:
		and	ah, CARRY		; Was carry set prior to the ROL?
		jz	noPrevCarry79		; No, it wasn't
		or	al, 001h		; Yes, set bit 1

noPrevCarry79:
		FlagSet				; Set the remaining flags

		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ecx, 16			; Restore CX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

inst79		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; DEC		7A	Extended	7	3	-aaa-

Alignment

inst7A		proc	private

		Clocktime	7

		and	dh, 0ffh - (OVERFLOW)	; Zero out overflow

		FetchInstructionWord		; Get our address for the data fetch
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		dec	al			; Subtract 1
		jno	noOverflow7A		; Did we overflow?
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflow7A:
		FlagSet				; Set the remaining flags

		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

inst7A		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; INC		7C	Extended	7	3	-aaa-

Alignment

inst7C		proc	private

		Clocktime	7

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord		; Get our address for the data fetch
		shl	ebx, 16			; Protect BX
		mov	bx, ax			; Save our address
		ReadMemoryByte			; Go get our data

		cmp	al, 07Fh		; Are we gonna overflow?
		jne	noOverflow7C		; No, skip down...
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflow7C:
		inc	al			; Add 1

		FlagSet				; Set the remaining flags

		shl	edx, 16			; Protect DX
		mov	dl, al			; Put the byte to write in DL
		mov	ax, bx			; Put the address to write to in AX

		WriteMemoryByte			; Write the data back out

		shr	edx, 16			; Restore DX
		shr	ebx, 16			; Restore BX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

inst7C		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; TST		7D	Extended	7	3	-aa0-

Alignment

inst7D		proc	private

		Clocktime	7

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord		; Get our address for the data fetch
		ReadMemoryByte			; Go get our data

		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 6	; Return to the main loop

inst7D		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; JMP		7E	Extended	3	3	-----

Alignment

inst7E		proc	private

		Clocktime	3

		FetchInstructionWord		; Get our address for the data fetch

		xor	esi, esi		; Zero out ESI
		mov	si, ax			; Move in our new address
		add	esi, ebp		; Add in the base offset

		xor	ah, ah			; Zero AH for later

		ExecuteNextInstruction 3	; Return to the main loop

inst7E		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CLR		7F	Extended	7	3	-0100

Alignment

inst7F		proc	private

		Clocktime	7

		and	dh, 0f0h		; Unset negative, zero, overflow and carry
		or	dh, ZERO		; Set zero

		FetchInstructionWord		; Get our address for the data fetch
		push	dx			; Save DX
		xor	dx, dx			; Clear DL

		WriteMemoryByte			; Write out zero

		pop	dx			; Restore DX

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

inst7F		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SUBA		80	Immediate	2	2	uaaaa

Alignment

inst80		proc	private

		Clocktime	2

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionByte		; Get our BYTE to subtract

		mov	ah, bh			; Make a copy of A
		sub	ah, al			; Subtract it...
		jno	noOverflow80		; No, skip down...
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflow80:
		sub	bh, al			; Now really subtract it
		adc	dh, 0			; If it carried, set the bit

		mov	al, bh			; Copy A to AL
		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

inst80		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPA		81	Immediate	2	2	uaaaa

Alignment

inst81		proc	private

		Clocktime	2

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionByte		; Get our BYTE to compare

		cmp	bh, al			; Compare it
		jno	noOverflow81		; No, skip down...
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflow81:
		mov	ah, al
		mov	al, bh
		sub	al, ah			; Do the compare (via sub)
		adc	dh, 0			; If it carried, set the bit

		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

inst81		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SBCA		82	Immediate	2	2	uaaaa

Alignment

inst82		proc	private

		Clocktime	2

		shl	ecx, 16			; Save CX
		mov	ch, dh			; Save original flags in CH
		and	ch, CARRY		; Isolate carry

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero 'em out

		FetchInstructionByte		; Get our BYTE to subtract

						; Check for OVERFLOW

		mov	ah, bh			; Put A in AH
		mov	cl, dh			; Put flags into CL
		shr	cl, 1			; Put carry bit into X86 carry

		sbb	ah, al			; Subtract and carry
		
		jno	noOverflow82		; Nope, it wasn't set
		or	dh, OVERFLOW		; Yes it was set, set overflow

						; Subtract it and check for CARRY
noOverflow82:
		sub	bh, al			; Now really subtract it
		adc	dh, 0			; If it carried, set the bit

		sub	bh, ch			; Subtract previous carry for A (or not)
		adc	dh, 0			; If it carried, set the bit

		mov	al, bh			; Copy A to AL	
		FlagSet				; Set the remaining flags

		shr	ecx, 16			; Restore CX
		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 2	; Return to the main loop

inst82		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SUBD		83	Immediate	4	3	-aaaa

Alignment

inst83		proc	private

		Clocktime	4

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionWord		; Get our WORD to subtract

		shl	ecx, 16			; Save CX
		mov	cx, bx			; Copy D to CX

		sub	cx, ax			; Subtract from D

		jno	noOverflow83		; No overflow, skip down...
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflow83:
		sub	bx, ax			; Now really subtract it
		adc	dh, 0			; If it carried, set the bit

		FlagSetD			; Set the remaining flags

		shr	ecx, 16			; Restore CX
		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 4	; Return to the main loop

inst83		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ANDA		84	Immediate	2	2	-aa0-

Alignment

inst84		proc	private

		Clocktime	2

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte		; Get our BYTE to AND

		and	bh, al			; Do the AND

		mov	al, bh			; Copy A to AL
		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

inst84		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BITA		85	Immediate	2	2	-aa0-

Alignment

inst85		proc	private

		Clocktime	2

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte		; Get our BYTE to AND

		and	al, bh			; Do the AND

		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

inst85		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDA		86	Immediate	2	2	-aa0-

Alignment

inst86		proc	private

		Clocktime	2

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte		; Get our BYTE to load into A

		mov	bh, al			; Load it...

		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

inst86		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; EORA		88	Immediate	2	2	-aa0-

Alignment

inst88		proc	private

		Clocktime	2

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte		; Get our BYTE to XOR with A

		xor	bh, al			; XOR it...

		mov	al, bh			; Copy A to AL
		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

inst88		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ADCA		89	Immediate	2	2	aaaaa

Alignment

inst89		proc	private

		Clocktime	2

		and	dh, 0ffh - (HALFCARRY or OVERFLOW)	; Zero 'em out

		FetchInstructionByte		; Get our BYTE to add

		shl	ecx, 16			; Save CX

						; Check for OVERFLOW

		mov	ah, bh			; Put A in AH
		mov	ch, al			; Put our data in CH
		mov	cl, dh			; Put our flags in CL
		shr	cl, 1			; Put carry in X86 carry

		adc	ah, ch			; Add data and carry to A

		jno	noOverflow89		; Nope, it wasn't set
		or	dh, OVERFLOW		; Yes it was set, set overflow

						; Check for HALFCARRY
noOverflow89:
		mov	ah, bh			; Put A in AH
		mov	ch, al			; Put our data in CH
		mov	cl, dh			; Put our flags in CL
		shr	cl, 1			; Put carry in X86 carry

		adc	ah, ch			; Add data and carry to A

		lahf				; Load the flags into AH
		test	ah, 010h		; Is the half-carry set?

		jz	noHalfcarry89		; Nope, it wasn't set
		or	dh, HALFCARRY		; Yes it was set, set halfcarry

						; Do the ADD and check for CARRY
noHalfcarry89:
		mov	ch, dh			; We need the old carry...
		and	ch, CARRY		; Isolate it
		and	dh, 0ffh - (CARRY)	; Zero out carry

		add	bh, al			; Now really add it to A
		adc	dh, 0			; If it carried, set the bit

		add	bh, ch			; Add in the previous carry (or not)
		adc	dh, 0			; If it carried, set the bit

		mov	al, bh			; Copy A to AL
		FlagSet				; Set the remaining flags

		shr	ecx, 16			; Restore CX

		ExecuteNextInstruction 2	; Return to the main loop

inst89		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ORA		8A	Immediate	2	2	-aa0-

Alignment

inst8A		proc	private

		Clocktime	2

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte		; Get our BYTE to AND

		or	bh, al			; Do the OR

		mov	al, bh			; Copy A to AL
		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

inst8A		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ADDA		8B	Immediate	2	2	aaaaa

Alignment

inst8B		proc	private

		Clocktime	2

		and	dh, 0ffh - (HALFCARRY or OVERFLOW or CARRY)	; Zero 'em out

		FetchInstructionByte		; Get our BYTE to add

		shl	ecx, 16			; Save CX
	
						; Check for OVERFLOW

		mov	ah, bh			; Put A in AH
		add	ah, al			; Add data to A

		jno	noOverflow8B		; Nope, it wasn't set
		or	dh, OVERFLOW		; Yes it was set, set overflow

						; Check for HALFCARRY
noOverflow8B:
		mov	ah, bh			; Put A in AH
		add	ah, al			; Add our data

		lahf				; Load the flags into AH
		test	ah, 010h		; Is the half-carry set?

		jz	noHalfcarry8B		; Nope, it wasn't set
		or	dh, HALFCARRY		; Yes it was set, set halfcarry

						; Do the ADD and check for CARRY
noHalfcarry8B:
		add	bh, al			; Now really add it to A
		adc	dh, 0			; If it carried, set the bit

		mov	al, bh			; Copy A to AL
		FlagSet				; Set the remaining flags

		shr	ecx, 16			; Restore CX

		ExecuteNextInstruction 2	; Return to the main loop

inst8B		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPX		8C	Immediate	4	3	-aaaa

Alignment

inst8C		proc	private

		Clocktime	4

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero 'em out

		FetchInstructionWord		; Get our WORD to compare

		cmp	cx, ax			; Compare data to X

		jno	noOverflow8C		; Nope, it wasn't set
		or	dh, OVERFLOW		; Yes it was set, set overflow

noOverflow8C:
		shl	ebx, 16			; Save BX

		mov	bx, ax			; Put our data in BX
		mov	ax, cx			; Put X in AX
		sub	ax, bx			; Follow through on the CMP

		adc	dh, 0			; If it carried, set the bit

		FlagSetWord			; Set the remaining flags

		shr	ebx, 16			; Restore BX
		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 4	; Return to the main loop

inst8C		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BSR		8D	Relative	7	2	-----

Alignment

inst8D		proc	private

		Clocktime	7

		FetchInstructionByte		; Get out offset...
		push	ax			; Save AX

		sub	esi, ebp		; Get our PC into SI
		shl	edx, 16			; Save DX
		mov	dx, si			; Copy the PC into DX
		add	esi, ebp		; Add the base back in...

		LoadStackS			; Load stack pointer S into AX
		PushStackWord			; Push DX onto the stack...
		UnloadStackS			; Unload stack pointer from AX

		shr	edx, 16			; Restore DX

		pop	ax			; Restore AX
		cbw				; Convert AL to AX
		sub	esi, ebp		; Get the PC in SI again
		add	si, ax			; Add it in
		add	esi, ebp		; Add back in our base

		xor	ah, ah			; Zero upper part of AX

		ExecuteNextInstruction 7	; Return to the main loop

inst8D		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDX		8E	Immediate	3	3	-aa0-

Alignment

inst8E		proc	private

		Clocktime	3

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord		; Get our WORD to load into AX

		mov	cx, ax			; Load it...

		FlagSetWord			; Set the remaining flags

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 3	; Return to the main loop

inst8E		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SUBA		90	Direct		4	2	uaaaa

Alignment

inst90		proc	private

		Clocktime	4

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchDirectWord			; Get our address for the data fetch
		ReadMemoryByte			; Go get our data

		mov	ah, bh			; Make a copy of A
		sub	ah, al			; Subtract it...
		jno	noOverflow90		; No, skip down...
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflow90:
		sub	bh, al			; Now really subtract it
		adc	dh, 0			; If it carried, set the bit

		mov	al, bh			; Copy A to AL
		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

inst90		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPA		91	Direct		4	2	uaaaa

Alignment

inst91		proc	private

		Clocktime	4

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchDirectWord			; Get our address for the data fetch
		ReadMemoryByte			; Go get our data

		cmp	bh, al			; Compare it
		jno	noOverflow91		; No, skip down...
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflow91:
		mov	ah, al
		mov	al, bh
		sub	al, ah			; Do the compare (via sub)
		adc	dh, 0			; If it carried, set the bit

		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

inst91		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SBCA		92	Immediate	4	2	uaaaa

Alignment

inst92		proc	private

		Clocktime	4

		shl	ecx, 16			; Save CX
		mov	ch, dh			; Save original flags in CH
		and	ch, CARRY		; Isolate carry

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero 'em out

		FetchDirectWord			; Get our address for the data fetch
		ReadMemoryByte			; Go get our data

						; Check for OVERFLOW

		mov	ah, bh			; Put A in AH
		mov	cl, dh			; Put flags into CL
		shr	cl, 1			; Put carry bit into X86 carry

		sbb	ah, al			; Subtract and carry
		
		jno	noOverflow92		; Nope, it wasn't set
		or	dh, OVERFLOW		; Yes it was set, set overflow

						; Subtract it and check for CARRY
noOverflow92:
		sub	bh, al			; Now really subtract it
		adc	dh, 0			; If it carried, set the bit

		sub	bh, ch			; Subtract previous carry for A (or not)
		adc	dh, 0			; If it carried, set the bit

		mov	al, bh			; Copy A to AL	
		FlagSet				; Set the remaining flags

		shr	ecx, 16			; Restore CX
		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 4	; Return to the main loop

inst92		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SUBD		93	Direct		6	2	-aaaa

Alignment

inst93		proc	private

		Clocktime	6

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchDirectWord			; Get our address for the data fetch
		ReadMemoryWord			; Go get our data

		shl	ecx, 16			; Save CX
		mov	cx, bx			; Copy D to CX

		sub	cx, ax			; Subtract from D

		jno	noOverflow93		; No overflow, skip down...
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflow93:
		sub	bx, ax			; Now really subtract it
		adc	dh, 0			; If it carried, set the bit

		FlagSetD			; Set the remaining flags

		shr	ecx, 16			; Restore CX
		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst93		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ANDA		94	Direct		4	2	-aa0-

Alignment

inst94		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord			; Get our address for the data fetch
		ReadMemoryByte			; Go get our data

		and	bh, al			; Do the AND

		mov	al, bh			; Copy A to AL
		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

inst94		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BITA		95	Direct		4	2	-aa0-

Alignment

inst95		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord			; Get our address for the data fetch
		ReadMemoryByte			; Go get our data

		and	al, bh			; Do the AND

		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

inst95		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDA		96	Direct		4	2	-aa0-

Alignment

inst96		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord			; Get our address for the data fetch
		ReadMemoryByte			; Go get our data

		mov	bh, al			; Load it...

		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

inst96		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; STA		97	Direct		4	2	-aa0-

Alignment

inst97		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord			; Get our address for the data fetch

		shl	edx, 16			; Protect DX
		mov	dl, bh			; Put the A in DL

		WriteMemoryByte			; Write the data out

		shr	edx, 16			; Restore DX

		mov	al, bh			; Copy A to AL
		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

inst97		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; EORA		98	Direct		4	2	-aa0-

Alignment

inst98		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord			; Get our address for the data fetch
		ReadMemoryByte			; Go get our data

		xor	bh, al			; XOR it...

		mov	al, bh			; Copy A to AL
		FlagSet				; Set the remaining flags


		ExecuteNextInstruction 4	; Return to the main loop

inst98		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ADCA		99	Direct		4	2	aaaaa

Alignment

inst99		proc	private

		Clocktime	4

		and	dh, 0ffh - (HALFCARRY or OVERFLOW)	; Zero 'em out

		FetchDirectWord			; Get our address for the data fetch
		ReadMemoryByte			; Go get our data

		shl	ecx, 16			; Save CX

						; Check for OVERFLOW

		mov	ah, bh			; Put A in AH
		mov	ch, al			; Put our data in CH
		mov	cl, dh			; Put our flags in CL
		shr	cl, 1			; Put carry in X86 carry

		adc	ah, ch			; Add data and carry to A

		jno	noOverflow99		; Nope, it wasn't set
		or	dh, OVERFLOW		; Yes it was set, set overflow

						; Check for HALFCARRY
noOverflow99:
		mov	ah, bh			; Put A in AH
		mov	ch, al			; Put our data in CH
		mov	cl, dh			; Put our flags in CL
		shr	cl, 1			; Put carry in X86 carry

		adc	ah, ch			; Add data and carry to A

		lahf				; Load the flags into AH
		test	ah, 010h		; Is the half-carry set?

		jz	noHalfcarry99		; Nope, it wasn't set
		or	dh, HALFCARRY		; Yes it was set, set halfcarry

						; Do the ADD and check for CARRY
noHalfcarry99:
		mov	ch, dh			; We need the old carry...
		and	ch, CARRY		; Isolate it
		and	dh, 0ffh - (CARRY)	; Zero out carry

		add	bh, al			; Now really add it to A
		adc	dh, 0			; If it carried, set the bit

		add	bh, ch			; Add in the previous carry (or not)
		adc	dh, 0			; If it carried, set the bit

		mov	al, bh			; Copy A to AL
		FlagSet				; Set the remaining flags

		shr	ecx, 16			; Restore CX

		ExecuteNextInstruction 4	; Return to the main loop

inst99		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ORA		9A	Direct		4	2	-aa0-

Alignment

inst9A		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord			; Get our address for the data fetch
		ReadMemoryByte			; Go get our data

		or	bh, al			; Do the OR

		mov	al, bh			; Copy A to AL
		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

inst9A		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ADDA		9B	Direct		4	2	aaaaa

Alignment

inst9B		proc	private

		Clocktime	4

		and	dh, 0ffh - (HALFCARRY or OVERFLOW or CARRY)	; Zero 'em out

		FetchDirectWord			; Get our address for the data fetch
		ReadMemoryByte			; Go get our data

		shl	ecx, 16			; Save CX
	
						; Check for OVERFLOW

		mov	ah, bh			; Put A in AH
		add	ah, al			; Add data to A

		jno	noOverflow9B		; Nope, it wasn't set
		or	dh, OVERFLOW		; Yes it was set, set overflow

						; Check for HALFCARRY
noOverflow9B:
		mov	ah, bh			; Put A in AH
		add	ah, al			; Add data to A

		lahf				; Load the flags into AH
		test	ah, 010h		; Is the half-carry set?

		jz	noHalfcarry9B		; Nope, it wasn't set
		or	dh, HALFCARRY		; Yes it was set, set halfcarry

						; Do the ADD and check for CARRY
noHalfcarry9B:
		add	bh, al			; Now really add it to A
		adc	dh, 0			; If it carried, set the bit

		mov	al, bh			; Copy A to AL
		FlagSet				; Set the remaining flags

		shr	ecx, 16			; Restore CX

		ExecuteNextInstruction 4	; Return to the main loop

inst9B		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPX		9C	Direct		6	2	-aaaa

Alignment

inst9C		proc	private

		Clocktime	6

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero 'em out

		FetchDirectWord			; Get our address for the data fetch
		ReadMemoryWord			; Go get our data

		cmp	cx, ax			; Compare data to X

		jno	noOverflow9C		; Nope, it wasn't set
		or	dh, OVERFLOW		; Yes it was set, set overflow

noOverflow9C:
		shl	ebx, 16			; Save BX
		
		mov	bx, ax			; Put our data in BX
		mov	ax, cx			; Put X in AX
		sub	ax, bx			; Follow through on the CMP

		adc	dh, 0			; If it carried, set the bit

		FlagSetWord			; Set the remaining flags

		shr	ebx, 16			; Restore BX
		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst9C		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; JSR		9D	Direct		7	2	-----

Alignment

inst9D		proc	private

		Clocktime	7

		FetchDirectWord			; Get our address for the data fetch
		ReadMemoryWord			; Go get our data

		push	ax			; Save AX

		sub	esi, ebp		; Get our PC into SI
		shl	edx, 16			; Save DX
		mov	dx, si			; Copy the PC into DX
		add	esi, ebp		; Add in our base

		LoadStackS			; Load stack pointer S into AX
		PushStackWord			; Push DX onto the stack...
		UnloadStackS			; Unload stack pointer from AX

		shr	edx, 16			; Restore DX

		xor	esi, esi		; Clear out ESI
		pop	ax			; Restore our jump value
		mov	si, ax			; Add it in
		add	esi, ebp		; Add back in our base

		xor	ah, ah			; Zero upper part of AX

		ExecuteNextInstruction 7	; Return to the main loop

inst9D		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDX		9E	Direct		5	2	-aa0-

Alignment

inst9E		proc	private

		Clocktime	5

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord			; Get our address for the data fetch
		ReadMemoryWord			; Go get our data

		mov	cx, ax			; Load it...

		FlagSetWord			; Set the remaining flags

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 5	; Return to the main loop

inst9E		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; STX		9F	Direct		5	2	-aa0-

Alignment

inst9F		proc	private

		Clocktime	5

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord			; Get our address for the data fetch

		shl	edx, 16			; Protect DX
		mov	dx, cx			; Put X in DX

		WriteMemoryWord			; Write the data out

		shr	edx, 16			; Restore DX

		FlagSetX			; Set the remaining flags

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 5	; Return to the main loop

inst9F		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SUBA		A0	Indexed		4	2	uaaaa

Alignment

instA0		proc	private

		Clocktime	4

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryByte			; Go get our data

		mov	ah, bh			; Make a copy of A
		sub	ah, al			; Subtract it...
		jno	noOverflowA0		; No, skip down...
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflowA0:
		sub	bh, al			; Now really subtract it
		adc	dh, 0			; If it carried, set the bit

		mov	al, bh			; Copy A to AL
		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instA0		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPA		A1	Indexed		4	2	uaaaa

Alignment

instA1		proc	private

		Clocktime	4

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryByte			; Go get our data

		cmp	bh, al			; Compare it
		jno	noOverflowA1		; No, skip down...
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflowA1:
		mov	ah, al
		mov	al, bh
		sub	al, ah			; Do the compare (via sub)
		adc	dh, 0			; If it carried, set the bit

		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instA1		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SBCA		A2	Indexed		4	2	uaaaa

Alignment

instA2		proc	private

		Clocktime	4

		shl	ecx, 16			; Save CX
		mov	ch, dh			; Save original flags in CH
		and	ch, CARRY		; Isolate carry

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero 'em out

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryByte			; Go get our data

						; Check for OVERFLOW

		mov	ah, bh			; Put A in AH
		mov	cl, dh			; Put flags into CL
		shr	cl, 1			; Put carry bit into X86 carry

		sbb	ah, al			; Subtract and carry
		
		jno	noOverflowA2		; Nope, it wasn't set
		or	dh, OVERFLOW		; Yes it was set, set overflow

						; Subtract it and check for CARRY
noOverflowA2:
		sub	bh, al			; Now really subtract it
		adc	dh, 0			; If it carried, set the bit

		sub	bh, ch			; Subtract previous carry for A (or not)
		adc	dh, 0			; If it carried, set the bit

		mov	al, bh			; Copy A to AL	
		FlagSet				; Set the remaining flags

		shr	ecx, 16			; Restore CX

		ExecuteNextInstruction 4	; Return to the main loop

instA2		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SUBD		A3	Indexed		6	2	-aaaa

Alignment

instA3		proc	private

		Clocktime	6

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryWord			; Go get our data

		shl	ecx, 16			; Save CX
		mov	cx, bx			; Copy D to CX

		sub	cx, ax			; Subtract from D

		jno	noOverflowA3		; No overflow, skip down...
		or	dh, OVERFLOW		; Yes, set the overflow bit

noOverflowA3:
		sub	bx, ax			; Now really subtract it
		adc	dh, 0			; If it carried, set the bit

		FlagSetD			; Set the remaining flags

		shr	ecx, 16			; Restore CX
		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

instA3		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ANDA		A4	Indexed		4	2	-aa0-

Alignment

instA4		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryByte			; Go get our data

		and	bh, al			; Do the AND

		mov	al, bh			; Copy A to AL
		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instA4		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BITA		A5	Indexed		4	2	-aa0-

Alignment

instA5		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryByte			; Go get our data

		and	al, bh			; Do the AND

		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instA5		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDA		A6	Indexed		4	2	-aa0-

Alignment

instA6		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryByte			; Go get our data

		mov	bh, al			; Load it...

		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instA6		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; STA		A7	Indexed		4	2	-aa0-

Alignment

instA7		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from

		shl	edx, 16			; Protect DX
		mov	dl, bh			; Put the A in DL

		WriteMemoryByte			; Write the data out

		shr	edx, 16			; Restore DX

		mov	al, bh			; Copy A to AL
		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instA7		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; EORA		A8	Indexed		4	2	-aa0-

Alignment

instA8		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryByte			; Go get our data

		xor	bh, al			; XOR it...

		mov	al, bh			; Copy A to AL
		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instA8		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ADCA		A9	Indexed		4	2	aaaaa

Alignment

instA9		proc	private

		Clocktime	4

		and	dh, 0ffh - (HALFCARRY or OVERFLOW)	; Zero 'em out

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryByte			; Go get our data

		shl	ecx, 16			; Save CX

						; Check for OVERFLOW

		mov	ah, bh			; Put A in AH
		mov	ch, al			; Put our data in CH
		mov	cl, dh			; Put our flags in CL
		shr	cl, 1			; Put carry in X86 carry

		adc	ah, ch			; Add data and carry to A

		jno	noOverflowA9		; Nope, it wasn't set
		or	dh, OVERFLOW		; Yes it was set, set overflow

						; Check for HALFCARRY
noOverflowA9:
		mov	ah, bh			; Put A in AH
		mov	ch, al			; Put our data in CH
		mov	cl, dh			; Put our flags in CL
		shr	cl, 1			; Put carry in X86 carry

		adc	ah, ch			; Add data and carry to A

		lahf				; Load the flags into AH
		test	ah, 010h		; Is the half-carry set?

		jz	noHalfcarryA9		; Nope, it wasn't set
		or	dh, HALFCARRY		; Yes it was set, set halfcarry

						; Do the ADD and check for CARRY
noHalfcarryA9:
		mov	ch, dh			; We need the old carry...
		and	ch, CARRY		; Isolate it
		and	dh, 0ffh - (CARRY)	; Zero out carry

		add	bh, al			; Now really add it to A
		adc	dh, 0			; If it carried, set the bit

		add	bh, ch			; Add in the previous carry (or not)
		adc	dh, 0			; If it carried, set the bit

		mov	al, bh			; Copy A to AL
		FlagSet				; Set the remaining flags

		shr	ecx, 16			; Restore CX

		ExecuteNextInstruction 4	; Return to the main loop

instA9		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ORA		AA	Indexed		4	2	-aa0-

Alignment

instAA		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryByte			; Go get our data

		or	bh, al			; Do the OR

		mov	al, bh			; Copy A to AL
		FlagSet				; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instAA		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ADDA		AB	Indexed		4	2	aaaaa

Alignment

instAB		proc	private

		Clocktime	4

		and	dh, 0ffh - (HALFCARRY or OVERFLOW or CARRY)	; Zero 'em out

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryByte			; Go get our data

		shl	ecx, 16			; Save CX
	
						; Check for OVERFLOW

		mov	ah, bh			; Put A in AH
		add	ah, al			; Add data to A

		jno	noOverflowAB		; Nope, it wasn't set
		or	dh, OVERFLOW		; Yes it was set, set overflow

						; Check for HALFCARRY
noOverflowAB:
		mov	ah, bh			; Put A in AH
		add	ah, al			; Add data to A

		lahf				; Load the flags into AH
		test	ah, 010h		; Is the half-carry set?

		jz	noHalfcarryAB		; Nope, it wasn't set
		or	dh, HALFCARRY		; Yes it was set, set halfcarry

						; Do the ADD and check for CARRY
noHalfcarryAB:
		add	bh, al			; Now really add it to A
		adc	dh, 0			; If it carried, set the bit

		mov	al, bh			; Copy A to AL
		FlagSet				; Set the remaining flags

		shr	ecx, 16			; Restore CX

		ExecuteNextInstruction 4	; Return to the main loop

instAB		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPX		AC	Indexed		6	2	-aaaa

Alignment

instAC		proc	private

		Clocktime	6

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero 'em out

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryWord			; Go get our data

		cmp	cx, ax			; Compare data to X

		jno	noOverflowAC		; Nope, it wasn't set
		or	dh, OVERFLOW		; Yes it was set, set overflow

noOverflowAC:
		shl	ebx, 16			; Save BX
		
		mov	bx, ax			; Put our data in BX
		mov	ax, cx			; Put X in AX
		sub	ax, bx			; Follow through on the CMP

		adc	dh, 0			; If it carried, set the bit

		FlagSetWord			; Set the remaining flags

		shr	ebx, 16			; Restore BX
		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

instAC		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; JSR		AD	Indexed		7	2	-----

Alignment

instAD		proc	private

		Clocktime	7

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		push	ax			; Save AX

		sub	esi, ebp		; Get our PC into SI
		shl	edx, 16			; Save DX
		mov	dx, si			; Copy the PC into DX
		add	esi, ebp		; Add in our base

		LoadStackS			; Load stack pointer S into AX
		PushStackWord			; Push DX onto the stack...
		UnloadStackS			; Unload stack pointer from AX

		shr	edx, 16			; Restore DX

		xor	esi, esi		; Clear out ESI
		pop	ax			; Restore our jump value
		mov	si, ax			; Add it in
		add	esi, ebp		; Add back in our base

		xor	ah, ah			; Zero upper part of AX

		ExecuteNextInstruction 7	; Return to the main loop

instAD		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDX		AE	Indexed		5	2	-aa0-

Alignment

instAE		proc	private

		Clocktime	5

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryWord			; Go get our data

		mov	cx, ax			; Load it...

		FlagSetWord			; Set the remaining flags

		xor	ah, ah			; Zero out AH

		ExecuteNextInstruction 5	; Return to the main loop

instAE		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; STX		AF	Indexed		5	2	-aa0-

Alignment

instAF		proc	private

		Clocktime	5

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from

		shl	edx, 16			; Protect DX
		mov	dx, cx			; Put X in DX

		WriteMemoryWord			; Write the data out

		shr	edx, 16			; Restore DX

		FlagSetX			; Set the remaining flags

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 5	; Return to the main loop

instAF		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SUBA		B0	Extended	5	3	uaaaa

Alignment

instB0		proc	private

		Clocktime	5

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryByte		; Go get our data

		mov	ah, bh		; Make a copy of A
		sub	ah, al		; Subtract it...
		jno	noOverflowB0	; No, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflowB0:
		sub	bh, al		; Now really subtract it
		adc	dh, 0		; If it carried, set the bit

		mov	al, bh		; Copy A to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 5	; Return to the main loop

instB0		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPA		B1	Extended	5	3	uaaaa

Alignment

instB1		proc	private

		Clocktime	5

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryByte		; Go get our data

		cmp	bh, al		; Compare it
		jno	noOverflowB1	; No, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflowB1:
		mov	ah, al
		mov	al, bh
		sub	al, ah		; Do the compare (via sub)
		adc	dh, 0		; If it carried, set the bit

		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instB1		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SBCA		B2	Extended	5	3	uaaaa

Alignment

instB2		proc	private

		Clocktime	5

		shl	ecx, 16		; Save CX
		mov	ch, dh		; Save original flags in CH
		and	ch, CARRY	; Isolate carry

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero 'em out

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryByte		; Go get our data

					; Check for OVERFLOW

		mov	ah, bh		; Put A in AH
		mov	cl, dh		; Put flags into CL
		shr	cl, 1		; Put carry bit into X86 carry

		sbb	ah, al		; Subtract and carry
		
		jno	noOverflowB2	; Nope, it wasn't set
		or	dh, OVERFLOW	; Yes it was set, set overflow

					; Subtract it and check for CARRY
noOverflowB2:
		sub	bh, al		; Now really subtract it
		adc	dh, 0		; If it carried, set the bit

		sub	bh, ch		; Subtract previous carry for A (or not)
		adc	dh, 0		; If it carried, set the bit

		mov	al, bh		; Copy A to AL	
		FlagSet			; Set the remaining flags

		shr	ecx, 16		; Restore CX

		ExecuteNextInstruction 5	; Return to the main loop

instB2		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SUBD		B3	Extended	7	3	-aaaa

Alignment

instB3		proc	private

		Clocktime	7

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryWord		; Go get our data

		shl	ecx, 16		; Save CX
		mov	cx, bx		; Copy D to CX

		sub	cx, ax		; Subtract from D

		jno	noOverflowB3	; No overflow, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflowB3:
		sub	bx, ax		; Now really subtract it
		adc	dh, 0		; If it carried, set the bit

		FlagSetD		; Set the remaining flags

		shr	ecx, 16		; Restore CX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

instB3		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ANDA		B4	Extended	5	3	-aa0-

Alignment

instB4		proc	private

		Clocktime	5

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryByte		; Go get our data

		and	bh, al		; Do the AND

		mov	al, bh		; Copy A to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 5	; Return to the main loop

instB4		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BITA		B5	Extended	5	3	-aa0-

Alignment

instB5		proc	private

		Clocktime	5

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryByte		; Go get our data

		and	al, bh		; Do the AND

		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 5	; Return to the main loop

instB5		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDA		B6	Extended	5	3	-aa0-

Alignment

instB6		proc	private

		Clocktime	5

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryByte		; Go get our data

		mov	bh, al		; Load it...

		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 5	; Return to the main loop

instB6		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; STA		B7	Extended	5	3	-aa0-

Alignment

instB7		proc	private

		Clocktime	5

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord	; Get our address to read the data

		shl	edx, 16		; Protect DX
		mov	dl, bh		; Put the A in DL

		WriteMemoryByte		; Write the data out

		shr	edx, 16		; Restore DX

		mov	al, bh		; Copy A to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 5	; Return to the main loop

instB7		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; EORA		B8	Extended	5	3	-aa0-

Alignment

instB8		proc	private

		Clocktime	5

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryByte		; Go get our data

		xor	bh, al		; XOR it...

		mov	al, bh		; Copy A to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 5	; Return to the main loop

instB8		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ADCA		B9	Extended	5	3	aaaaa

Alignment

instB9		proc	private

		Clocktime	5

		and	dh, 0ffh - (HALFCARRY or OVERFLOW)	; Zero 'em out

		FetchInstructionWord		; Get our address to read the data
		ReadMemoryByte			; Go get our data

		shl	ecx, 16			; Save CX

						; Check for OVERFLOW

		mov	ah, bh			; Put A in AH
		mov	ch, al			; Put our data in CH
		mov	cl, dh			; Put our flags in CL
		shr	cl, 1			; Put carry in X86 carry

		adc	ah, ch			; Add data and carry to A

		jno	noOverflowB9		; Nope, it wasn't set
		or	dh, OVERFLOW		; Yes it was set, set overflow

						; Check for HALFCARRY
noOverflowB9:
		mov	ah, bh			; Put A in AH
		mov	ch, al			; Put our data in CH
		mov	cl, dh			; Put our flags in CL
		shr	cl, 1			; Put carry in X86 carry

		adc	ah, ch			; Add data and carry to A

		lahf				; Load the flags into AH
		test	ah, 010h		; Is the half-carry set?

		jz	noHalfcarryB9		; Nope, it wasn't set
		or	dh, HALFCARRY		; Yes it was set, set halfcarry

						; Do the ADD and check for CARRY
noHalfcarryB9:
		mov	ch, dh			; We need the old carry...
		and	ch, CARRY		; Isolate it
		and	dh, 0ffh - (CARRY)	; Zero out carry

		add	bh, al			; Now really add it to A
		adc	dh, 0			; If it carried, set the bit

		add	bh, ch			; Add in the previous carry (or not)
		adc	dh, 0			; If it carried, set the bit

		mov	al, bh			; Copy A to AL
		FlagSet				; Set the remaining flags

		shr	ecx, 16			; Restore CX

		ExecuteNextInstruction 5	; Return to the main loop

instB9		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ORA		BA	Extended	5	3	-aa0-

Alignment

instBA		proc	private

		Clocktime	5

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryByte		; Go get our data

		or	bh, al		; Do the OR

		mov	al, bh		; Copy A to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 5	; Return to the main loop

instBA		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ADDA		BB	Extended	5	3	aaaaa

Alignment

instBB		proc	private

		Clocktime	5

		and	dh, 0ffh - (HALFCARRY or OVERFLOW or CARRY)	; Zero 'em out

		FetchInstructionWord		; Get our address to read the data
		ReadMemoryByte			; Go get our data

		shl	ecx, 16			; Save CX
	
						; Check for OVERFLOW

		mov	ah, bh			; Put A in AH
		add	ah, al			; Add data to A

		jno	noOverflowBB		; Nope, it wasn't set
		or	dh, OVERFLOW		; Yes it was set, set overflow

						; Check for HALFCARRY
noOverflowBB:
		mov	ah, bh			; Put A in AH
		add	ah, al			; Add data to A

		lahf				; Load the flags into AH
		test	ah, 010h		; Is the half-carry set?

		jz	noHalfcarryBB		; Nope, it wasn't set
		or	dh, HALFCARRY		; Yes it was set, set halfcarry

						; Do the ADD and check for CARRY
noHalfcarryBB:
		add	bh, al			; Now really add it to A
		adc	dh, 0			; If it carried, set the bit

		mov	al, bh			; Copy A to AL
		FlagSet				; Set the remaining flags

		shr	ecx, 16			; Restore CX

		ExecuteNextInstruction 5	; Return to the main loop

instBB		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPX		BC	Extended	7	3	-aaaa

Alignment

instBC		proc	private

		Clocktime	7

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero 'em out

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryWord		; Go get our data

		cmp	cx, ax		; Compare data to X

		jno	noOverflowBC	; Nope, it wasn't set
		or	dh, OVERFLOW	; Yes it was set, set overflow

noOverflowBC:
		shl	ebx, 16		; Save BX
		
		mov	bx, ax		; Put our data in BX
		mov	ax, cx		; Put X in AX
		sub	ax, bx		; Follow through on the CMP

		adc	dh, 0		; If it carried, set the bit

		FlagSetWord		; Set the remaining flags

		shr	ebx, 16		; Restore BX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

instBC		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; JSR		BD	Extended	8	3	-----

Alignment

instBD		proc	private

		Clocktime	8

		FetchInstructionWord	; Get our address to read the data
		push	ax		; Save AX

		sub	esi, ebp	; Get our PC into SI
		shl	edx, 16		; Save DX
		mov	dx, si		; Copy the PC into DX
		add	esi, ebp	; Add in the base again

		LoadStackS		; Load stack pointer S into AX
		PushStackWord		; Push DX onto the stack...
		UnloadStackS		; Unload stack pointer from AX

		shr	edx, 16		; Restore DX

		xor	esi, esi	; Clear out ESI
		pop	ax		; Restore our jump value
		mov	si, ax		; Add it in
		add	esi, ebp	; Add back in our base

		xor	ah, ah		; Zero upper part of AX

		ExecuteNextInstruction 8	; Return to the main loop

instBD		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDX		BE	Extended	6	3	-aa0-

Alignment

instBE		proc	private

		Clocktime	6

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryWord		; Go get our data

		mov	cx, ax		; Load it...

		FlagSetWord		; Set the remaining flags

		xor	ah, ah			; Zero out AH

		ExecuteNextInstruction 6	; Return to the main loop

instBE		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; STX		BF	Extended	6	3	-aa0-

Alignment

instBF		proc	private

		Clocktime	6

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord	; Get our address to read the data

		shl	edx, 16		; Protect DX
		mov	dx, cx		; Put X in DX

		WriteMemoryWord		; Write the data out

		shr	edx, 16		; Restore DX

		FlagSetX		; Set the remaining flags

		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

instBF		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SUBB		C0	Immediate	2	2	uaaaa

Alignment

instC0		proc	private

		Clocktime	2

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionByte	; Get our BYTE to subtract

		mov	ah, bl		; Make a copy of B
		sub	ah, al		; Subtract it...
		jno	noOverflowC0	; No, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflowC0:
		sub	bl, al		; Now really subtract it
		adc	dh, 0		; If it carried, set the bit

		mov	al, bl		; Copy B to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

instC0		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPB		C1	Immediate	2	2	uaaaa

Alignment

instC1		proc	private

		Clocktime	2

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionByte	; Get our BYTE to compare

		cmp	bl, al		; Compare it
		jno	noOverflowC1	; No, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflowC1:
		mov	ah, al
		mov	al, bl
		sub	al, ah		; Do the compare (via sub)
		adc	dh, 0		; If it carried, set the bit

		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

instC1		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SBCB		C2	Immediate	2	2	uaaaa

Alignment

instC2		proc	private

		Clocktime	2

		shl	ecx, 16		; Save CX
		mov	ch, dh		; Save original flags in CH
		and	ch, CARRY	; Isolate carry

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero 'em out

		FetchInstructionByte	; Get our BYTE to subtract

					; Check for OVERFLOW

		mov	ah, bl		; Put B in AH
		mov	cl, dh		; Put flags into CL
		shr	cl, 1		; Put carry bit into X86 carry

		sbb	ah, al		; Subtract and carry
		
		jno	noOverflowC2	; Nope, it wasn't set
		or	dh, OVERFLOW	; Yes it was set, set overflow

					; Subtract it and check for CARRY
noOverflowC2:
		sub	bl, al		; Now really subtract it
		adc	dh, 0		; If it carried, set the bit

		sub	bl, ch		; Subtract previous carry for B (or not)
		adc	dh, 0		; If it carried, set the bit

		mov	al, bl		; Copy B to AL	
		FlagSet			; Set the remaining flags

		shr	ecx, 16		; Restore CX

		ExecuteNextInstruction 2	; Return to the main loop

instC2		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ADDD		C3	Immediate	4	3	-aaaa

Alignment

instC3		proc	private

		Clocktime	4

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionWord	; Get our WORD to subtract

		shl	ecx, 16		; Save CX
		mov	cx, bx		; Copy D to CX

		add	cx, ax		; Add to D

		jno	noOverflowC3	; No overflow, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflowC3:
		add	bx, ax		; Now really add it
		adc	dh, 0		; If it carried, set the bit

		mov	ax, bx		; Copy D to AX
		FlagSetWord		; Set the remaining flags

		shr	ecx, 16		; Restore CX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 4	; Return to the main loop

instC3		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ANDB		C4	Immediate	2	2	-aa0-

Alignment

instC4		proc	private

		Clocktime	2

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte	; Get our BYTE to AND

		and	bl, al		; Do the AND

		mov	al, bl		; Copy B to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

instC4		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BITB		C5	Immediate	2	2	-aa0-

Alignment

instC5		proc	private

		Clocktime	2

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte	; Get our BYTE to AND

		and	al, bl		; Do the AND

		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

instC5		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDB		C6	Immediate	2	2	-aa0-

Alignment

instC6		proc	private

		Clocktime	2

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte	; Get our BYTE to load into A

		mov	bl, al		; Load it...

		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

instC6		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; EORB		C8	Immediate	2	2	-aa0-

Alignment

instC8		proc	private

		Clocktime	2

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte	; Get our BYTE to XOR with A

		xor	bl, al		; XOR it...

		mov	al, bl		; Copy A to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

instC8		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ADCB		C9	Immediate	2	2	aaaaa

Alignment

instC9		proc	private

		Clocktime	2

		and	dh, 0ffh - (HALFCARRY or OVERFLOW)	; Zero 'em out

		FetchInstructionByte		; Get our BYTE to add

		shl	ecx, 16			; Save CX

						; Check for OVERFLOW

		mov	ah, bl			; Put B in AH
		mov	ch, al			; Put our data in CH
		mov	cl, dh			; Put our flags in CL
		shr	cl, 1			; Put carry in X86 carry

		adc	ah, ch			; Add data and carry to B

		jno	noOverflowC9		; Nope, it wasn't set
		or	dh, OVERFLOW		; Yes it was set, set overflow

						; Check for HALFCARRY
noOverflowC9:
		mov	ah, bl			; Put B in AH
		mov	ch, al			; Put our data in CH
		mov	cl, dh			; Put our flags in CL
		shr	cl, 1			; Put carry in X86 carry

		adc	ah, ch			; Add data and carry to B


		lahf				; Load the flags into AH
		test	ah, 010h		; Is the half-carry set?

		jz	noHalfcarryC9		; Nope, it wasn't set
		or	dh, HALFCARRY		; Yes it was set, set halfcarry

						; Do the ADD and check for CARRY
noHalfcarryC9:
		mov	ch, dh			; We need the old carry...
		and	ch, CARRY		; Isolate it
		and	dh, 0ffh - (CARRY)	; Zero out carry

		add	bl, al			; Now really add it to B
		adc	dh, 0			; If it carried, set the bit

		add	bl, ch			; Add in the previous carry (or not)
		adc	dh, 0			; If it carried, set the bit

		mov	al, bl			; Copy B to AL
		FlagSet				; Set the remaining flags

		shr	ecx, 16			; Restore CX

		ExecuteNextInstruction 2	; Return to the main loop

instC9		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ORB		CA	Immediate	2	2	-aa0-

Alignment

instCA		proc	private

		Clocktime	2

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte	; Get our BYTE to AND

		or	bl, al		; Do the OR

		mov	al, bl		; Copy B to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 2	; Return to the main loop

instCA		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ADDB		CB	Immediate	2	2	aaaaa

Alignment

instCB		proc	private

		Clocktime	2

		and	dh, 0ffh - (HALFCARRY or OVERFLOW or CARRY)	; Zero 'em out

		FetchInstructionByte		; Get our BYTE to add

		shl	ecx, 16			; Save CX
	
						; Check for OVERFLOW

		mov	ah, bl			; Put B in AH
		add	ah, al			; Add data to B

		jno	noOverflowCB		; Nope, it wasn't set
		or	dh, OVERFLOW		; Yes it was set, set overflow

						; Check for HALFCARRY
noOverflowCB:
		mov	ah, bl			; Put B in AH
		add	ah, al			; Add data to B

		lahf				; Load the flags into AH
		test	ah, 010h		; Is the half-carry set?

		jz	noHalfcarryCB		; Nope, it wasn't set
		or	dh, HALFCARRY		; Yes it was set, set halfcarry

						; Do the ADD and check for CARRY
noHalfcarryCB:
		add	bl, al			; Now really add it to B
		adc	dh, 0			; If it carried, set the bit

		mov	al, bl			; Copy B to AL
		FlagSet				; Set the remaining flags

		shr	ecx, 16			; Restore CX

		ExecuteNextInstruction 2	; Return to the main loop

instCB		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDD		CC	Immediate	3	3	-aa0-

Alignment

instCC		proc	private

		Clocktime	4

		and	dh, 0ffh - (OVERFLOW)	; Zero 'em out

		FetchInstructionWord

		mov	bx, ax		; Copy data into D

		FlagSetD		; Set the remaining flags

		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 4	; Return to the main loop

instCC		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDU		CE	Immediate	3	3	-aa0-

Alignment

instCE		proc	private

		Clocktime	3

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord	; Get our WORD to load into AX

ifdef MASM
		mov	_m6809u, ax
endif

		FlagSetWord		; Set the remaining flags

		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 3	; Return to the main loop

instCE		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SUBB		D0	Direct		4	2	uaaaa

Alignment

instD0		proc	private

		Clocktime	4

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchDirectWord		; Get our address for the data fetch
		ReadMemoryByte		; Go get our data

		mov	ah, bl		; Make a copy of B
		sub	ah, al		; Subtract it...
		jno	noOverflowD0	; No, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflowD0:
		sub	bl, al		; Now really subtract it
		adc	dh, 0		; If it carried, set the bit

		mov	al, bl		; Copy B to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instD0		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPB		D1	Direct		4	2	uaaaa

Alignment

instD1		proc	private

		Clocktime	4

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchDirectWord		; Get our address for the data fetch
		ReadMemoryByte		; Go get our data

		cmp	bl, al		; Compare it
		jno	noOverflowD1	; No, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflowD1:
		mov	ah, al
		mov	al, bl
		sub	al, ah		; Do the compare (via sub)
		adc	dh, 0		; If it carried, set the bit

		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instD1		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SBCB		D2	Immediate	4	2	uaaaa

Alignment

instD2		proc	private

		Clocktime	4

		shl	ecx, 16		; Save CX
		mov	ch, dh		; Save original flags in CH
		and	ch, CARRY	; Isolate carry

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero 'em out

		FetchDirectWord		; Get our address for the data fetch
		ReadMemoryByte		; Go get our data

					; Check for OVERFLOW

		mov	ah, bl		; Put B in AH
		mov	cl, dh		; Put flags into CL
		shr	cl, 1		; Put carry bit into X86 carry

		sbb	ah, al		; Subtract and carry
		
		jno	noOverflowD2	; Nope, it wasn't set
		or	dh, OVERFLOW	; Yes it was set, set overflow

					; Subtract it and check for CARRY
noOverflowD2:
		sub	bl, al		; Now really subtract it
		adc	dh, 0		; If it carried, set the bit

		sub	bh, ch		; Subtract previous carry for A (or not)
		adc	dh, 0		; If it carried, set the bit

		mov	al, bl		; Copy A to AL	
		FlagSet			; Set the remaining flags

		shr	ecx, 16		; Restore CX

		ExecuteNextInstruction 4	; Return to the main loop

instD2		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ADDD		D3	Direct		6	2	-aaaa

Alignment

instD3		proc	private

		Clocktime	6

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchDirectWord		; Get our address for the data fetch
		ReadMemoryWord		; Go get our data

		shl	ecx, 16		; Save CX
		mov	cx, bx		; Copy D to CX

		add	cx, ax		; Add from D

		jno	noOverflowD3	; No overflow, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflowD3:
		add	bx, ax		; Now really add it
		adc	dh, 0		; If it carried, set the bit

		mov	ax, bx		; Copy D to AX
		FlagSetWord		; Set the remaining flags

		shr	ecx, 16		; Restore CX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

instD3		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ANDB		D4	Direct		4	2	-aa0-

Alignment

instD4		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord		; Get our address for the data fetch
		ReadMemoryByte		; Go get our data

		and	bl, al		; Do the AND

		mov	al, bl		; Copy A to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instD4		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BITB		D5	Direct		4	2	-aa0-

Alignment

instD5		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord		; Get our address for the data fetch
		ReadMemoryByte		; Go get our data

		and	al, bl		; Do the AND

		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instD5		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDB		D6	Direct		4	2	-aa0-

Alignment

instD6		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord		; Get our address for the data fetch
		ReadMemoryByte		; Go get our data

		mov	bl, al		; Load it...

		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instD6		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; STB		D7	Direct		4	2	-aa0-

Alignment

instD7		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord		; Get our address for the data fetch

		shl	edx, 16		; Protect DX
		mov	dl, bl		; Put the B in DL

		WriteMemoryByte		; Write the data out

		shr	edx, 16		; Restore DX

		mov	al, bl		; Copy B to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instD7		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; EORB		D8	Direct		4	2	-aa0-

Alignment

instD8		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord		; Get our address for the data fetch
		ReadMemoryByte		; Go get our data

		xor	bl, al		; XOR it...

		mov	al, bl		; Copy B to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instD8		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ADCB		D9	Direct		4	2	aaaaa

Alignment

instD9		proc	private

		Clocktime	4

		and	dh, 0ffh - (HALFCARRY or OVERFLOW)	; Zero 'em out

		FetchDirectWord			; Get our address for the data fetch
		ReadMemoryByte			; Go get our data

		shl	ecx, 16			; Save CX

						; Check for OVERFLOW

		mov	ah, bl			; Put B in AH
		mov	ch, al			; Put our data in CH
		mov	cl, dh			; Put our flags in CL
		shr	cl, 1			; Put carry in X86 carry

		adc	ah, ch			; Add data and carry to A

		jno	noOverflowD9		; Nope, it wasn't set
		or	dh, OVERFLOW		; Yes it was set, set overflow

						; Check for HALFCARRY
noOverflowD9:
		mov	ah, bl			; Put B in AH
		mov	ch, al			; Put our data in CH
		mov	cl, dh			; Put our flags in CL
		shr	cl, 1			; Put carry in X86 carry

		adc	ah, ch			; Add data and carry to A

		lahf				; Load the flags into AH
		test	ah, 010h		; Is the half-carry set?

		jz	noHalfcarryD9		; Nope, it wasn't set
		or	dh, HALFCARRY		; Yes it was set, set halfcarry

						; Do the ADD and check for CARRY
noHalfcarryD9:
		mov	ch, dh			; We need the old carry...
		and	ch, CARRY		; Isolate it
		and	dh, 0ffh - (CARRY)	; Zero out carry

		add	bl, al			; Now really add it to B
		adc	dh, 0			; If it carried, set the bit

		add	bl, ch			; Add in the previous carry (or not)
		adc	dh, 0			; If it carried, set the bit

		mov	al, bl			; Copy B to AL
		FlagSet				; Set the remaining flags

		shr	ecx, 16			; Restore CX

		ExecuteNextInstruction 4	; Return to the main loop

instD9		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ORB		DA	Direct		4	2	-aa0-

Alignment

instDA		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord		; Get our address for the data fetch
		ReadMemoryByte		; Go get our data

		or	bl, al		; Do the OR

		mov	al, bl		; Copy B to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instDA		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ADDB		DB	Direct		4	2	aaaaa

Alignment

instDB		proc	private

		Clocktime	4

		and	dh, 0ffh - (HALFCARRY or OVERFLOW or CARRY)	; Zero 'em out

		FetchDirectWord			; Get our address for the data fetch
		ReadMemoryByte			; Go get our data

		shl	ecx, 16			; Save CX
	
						; Check for OVERFLOW

		mov	ah, bl			; Put B in AH
		add	ah, al			; Add data to B

		jno	noOverflowDB		; Nope, it wasn't set
		or	dh, OVERFLOW		; Yes it was set, set overflow

						; Check for HALFCARRY
noOverflowDB:
		mov	ah, bl			; Put B in AH
		add	ah, al			; Add data to B

		lahf				; Load the flags into AH
		test	ah, 010h		; Is the half-carry set?

		jz	noHalfcarryDB		; Nope, it wasn't set
		or	dh, HALFCARRY		; Yes it was set, set halfcarry

						; Do the ADD and check for CARRY
noHalfcarryDB:
		add	bl, al			; Now really add it to B
		adc	dh, 0			; If it carried, set the bit

		mov	al, bl			; Copy B to AL
		FlagSet				; Set the remaining flags

		shr	ecx, 16			; Restore CX

		ExecuteNextInstruction 4	; Return to the main loop

instDB		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDD		DC	Direct		5	2	-aa0-

Alignment

instDC		proc	private

		Clocktime	5

		and	dh, 0ffh - (OVERFLOW)	; Zero 'em out

		FetchDirectWord		; Get our address for the data fetch
		ReadMemoryWord		; Go get our data

		mov	bx, ax		; Copy data into D

		FlagSetD		; Set the remaining flags

		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 5	; Return to the main loop

instDC		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; STD		DD	Direct		5	2	-aa0-

Alignment

instDD		proc	private

		Clocktime	5

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord		; Get our address for the data fetch

		shl	edx, 16		; Protect DX
		mov	dx, bx		; Put D in DX

		WriteMemoryWord		; Write the data out

		shr	edx, 16		; Restore DX

		FlagSetD		; Set the remaining flags

		xor	ah, ah		; Zero upper part of AX

		ExecuteNextInstruction 5	; Return to the main loop

instDD		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDU		DE	Direct		5	2	-aa0-

Alignment

instDE		proc	private

		Clocktime	5

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord		; Get our address for the data fetch
		ReadMemoryWord		; Go get our data
 
ifdef MASM
		mov	_m6809u, ax
endif

		FlagSetWord		; Set the remaining flags

		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 5	; Return to the main loop

instDE		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; STU		DF	Direct		5	2	-aa0-

Alignment

instDF		proc	private

		Clocktime	5

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord		; Get our address for the data fetch

		shl	edx, 16		; Protect DX
ifdef MASM
		mov	dx, _m6809u
endif

		WriteMemoryWord		; Write the data out

		mov	ax, dx		; Copy U to AX
		shr	edx, 16		; Restore DX

		FlagSetWord		; Set the remaining flags

		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 5	; Return to the main loop

instDF		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SUBB		E0	Indexed		4	2	uaaaa

Alignment

instE0		proc	private

		Clocktime	4

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionByte	; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryByte		; Go get our data

		mov	ah, bl		; Make a copy of B
		sub	ah, al		; Subtract it...
		jno	noOverflowE0	; No, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflowE0:
		sub	bl, al		; Now really subtract it
		adc	dh, 0		; If it carried, set the bit

		mov	al, bl		; Copy B to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instE0		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPB		E1	Indexed		4	2	uaaaa

Alignment

instE1		proc	private

		Clocktime	4

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionByte	; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryByte		; Go get our data

		cmp	bl, al		; Compare it
		jno	noOverflowE1	; No, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflowE1:
		mov	ah, al
		mov	al, bl
		sub	al, ah		; Do the compare (via sub)
		adc	dh, 0		; If it carried, set the bit

		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instE1		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SBCB		E2	Indexed		4	2	uaaaa

Alignment

instE2		proc	private

		Clocktime	4

		shl	ecx, 16		; Save CX
		mov	ch, dh		; Save original flags in CH
		and	ch, CARRY	; Isolate carry

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero 'em out

		FetchInstructionByte	; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryByte		; Go get our data

					; Check for OVERFLOW

		mov	ah, bl		; Put B in AH
		mov	cl, dh		; Put flags into CL
		shr	cl, 1		; Put carry bit into X86 carry

		sbb	ah, al		; Subtract and carry
		
		jno	noOverflowE2	; Nope, it wasn't set
		or	dh, OVERFLOW	; Yes it was set, set overflow

					; Subtract it and check for CARRY
noOverflowE2:
		sub	bl, al		; Now really subtract it
		adc	dh, 0		; If it carried, set the bit

		sub	bl, ch		; Subtract previous carry for A (or not)
		adc	dh, 0		; If it carried, set the bit

		mov	al, bl		; Copy B to AL	
		FlagSet			; Set the remaining flags

		shr	ecx, 16		; Restore CX

		ExecuteNextInstruction 4	; Return to the main loop

instE2		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ADDD		E3	Indexed		6	2	-aaaa

Alignment

instE3		proc	private

		Clocktime	6

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionByte	; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryWord		; Go get our data

		shl	ecx, 16		; Save CX
		mov	cx, bx		; Copy D to CX

		add	cx, ax		; Add to D

		jno	noOverflowE3	; No overflow, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflowE3:
		add	bx, ax		; Now really add it
		adc	dh, 0		; If it carried, set the bit

		FlagSetD		; Set the remaining flags

		shr	ecx, 16		; Restore CX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

instE3		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ANDB		E4	Indexed		4	2	-aa0-

Alignment

instE4		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte	; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryByte		; Go get our data

		and	bl, al		; Do the AND

		mov	al, bl		; Copy B to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instE4		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BITB		E5	Indexed		4	2	-aa0-

Alignment

instE5		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte	; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryByte		; Go get our data

		and	al, bl		; Do the AND

		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instE5		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDB		E6	Indexed		4	2	-aa0-

Alignment

instE6		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte	; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryByte		; Go get our data

		mov	bl, al		; Load it...

		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instE6		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; STB		E7	Indexed		4	2	-aa0-

Alignment

instE7		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte	; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from

		shl	edx, 16		; Protect DX
		mov	dl, bl		; Put B in DL

		WriteMemoryByte		; Write the data out

		shr	edx, 16		; Restore DX

		mov	al, bl		; Copy B to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instE7		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; EORB		E8	Indexed		4	2	-aa0-

Alignment

instE8		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte	; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryByte		; Go get our data

		xor	bl, al		; XOR it...

		mov	al, bl		; Copy A to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instE8		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ADCB		E9	Indexed		4	2	aaaaa

Alignment

instE9		proc	private

		Clocktime	4

		and	dh, 0ffh - (HALFCARRY or OVERFLOW)	; Zero 'em out

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryByte			; Go get our data

		shl	ecx, 16			; Save CX

						; Check for OVERFLOW

		mov	ah, bl			; Put B in AH
		mov	ch, al			; Put our data in CH
		mov	cl, dh			; Put our flags in CL
		shr	cl, 1			; Put carry in X86 carry

		adc	ah, ch			; Add data and carry to B

		jno	noOverflowE9		; Nope, it wasn't set
		or	dh, OVERFLOW		; Yes it was set, set overflow

						; Check for HALFCARRY
noOverflowE9:
		mov	ah, bl			; Put B in AH
		mov	ch, al			; Put our data in CH
		mov	cl, dh			; Put our flags in CL
		shr	cl, 1			; Put carry in X86 carry

		adc	ah, ch			; Add data and carry to B

		lahf				; Load the flags into AH
		test	ah, 010h		; Is the half-carry set?

		jz	noHalfcarryE9		; Nope, it wasn't set
		or	dh, HALFCARRY		; Yes it was set, set halfcarry

						; Do the ADD and check for CARRY
noHalfcarryE9:
		mov	ch, dh			; We need the old carry...
		and	ch, CARRY		; Isolate it
		and	dh, 0ffh - (CARRY)	; Zero out carry

		add	bl, al			; Now really add it to B
		adc	dh, 0			; If it carried, set the bit

		add	bl, ch			; Add in the previous carry (or not)
		adc	dh, 0			; If it carried, set the bit

		mov	al, bl			; Copy B to AL
		FlagSet				; Set the remaining flags

		shr	ecx, 16			; Restore CX

		ExecuteNextInstruction 4	; Return to the main loop

instE9		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ORB		EA	Indexed		4	2	-aa0-

Alignment

instEA		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte	; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryByte		; Go get our data

		or	bl, al		; Do the OR

		mov	al, bl		; Copy B to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instEA		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ADDB		EB	Indexed		4	2	aaaaa

Alignment

instEB		proc	private

		Clocktime	4

		and	dh, 0ffh - (HALFCARRY or OVERFLOW or CARRY)	; Zero 'em out

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryByte			; Go get our data

		shl	ecx, 16			; Save CX
	
						; Check for OVERFLOW

		mov	ah, bl			; Put B in AH
		add	ah, al			; Add data to B

		jno	noOverflowEB		; Nope, it wasn't set
		or	dh, OVERFLOW		; Yes it was set, set overflow

						; Check for HALFCARRY
noOverflowEB:
		mov	ah, bl			; Put B in AH
		add	ah, al			; Add data to B

		lahf				; Load the flags into AH
		test	ah, 010h		; Is the half-carry set?

		jz	noHalfcarryEB		; Nope, it wasn't set
		or	dh, HALFCARRY		; Yes it was set, set halfcarry

						; Do the ADD and check for CARRY
noHalfcarryEB:
		add	bl, al			; Now really add it to B
		adc	dh, 0			; If it carried, set the bit

		mov	al, bl			; Copy B to AL
		FlagSet				; Set the remaining flags

		shr	ecx, 16			; Restore CX

		ExecuteNextInstruction 4	; Return to the main loop

instEB		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDD		EC	Indexed		6	2	-aa0-

Alignment

instEC		proc	private

		Clocktime	6

		and	dh, 0ffh - (OVERFLOW)	; Zero 'em out

		FetchInstructionByte	; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryWord		; Go get our data

		mov	bx, ax		; Copy data into D

		FlagSetD		; Set the remaining flags

		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

instEC		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; STD		ED	Indexed		5	2	-aa0-

Alignment

instED		proc	private

		Clocktime	5

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte	; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from

		shl	edx, 16		; Protect DX
		mov	dx, bx		; Put D in DX

		WriteMemoryWord		; Write the data out

		shr	edx, 16		; Restore DX

		FlagSetD		; Set the remaining flags

		xor	ah, ah		; Zero upper part of AX

		ExecuteNextInstruction 5	; Return to the main loop

instED		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDU		EE	Indexed		5	2	-aa0-

Alignment

instEE		proc	private

		Clocktime	5

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte	; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryWord		; Go get our data

 
ifdef MASM
		mov	_m6809u, ax
endif

		FlagSetWord		; Set the remaining flags

		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 5	; Return to the main loop

instEE		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; STU		EF	Indexed		5	2	-aa0-

Alignment

instEF		proc	private

		Clocktime	5

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte	; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from

		shl	edx, 16		; Protect DX
ifdef MASM
		mov	dx, _m6809u
endif

		WriteMemoryWord		; Write the data out

		mov	ax, dx		; Copy U to AX
		shr	edx, 16		; Restore DX

		FlagSetWord		; Set the remaining flags

		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 5	; Return to the main loop

instEF		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SUBB		F0	Extended	5	3	uaaaa

Alignment

instF0		proc	private

		Clocktime	5

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryByte		; Go get our data

		mov	ah, bl		; Make a copy of B
		sub	ah, al		; Subtract it...
		jno	noOverflowF0	; No, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflowF0:
		sub	bl, al		; Now really subtract it
		adc	dh, 0		; If it carried, set the bit

		mov	al, bl		; Copy B to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 5	; Return to the main loop

instF0		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPB		F1	Extended	5	3	uaaaa

Alignment

instF1		proc	private

		Clocktime	5

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryByte		; Go get our data

		cmp	bl, al		; Compare it
		jno	noOverflowF1	; No, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflowF1:
		mov	ah, al
		mov	al, bl
		sub	al, ah		; Do the compare (via sub)
		adc	dh, 0		; If it carried, set the bit

		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 4	; Return to the main loop

instF1		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SBCB		F2	Extended	5	3	uaaaa

Alignment

instF2		proc	private

		Clocktime	5

		shl	ecx, 16		; Save CX
		mov	ch, dh		; Save original flags in CH
		and	ch, CARRY	; Isolate carry

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero 'em out

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryByte		; Go get our data

					; Check for OVERFLOW

		mov	ah, bl		; Put B in AH
		mov	cl, dh		; Put flags into CL
		shr	cl, 1		; Put carry bit into X86 carry

		sbb	ah, al		; Subtract and carry
		
		jno	noOverflowF2	; Nope, it wasn't set
		or	dh, OVERFLOW	; Yes it was set, set overflow

					; Subtract it and check for CARRY
noOverflowF2:
		sub	bl, al		; Now really subtract it
		adc	dh, 0		; If it carried, set the bit

		sub	bl, cl		; Subtract previous carry for B (or not)
		adc	dh, 0		; If it carried, set the bit

		mov	al, bl		; Copy B to AL	
		FlagSet			; Set the remaining flags

		shr	ecx, 16		; Restore CX

		ExecuteNextInstruction 5	; Return to the main loop

instF2		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ADDD		F3	Extended	7	3	-aaaa

Alignment

instF3		proc	private

		Clocktime	7

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryWord		; Go get our data

		shl	ecx, 16		; Save CX
		mov	cx, bx		; Copy D to CX

		add	cx, ax		; Add to D

		jno	noOverflowF3	; No overflow, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflowF3:
		add	bx, ax		; Now really subtract it
		adc	dh, 0		; If it carried, set the bit

		FlagSetD		; Set the remaining flags

		shr	ecx, 16		; Restore CX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

instF3		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ANDB		F4	Extended	5	3	-aa0-

Alignment

instF4		proc	private

		Clocktime	5

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryByte		; Go get our data

		and	bl, al		; Do the AND

		mov	al, bl		; Copy B to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 5	; Return to the main loop

instF4		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; BITB		F5	Extended	5	3	-aa0-

Alignment

instF5		proc	private

		Clocktime	5

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryByte		; Go get our data

		and	al, bl		; Do the AND

		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 5	; Return to the main loop

instF5		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDB		F6	Extended	5	3	-aa0-

Alignment

instF6		proc	private

		Clocktime	5

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryByte		; Go get our data

		mov	bl, al		; Load it...

		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 5	; Return to the main loop

instF6		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; STB		F7	Extended	5	3	-aa0-

Alignment

instF7		proc	private

		Clocktime	5

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord	; Get our address to read the data

		shl	edx, 16		; Protect DX
		mov	dl, bl		; Put the B in DL

		WriteMemoryByte		; Write the data out

		shr	edx, 16		; Restore DX

		mov	al, bl		; Copy B to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 5	; Return to the main loop

instF7		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; EORB		F8	Extended	5	3	-aa0-

Alignment

instF8		proc	private

		Clocktime	5

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryByte		; Go get our data

		xor	bl, al		; XOR it...

		mov	al, bl		; Copy B to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 5	; Return to the main loop

instF8		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ADCB		F9	Extended	5	3	aaaaa

Alignment

instF9		proc	private

		Clocktime	5

		and	dh, 0ffh - (HALFCARRY or OVERFLOW)	; Zero 'em out

		FetchInstructionWord		; Get our address to read the data
		ReadMemoryByte			; Go get our data

		shl	ecx, 16			; Save CX

						; Check for OVERFLOW

		mov	ah, bl			; Put B in AH
		mov	ch, al			; Put our data in CH
		mov	cl, dh			; Put our flags in CL
		shr	cl, 1			; Put carry in X86 carry

		adc	ah, ch			; Add data and carry to B

		jno	noOverflowF9		; Nope, it wasn't set
		or	dh, OVERFLOW		; Yes it was set, set overflow

						; Check for HALFCARRY
noOverflowF9:
		mov	ah, bl			; Put B in AH
		mov	ch, al			; Put our data in CH
		mov	cl, dh			; Put our flags in CL
		shr	cl, 1			; Put carry in X86 carry

		adc	ah, ch			; Add data and carry to B

		lahf				; Load the flags into AH
		test	ah, 010h		; Is the half-carry set?

		jz	noHalfcarryF9		; Nope, it wasn't set
		or	dh, HALFCARRY		; Yes it was set, set halfcarry

						; Do the ADD and check for CARRY
noHalfcarryF9:
		mov	ch, dh			; We need the old carry...
		and	ch, CARRY		; Isolate it
		and	dh, 0ffh - (CARRY)	; Zero out carry

		add	bl, al			; Now really add it to B
		adc	dh, 0			; If it carried, set the bit

		add	bl, ch			; Add in the previous carry (or not)
		adc	dh, 0			; If it carried, set the bit

		mov	al, bl			; Copy B to AL
		FlagSet				; Set the remaining flags

		shr	ecx, 16			; Restore CX

		ExecuteNextInstruction 5	; Return to the main loop

instF9		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ORB		FA	Extended	5	3	-aa0-

Alignment

instFA		proc	private

		Clocktime	5

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryByte		; Go get our data

		or	bl, al		; Do the OR

		mov	al, bl		; Copy B to AL
		FlagSet			; Set the remaining flags

		ExecuteNextInstruction 5	; Return to the main loop

instFA		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; ADDB		FB	Extended	5	3	aaaaa

Alignment

instFB		proc	private

		Clocktime	5

		and	dh, 0ffh - (HALFCARRY or OVERFLOW or CARRY)	; Zero 'em out

		FetchInstructionWord		; Get our address to read the data
		ReadMemoryByte			; Go get our data

		shl	ecx, 16			; Save CX
	
						; Check for OVERFLOW

		mov	ah, bl			; Put B in AH
		add	ah, al			; Add data to B

		jno	noOverflowFB		; Nope, it wasn't set
		or	dh, OVERFLOW		; Yes it was set, set overflow

						; Check for HALFCARRY
noOverflowFB:
		mov	ah, bl			; Put B in AH
		add	ah, al			; Add data to B

		lahf				; Load the flags into AH
		test	ah, 010h		; Is the half-carry set?

		jz	noHalfcarryFB		; Nope, it wasn't set
		or	dh, HALFCARRY		; Yes it was set, set halfcarry

						; Do the ADD and check for CARRY
noHalfcarryFB:
		add	bl, al			; Now really add it to B
		adc	dh, 0			; If it carried, set the bit

		mov	al, bl			; Copy B to AL
		FlagSet				; Set the remaining flags

		shr	ecx, 16			; Restore CX

		ExecuteNextInstruction 5	; Return to the main loop

instFB		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDD		FC	Extended	6	3	-aa0-

Alignment

instFC		proc	private

		Clocktime	6

		and	dh, 0ffh - (OVERFLOW)	; Zero 'em out

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryWord		; Go get our data

		mov	bx, ax		; Copy data into D

		FlagSetD		; Set the remaining flags

		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

instFC		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; STD		FD	Extended	6	3	-aa0-

Alignment

instFD		proc	private

		Clocktime	6

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord	; Get our address to read the data

		shl	edx, 16		; Protect DX
		mov	dx, bx		; Put D in DX

		WriteMemoryWord		; Write the data out

		shr	edx, 16		; Restore DX

		FlagSetD		; Set the remaining flags

		xor	ah, ah		; Zero upper part of AX

		ExecuteNextInstruction 6	; Return to the main loop

instFD		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDU		FE	Extended	6	3	-aa0-

Alignment

instFE		proc	private

		Clocktime	6

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryWord		; Go get our data


ifdef MASM
		mov	_m6809u, ax
endif

		FlagSetWord		; Set the remaining flags

		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

instFE		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; STU		FF	Extended	6	3	-aa0-

Alignment

instFF		proc	private

		Clocktime	6

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord	; Get our address to read the data

		shl	edx, 16		; Protect DX
ifdef MASM
		mov	dx, _m6809u
endif

		WriteMemoryWord		; Write the data out

		mov	ax, dx		; Copy U to AX
		shr	edx, 16		; Restore DX

		FlagSetWord		; Set the remaining flags

		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

instFF		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LBRN		1021	Relative	5	4	-----

Alignment

inst121		proc	private

		Clocktime	5

		FetchInstructionWord		; Get the offset
		ExecuteNextInstruction 5	; Return to the main loop

inst121		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LBHI		1022	Relative	6	4	-----

Alignment

inst122		proc	private

		Clocktime	6

		FetchInstructionWord		; Get the offset

		shl	ecx, 16			; Save CX

		mov	cl, dh			; Copy our flags
		and	cl, (ZERO or CARRY)	; Isolate zero and carry

		jnz	noBranch122		; One of the is set so no branch

		LBranch				; Branch

noBranch122:
		shr	ecx, 16			; Restore CX
		xor	ah, ah

		ExecuteNextInstruction 6	; Return to the main loop

inst122		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LBLS		1023	Relative	6	4	-----

Alignment

inst123		proc	private

		Clocktime	6

		FetchInstructionWord		; Get the offset

		shl	ecx, 16			; Save CX

		mov	cl, dh			; Copy our flags
		and	cl, (ZERO or CARRY)	; Isolate zero and carry

		jz	noBranch123		; One of the is set so no branch

		LBranch				; Branch

noBranch123:
		shr	ecx, 16			; Restore CX
		xor	ah, ah
		ExecuteNextInstruction 6	; Return to the main loop

inst123		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LBCC		1024	Relative	6	4	-----

Alignment

inst124		proc	private

		Clocktime	6

		FetchInstructionWord		; Get the offset

		shl	ecx, 16			; Save CX

		mov	cl, dh			; Copy our flags
		and	cl, (CARRY)		; Isolate carry

		jnz	noBranch124		; Carry is set so no branch

		LBranch				; Branch

noBranch124:
		shr	ecx, 16			; Restore CX
		xor	ah, ah
		ExecuteNextInstruction 6	; Return to the main loop

inst124		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LBCS		1025	Relative	6	4	-----

Alignment

inst125		proc	private

		Clocktime	6

		FetchInstructionWord		; Get the offset

		shl	ecx, 16			; Save CX

		mov	cl, dh			; Copy our flags
		and	cl, (CARRY)		; Isolate carry
		jz	noBranch125		; Carry is not set so no branch

		LBranch				; Branch

noBranch125:
		shr	ecx, 16			; Restore CX
		xor	ah, ah
		ExecuteNextInstruction 6	; Return to the main loop

inst125		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LBNE		1026	Relative	6	4	-----

Alignment

inst126		proc	private

		Clocktime	6

		FetchInstructionWord		; Get the offset

		shl	ecx, 16			; Save CX

		mov	cl, dh			; Copy our flags
		and	cl, (ZERO)		; Isolate zero
		jnz	noBranch126		; Zero is set so no branch

		LBranch				; Branch

noBranch126:
		shr	ecx, 16			; Restore CX
		xor	ah, ah

		ExecuteNextInstruction 6	; Return to the main loop

inst126		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LBEQ		1027	Relative	6	4	-----

Alignment

inst127		proc	private

		Clocktime	6

		FetchInstructionWord		; Get the offset

		shl	ecx, 16			; Save CX

		mov	cl, dh			; Copy our flags
		and	cl, (ZERO)		; Isolate zero
		jz	noBranch127		; Zero is not set so no branch

		LBranch				; Branch

noBranch127:
		shr	ecx, 16			; Restore CX
		xor	ah, ah
		ExecuteNextInstruction 6	; Return to the main loop

inst127		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LBVC		1028	Relative	6	4	-----

Alignment

inst128		proc	private

		Clocktime	6

		FetchInstructionWord		; Get the offset

		shl	ecx, 16			; Save CX

		mov	cl, dh			; Copy our flags
		and	cl, (OVERFLOW)		; Isolate overflow
		jnz	noBranch128		; Overflow is set so no branch

		LBranch				; Branch

noBranch128:
		shr	ecx, 16			; Restore CX
		xor	ah, ah
		ExecuteNextInstruction 6	; Return to the main loop

inst128		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LBVS		1029	Relative	6	4	-----

Alignment

inst129		proc	private

		Clocktime	6

		FetchInstructionWord		; Get the offset

		shl	ecx, 16			; Save CX
		mov	cl, dh			; Copy our flags
		and	cl, (OVERFLOW)		; Isolate overflow
		jz	noBranch129		; Overflow is not set so no branch

		LBranch				; Branch

noBranch129:
		shr	ecx, 16			; Restore CX
		xor	ah, ah
		ExecuteNextInstruction 6	; Return to the main loop

inst129		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LBPL		102A	Relative	6	4	-----

Alignment

inst12A		proc	private

		Clocktime	6

		FetchInstructionWord		; Get the offset

		shl	ecx, 16			; Save CX

		mov	cl, dh			; Copy our flags
		and	cl, (NEGATIVE)		; Isolate negative
		jnz	noBranch12A		; Negative is set so no branch

		LBranch				; Branch

noBranch12A:
		shr	ecx, 16			; Restore CX
		xor	ah, ah
		ExecuteNextInstruction 6	; Return to the main loop

inst12A		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LBMI		102B	Relative	6	4	-----

Alignment

inst12B		proc	private

		Clocktime	6

		FetchInstructionWord		; Get the offset

		shl	ecx, 16			; Save CX

		mov	cl, dh			; Copy our flags
		and	cl, (NEGATIVE)		; Isolate negative
		jz	noBranch12B		; Negative is not set so no branch

		LBranch				; Branch

noBranch12B:
		shr	ecx, 16			; Restore CX
		xor	ah, ah
		ExecuteNextInstruction 6	; Return to the main loop

inst12B		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LBGE		102C	Relative	6	4	-----

Alignment

inst12C		proc	private

		Clocktime	6

		FetchInstructionWord		; Get the offset

		shl	eax, 16			; Save our offset

		mov	ah, dh			; Copy our flags
		and	ah, (OVERFLOW)		; Isolate overflw
		shl	ah, 2			; Put overflow under negative
		mov	al, dh			; Copy our flags
		and	al, (NEGATIVE)		; Isolate negative
		xor	al, ah			; XOR negative and overflow
		jnz	noBranch12C		; Only one is not set so no branch

		shr	eax, 16			; Restore our offset
		
		LBranch				; Branch

noBranch12C:
		xor	eax, eax		; Clear out EAX
		ExecuteNextInstruction 6	; Return to the main loop

inst12C		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LBLT		102D	Relative	6	4	-----

Alignment

inst12D		proc	private

		Clocktime	6

		FetchInstructionWord		; Get the offset

		shl	eax, 16			; Save our offset

		mov	ah, dh			; Copy our flags
		and	ah, (OVERFLOW)		; Isolate overflw
		shl	ah, 2			; Put overflow under negative
		mov	al, dh			; Copy our flags
		and	al, (NEGATIVE)		; Isolate negative
		xor	al, ah			; XOR negative and overflow
		jz	noBranch12D		; Only one is not set so no branch

		shr	eax, 16			; Restore our offset

		LBranch				; Branch

noBranch12D:
		xor	eax, eax		; Clear out EAX

		ExecuteNextInstruction 6	; Return to the main loop

inst12D		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LBGT		102E	Relative	6	4	-----

Alignment

inst12E		proc	private

		Clocktime	6

		FetchInstructionWord		; Get the offset

		shl	eax, 16			; Save our offset

		mov	ah, dh			; Copy our flags
		and	ah, (OVERFLOW)		; Isolate overflw
		shl	ah, 2			; Put overflow under negative
		mov	al, dh			; Copy our flags
		and	al, (NEGATIVE)		; Isolate negative
		xor	al, ah			; XOR negative and overflow
		mov	ah, dh			; Copy our flags
		and	ah, (ZERO)		; Isolate zero
		or	al, ah			; OR zero and XOR from above
		jnz	noBranch12E		; Only one is not set so no branch

		shr	eax, 16			; Restore our offset

		LBranch				; Branch

noBranch12E:
		xor	eax, eax		; Clear out EAX

		ExecuteNextInstruction 6	; Return to the main loop

inst12E		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LBLE		102F	Relative	6	4	-----

Alignment

inst12F		proc	private

		Clocktime	6

		FetchInstructionWord		; Get the offset

		shl	eax, 16			; Save our offset

		mov	ah, dh			; Copy our flags
		and	ah, (OVERFLOW)		; Isolate overflw
		shl	ah, 2			; Put overflow under negative
		mov	al, dh			; Copy our flags
		and	al, (NEGATIVE)		; Isolate negative
		xor	al, ah			; XOR negative and overflow
		mov	ah, dh			; Copy our flags
		and	ah, (ZERO)		; Isolate zero
		or	al, ah			; OR zero with XOR from above
		jz	noBranch12F		; Only one is not set so no branch

		shr	eax, 16			; Restore our offset

		LBranch				; Branch

noBranch12F:
		xor	eax, eax		; Clear out EAX

		ExecuteNextInstruction 6	; Return to the main loop

inst12F		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SWI2		103F	Inherent	20	2	-----

Alignment

inst13F		proc	private

		Clocktime	20

ifdef MASM
		mov	_m6809dp, dl		; Save DP
		mov	_m6809cc, dh		; Save CC
endif
		shl	edx, 16			; Save DX

		LoadStackS			; Load stack pointer S into AX

		sub	esi, ebp		; Get our PC in SI
		mov	dx, si			; Copy it to DX
		add	esi, ebp		; Add the base back in...
		PushStackWord			; Push it on the stack

ifdef MASM
		mov	dx, _m6809u		; Put U into DX
endif
		PushStackWord			; Push it on the stack

ifdef MASM
		mov	dx, _m6809y		; Put Y into DX
endif
		PushStackWord			; Push it on the stack

		mov	dx, cx			; Put X into DX
		PushStackWord			; Push it on the stack

ifdef MASM
		mov	dl, _m6809dp		; Put DP into DL
endif
		PushStackByte			; Push it on the stack

		mov	dl, bl			; Put B into DL
		PushStackByte			; Push it on the stack

		mov	dl, bh			; Put A into DL
		PushStackByte			; Push it on the stack

ifdef MASM
		mov	dl, _m6809cc		; Put CC into DL
endif
		PushStackByte			; Push it on the stack

		UnloadStackS			; Unload stack pointer from AX

		mov	ax, 0fff4h		; The address of the SWI2 vector
		ReadMemoryWord			; Read it

		xor	esi, esi		; Clear out ESI
		mov	si, ax			; Put the new PC in SI
		add	esi, ebp		; Add in our base

		shr	edx, 16			; Restore CC and DP
		or	dh, ENTIRE		; Set entire flag

		xor	ah, ah			; Zero out AH

		ExecuteNextInstruction 20	; Return to the main loop

inst13F		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPD		1083	Immediate	5	4	-aaaa

Alignment

inst183		proc	private

		Clocktime	5

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionWord	; Get our WORD to subtract

		shl	ecx, 16		; Save CX

		cmp	bx, ax		; Subtract from D

		jno	noOverflow183	; No overflow, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflow183:
		mov	cx, bx		; Copy D to CX
		sub	cx, ax		; Subtract it
		adc	dh, 0		; If it carried, set the bit

		mov	ax, cx		; Copy CX to AX
		FlagSetWord		; Set the remaining flags

		shr	ecx, 16		; Restore CX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 5	; Return to the main loop

inst183		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPY		108C	Immediate	5	4	-aaaa

Alignment

inst18C		proc	private

		Clocktime	5

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero 'em out

		FetchInstructionWord	; Get our WORD to compare

		shl	ebx, 16		; Save BX

ifdef MASM
		mov	bx, _m6809y	; Copy Y into BX
endif

		cmp	bx, ax		; Compare data to Y

		jno	noOverflow18C	; Nope, it wasn't set
		or	dh, OVERFLOW	; Yes it was set, set overflow

noOverflow18C:
		sub	bx, ax		; Follow through on the CMP

		adc	dh, 0		; If it carried, set the bit

		mov	ax, bx		; Copy BX to AX
		FlagSetWord		; Set the remaining flags

		shr	ebx, 16		; Restore BX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 5	; Return to the main loop

inst18C		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDY		108E	Immediate	4	4	-aa0-

Alignment

inst18E		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord	; Get our WORD to load into AX

ifdef MASM
		mov	_m6809y, ax		; Load it...
endif

		FlagSetWord		; Set the remaining flags

		xor	ah, ah			; Zero out AH

		ExecuteNextInstruction 4	; Return to the main loop

inst18E		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPD		1093	Direct		7	3	-aaaa

Alignment

inst193		proc	private

		Clocktime	7

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchDirectWord		; Get our address for the data fetch
		ReadMemoryWord		; Go get our data

		shl	ecx, 16		; Save CX

		cmp	bx, ax		; Compare D to data

		jno	noOverflow193	; No overflow, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflow193:
		mov	cx, bx		; Copy D to CX
		sub	cx, ax		; Subtract it
		adc	dh, 0		; If it carried, set the bit

		mov	ax, cx		; Copy CX to AX
		FlagSetWord		; Set the remaining flags

		shr	ecx, 16		; Restore CX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

inst193		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPY		109C	Direct		7	3	-aaaa

Alignment

inst19C		proc	private

		Clocktime	7

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero 'em out

		FetchDirectWord		; Get our address for the data fetch
		ReadMemoryWord		; Go get our data

		shl	ebx, 16		; Save BX

ifdef MASM
		mov	bx, _m6809y	; Copy Y into BX
endif

		cmp	bx, ax		; Compare data to Y

		jno	noOverflow19C	; Nope, it wasn't set
		or	dh, OVERFLOW	; Yes it was set, set overflow

noOverflow19C:
		sub	bx, ax		; Follow through on the CMP

		adc	dh, 0		; If it carried, set the bit

		mov	ax, bx		; Copy BX to AX
		FlagSetWord		; Set the remaining flags

		shr	ebx, 16		; Restore BX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

inst19C		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDY		109E	Direct		6	3	-aa0-

Alignment

inst19E		proc	private

		Clocktime	6

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord		; Get our address for the data fetch
		ReadMemoryWord		; Go get our data

ifdef MASM
		mov	_m6809y, ax		; Load it...
endif

		FlagSetWord		; Set the remaining flags

		xor	ah, ah			; Zero out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst19E		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; STY		109F	Direct		6	3	-aa0-

Alignment

inst19F		proc	private

		Clocktime	6

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord		; Get our address for the data fetch

		shl	edx, 16		; Protect DX
ifdef MASM
		mov	dx, _m6809y	; Put Y in DX
endif

		WriteMemoryWord		; Write the data out

		mov	ax, dx		; Copy DX to AX

		FlagSetWord		; Set the remaining flags

		shr	edx, 16		; Restore DX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst19F		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPD		10A3	Indexed		7	3	-aaaa

Alignment

inst1A3		proc	private

		Clocktime	7

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionByte	; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryWord		; Go get our data

		shl	ecx, 16		; Save CX

		cmp	bx, ax		; Compare D to data

		jno	noOverflow1A3	; No overflow, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflow1A3:
		mov	cx, bx		; Copy D to CX
		sub	cx, ax		; Subtract it
		adc	dh, 0		; If it carried, set the bit

		mov	ax, cx		; Copy CX to AX
		FlagSetWord		; Set the remaining flags

		shr	ecx, 16		; Restore CX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

inst1A3		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPY		10AC	Indexed		7	3	-aaaa

Alignment

inst1AC		proc	private

		Clocktime	7

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero 'em out

		FetchInstructionByte	; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryWord		; Go get our data

		shl	ebx, 16		; Save BX

ifdef MASM
		mov	bx, _m6809y	; Copy Y into BX
endif

		cmp	bx, ax		; Compare data to Y

		jno	noOverflow1AC	; Nope, it wasn't set
		or	dh, OVERFLOW	; Yes it was set, set overflow

noOverflow1AC:
		sub	bx, ax		; Follow through on the CMP

		adc	dh, 0		; If it carried, set the bit

		mov	ax, bx		; Copy BX to AX
		FlagSetWord		; Set the remaining flags

		shr	ebx, 16		; Restore BX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

inst1AC		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDY		10AE	Indexed		6	3	-aa0-

Alignment

inst1AE		proc	private

		Clocktime	6

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte	; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryWord		; Go get our data

ifdef MASM
		mov	_m6809y, ax		; Load it...
endif

		FlagSetWord		; Set the remaining flags

		xor	ah, ah			; Zero out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst1AE		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; STY		10AF	Indexed		6	3	-aa0-

Alignment

inst1AF		proc	private

		Clocktime	6

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte	; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from

		shl	edx, 16		; Protect DX
ifdef MASM
		mov	dx, _m6809y	; Put Y in DX
endif

		WriteMemoryWord		; Write the data out

		mov	ax, dx		; Copy DX to AX
		FlagSetWord		; Set the remaining flags

		shr	edx, 16		; Restore DX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst1AF		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPD		10B3	Extended	8	4	-aaaa

Alignment

inst1B3		proc	private

		Clocktime	8

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryWord		; Go get our data

		shl	ecx, 16		; Save CX

		cmp	bx, ax		; Compare D to data

		jno	noOverflow1B3	; No overflow, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflow1B3:
		mov	cx, bx		; Copy D to CX
		sub	cx, ax		; Subtract it
		adc	dh, 0		; If it carried, set the bit

		mov	ax, cx		; Copy CX to AX
		FlagSetWord		; Set the remaining flags

		shr	ecx, 16		; Restore CX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 8	; Return to the main loop

inst1B3		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPY		10BC	Extended	8	4	-aaaa

Alignment

inst1BC		proc	private

		Clocktime	8

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero 'em out

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryWord		; Go get our data

		shl	ebx, 16		; Save BX

ifdef MASM
		mov	bx, _m6809y	; Copy Y into BX
endif

		cmp	bx, ax		; Compare data to Y

		jno	noOverflow1BC	; Nope, it wasn't set
		or	dh, OVERFLOW	; Yes it was set, set overflow

noOverflow1BC:
		sub	bx, ax		; Follow through on the CMP

		adc	dh, 0		; If it carried, set the bit

		mov	ax, bx		; Copy BX to AX
		FlagSetWord		; Set the remaining flags

		shr	ebx, 16		; Restore BX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 8	; Return to the main loop

inst1BC		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDY		10BE	Extended	7	4	-aa0-

Alignment

inst1BE		proc	private

		Clocktime	7

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryWord		; Go get our data

ifdef MASM
		mov	_m6809y, ax		; Load it...
endif

		FlagSetWord		; Set the remaining flags

		xor	ah, ah			; Zero out AH

		ExecuteNextInstruction 7	; Return to the main loop

inst1BE		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; STY		10BF	Extended	7	4	-aa0-

Alignment

inst1BF		proc	private

		Clocktime	7

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord	; Get our address to read the data

		shl	edx, 16		; Protect DX
ifdef MASM
		mov	dx, _m6809y	; Put Y in DX
endif

		WriteMemoryWord		; Write the data out

		mov	ax, dx		; Copy DX to AX
		FlagSetWord		; Set the remaining flags

		shr	edx, 16		; Restore DX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

inst1BF		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDS		10CE	Immediate	4	4	-aa0-

Alignment

inst1CE		proc	private

		Clocktime	4

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord	; Get our WORD to load into AX

ifdef MASM
		mov	_m6809s, ax
endif

		FlagSetWord		; Set the remaining flags

		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 4	; Return to the main loop

inst1CE		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDS		10DE	Direct		6	3	-aa0-

Alignment

inst1DE		proc	private

		Clocktime	6

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord		; Get our address for the data fetch
		ReadMemoryWord		; Go get our data
 
ifdef MASM
		mov	_m6809s, ax
endif

		FlagSetWord		; Set the remaining flags

		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst1DE		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; STS		10DF	Direct		6	3	-aa0-

Alignment

inst1DF		proc	private

		Clocktime	6

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchDirectWord		; Get our address for the data fetch

		shl	edx, 16		; Protect DX
ifdef MASM
		mov	dx, _m6809s
endif

		WriteMemoryWord		; Write the data out

		mov	ax, dx		; Copy U to AX
		shr	edx, 16		; Restore DX

		FlagSetWord		; Set the remaining flags

		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst1DF		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDS		10EE	Indexed		6	3	-aa0-

Alignment

inst1EE		proc	private

		Clocktime	6

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte	; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryWord		; Go get our data

 
ifdef MASM
		mov	_m6809s, ax
endif

		FlagSetWord		; Set the remaining flags

		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst1EE		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; STS		1EF	Indexed		6	3	-aa0-

Alignment

inst1EF		proc	private

		Clocktime	6

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionByte		; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from

		shl	edx, 16			; Protect DX
ifdef MASM
		mov	dx, _m6809s
endif

		WriteMemoryWord			; Write the data out

		mov	ax, dx			; Copy S to AX
		shr	edx, 16			; Restore DX

		FlagSetWord			; Set the remaining flags

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 6	; Return to the main loop

inst1EF		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; LDS		10FE	Extended	7	4	-aa0-

Alignment

inst1FE		proc	private

		Clocktime	7

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord		; Get our address to read the data
		ReadMemoryWord			; Go get our data


ifdef MASM
		mov	_m6809s, ax
endif

		FlagSetWord			; Set the remaining flags

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

inst1FE		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; STS		10FF	Extended	7	4	-aa0-

Alignment

inst1FF		proc	private

		Clocktime	7

		and	dh, 0ffh - ( OVERFLOW )	; Zero out overflow

		FetchInstructionWord		; Get our address to read the data

		shl	edx, 16			; Protect DX
ifdef MASM
		mov	dx, _m6809s
endif

		WriteMemoryWord			; Write the data out

		mov	ax, dx			; Copy S to AX
		shr	edx, 16			; Restore DX

		FlagSetWord			; Set the remaining flags

		xor	ah, ah			; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

inst1FF		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; SWI3		113F	Inherent	20	2	-----

Alignment

inst23F		proc	private

		Clocktime	20

ifdef MASM
		mov	_m6809dp, dl		; Save DP
		mov	_m6809cc, dh		; Save CC
endif
		shl	edx, 16			; Save DX

		LoadStackS			; Load stack pointer S into AX

		sub	esi, ebp		; Get our PC in SI
		mov	dx, si			; Copy it to DX
		add	esi, ebp		; Add the base back in...
		PushStackWord			; Push it on the stack

ifdef MASM
		mov	dx, _m6809u		; Put U into DX
endif
		PushStackWord			; Push it on the stack

ifdef MASM
		mov	dx, _m6809y		; Put Y into DX
endif
		PushStackWord			; Push it on the stack

		mov	dx, cx			; Put X into DX
		PushStackWord			; Push it on the stack

ifdef MASM
		mov	dl, _m6809dp		; Put DP into DL
endif
		PushStackByte			; Push it on the stack

		mov	dl, bl			; Put B into DL
		PushStackByte			; Push it on the stack

		mov	dl, bh			; Put A into DL
		PushStackByte			; Push it on the stack

ifdef MASM
		mov	dl, _m6809cc		; Put CC into DL
endif
		PushStackByte			; Push it on the stack

		UnloadStackS			; Unload stack pointer from AX

		mov	ax, 0fff2h		; The address of the SWI3 vector
		ReadMemoryWord			; Read it

		xor	esi, esi		; Clear out ESI
		mov	si, ax			; Put the new PC in SI
		add	esi, ebp		; Add in our base

		shr	edx, 16			; Restore CC and DP
		or	dh, ENTIRE		; Set entire flag

		xor	ah, ah			; Zero out AH

		ExecuteNextInstruction 20	; Return to the main loop

inst23F		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPU		1183	Immediate	5	4	-aaaa

Alignment

inst283		proc	private

		Clocktime	5

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionWord	; Get our WORD to subtract

		shl	ebx, 16		; Save BX

ifdef MASM
		mov	bx, _m6809u	; Copy U into BX
endif

		cmp	bx, ax		; Compare data to U

		jno	noOverflow283	; No overflow, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflow283:
		sub	bx, ax		; Follow through on the CMP

		adc	dh, 0		; If it carried, set the bit

		mov	ax, bx		; Copy BX to AX
		FlagSetWord		; Set the remaining flags

		shr	ebx, 16		; Restore BX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 5	; Return to the main loop

inst283		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPS		118C	Immediate	5	4	-aaaa

Alignment

inst28C		proc	private

		Clocktime	5

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero 'em out

		FetchInstructionWord	; Get our WORD to compare

		shl	ebx, 16		; Save BX

ifdef MASM
		mov	bx, _m6809s	; Copy S into BX
endif

		cmp	bx, ax		; Compare data to S

		jno	noOverflow28C	; Nope, it wasn't set
		or	dh, OVERFLOW	; Yes it was set, set overflow

noOverflow28C:
		sub	bx, ax		; Follow through on the CMP

		adc	dh, 0		; If it carried, set the bit

		mov	ax, bx		; Copy BX to AX
		FlagSetWord		; Set the remaining flags

		shr	ebx, 16		; Restore BX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 5	; Return to the main loop

inst28C		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPU		1193	Direct		7	3	-aaaa

Alignment

inst293		proc	private

		Clocktime	7

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchDirectWord		; Get our address for the data fetch
		ReadMemoryWord		; Go get our data

		shl	ebx, 16		; Save BX

ifdef MASM
		mov	bx, _m6809u	; Copy U into BX
endif

		cmp	bx, ax		; Compare data to U

		jno	noOverflow293	; No overflow, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflow293:
		sub	bx, ax		; Follow through on the CMP

		adc	dh, 0		; If it carried, set the bit

		mov	ax, bx		; Copy BX to AX
		FlagSetWord		; Set the remaining flags

		shr	ebx, 16		; Restore BX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

inst293		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPS		119C	Direct		7	3	-aaaa

Alignment

inst29C		proc	private

		Clocktime	7

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero 'em out

		FetchDirectWord		; Get our address for the data fetch
		ReadMemoryWord		; Go get our data

		shl	ebx, 16		; Save BX

ifdef MASM
		mov	bx, _m6809s	; Copy S into BX
endif

		cmp	bx, ax		; Compare data to S

		jno	noOverflow29C	; Nope, it wasn't set
		or	dh, OVERFLOW	; Yes it was set, set overflow

noOverflow29C:
		sub	bx, ax		; Follow through on the CMP

		adc	dh, 0		; If it carried, set the bit

		mov	ax, bx		; Copy BX to AX
		FlagSetWord		; Set the remaining flags

		shr	ebx, 16		; Restore BX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

inst29C		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPU		11A3	Indexed		7	3	-aaaa

Alignment

inst2A3		proc	private

		Clocktime	7

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionByte	; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryWord		; Go get our data

		shl	ebx, 16		; Save BX

ifdef MASM
		mov	bx, _m6809u	; Copy U into BX
endif

		cmp	bx, ax		; Compare data to U

		jno	noOverflow2A3	; No overflow, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflow2A3:
		sub	bx, ax		; Follow through on the CMP

		adc	dh, 0		; If it carried, set the bit

		mov	ax, bx		; Copy BX to AX
		FlagSetWord		; Set the remaining flags

		shr	ebx, 16		; Restore BX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

inst2A3		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPS		11AC	Indexed		7	3	-aaaa

Alignment

inst2AC		proc	private

		Clocktime	7

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero 'em out

		FetchInstructionByte	; Get our postbyte for the index fetch into AL
		call	m6809GetIndexWord	; Get our address to fetch from
		ReadMemoryWord		; Go get our data

		shl	ebx, 16		; Save BX

ifdef MASM
		mov	bx, _m6809s	; Copy S into BX
endif

		cmp	bx, ax		; Compare data to S

		jno	noOverflow2AC	; Nope, it wasn't set
		or	dh, OVERFLOW	; Yes it was set, set overflow

noOverflow2AC:
		sub	bx, ax		; Follow through on the CMP

		adc	dh, 0		; If it carried, set the bit

		mov	ax, bx		; Copy BX to AX
		FlagSetWord		; Set the remaining flags

		shr	ebx, 16		; Restore BX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 7	; Return to the main loop

inst2AC		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPU		11B3	Extended	8	4	-aaaa

Alignment

inst2B3		proc	private

		Clocktime	8

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero out carry and overflow

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryWord		; Go get our data

		shl	ebx, 16		; Save BX

ifdef MASM
		mov	bx, _m6809u	; Copy U into BX
endif

		cmp	bx, ax		; Compare data to U

		jno	noOverflow2B3	; No overflow, skip down...
		or	dh, OVERFLOW	; Yes, set the overflow bit

noOverflow2B3:
		sub	bx, ax		; Follow through on the CMP

		adc	dh, 0		; If it carried, set the bit

		mov	ax, bx		; Copy BX to AX
		FlagSetWord		; Set the remaining flags

		shr	ebx, 16		; Restore BX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 8	; Return to the main loop

inst2B3		endp

;		Opcode	Addressing
; Instruction	Hex	Mode		Cycles	Bytes	HNZVC
;
; CMPS		11BC	Extended	8	4	-aaaa

Alignment

inst2BC		proc	private

		Clocktime	8

		and	dh, 0ffh - (OVERFLOW or CARRY)	; Zero 'em out

		FetchInstructionWord	; Get our address to read the data
		ReadMemoryWord		; Go get our data

		shl	ebx, 16		; Save BX

ifdef MASM
		mov	bx, _m6809s	; Copy S into BX
endif

		cmp	bx, ax		; Compare data to S

		jno	noOverflow2BC	; Nope, it wasn't set
		or	dh, OVERFLOW	; Yes it was set, set overflow

noOverflow2BC:
		sub	bx, ax		; Follow through on the CMP

		adc	dh, 0		; If it carried, set the bit

		mov	ax, bx		; Copy BX to AX
		FlagSetWord		; Set the remaining flags

		shr	ebx, 16		; Restore BX
		xor	ah, ah		; Clear out AH

		ExecuteNextInstruction 8	; Return to the main loop

inst2BC		endp

; Support routines go here

;************************************************************************
;									*
; Name	: m6809reset()							*
;									*
; Entry : Nothing							*
;									*
; Exit	: Nothing							*
;									*
; Description:								*
;									*
; This routine will reset the 6809's flags, registers, & interrupt modes *
;									*
;************************************************************************

Alignment

ifdef MASM
m6809reset_	proc
_m6809reset::
endif

		push	ebp

		xor	eax, eax	; Zero AX

ifdef MASM
		mov	ebp, _m6809Base		; Get the base address
		mov	_m6809a, al;		; Zero A
		mov	_m6809b, al;		; Zero B
		mov	_m6809dp, al;		; Zero DP
		mov	_m6809cc, (IRQ or FIRQ)	; Zero CC and set the IRQ and FIRQ flags
endif

		mov	ax, 0fffeh	; Reset vector
		mov     ax, [ebp + eax]	; Get our reset vector address
		xchg	ah,al		; Make it little endian

ifdef MASM
		mov	_m6809pc, ax	; Store our new PC
endif

		xor	ah, ah		; Zero AH
		pop	ebp

		ret

ifdef MASM
m6809reset_	endp
endif

;************************************************************************
;									*
; Name	: m6809nmi_							*
;									*
; Entry : Nothing							*
;									*
; Exit	: 0=NMI was taken. NMI Is always taken				*
;									*
; Description:								*
;									*
; This routine simulates a virtual NMI to the emulated 6809. No code	*
; execution actually takes place - only the PC is reset and the current *
; PC gets pushed to the stack.						*
;									*
;************************************************************************

Alignment

ifdef MASM
m6809nmi_	proc
_m6809nmi::
endif

		xor	eax, eax		; Zero EAX

ifdef MASM
		cmp	byte ptr _inNmi, al	; Are we in an nmi?
endif

		jne	nmiNotTaken

ifdef MASM
		inc	_inNmi
endif

		Clocktime	19		; Chew up 19 cycles...

		push	edx			; Save EDX
		push	ebp			; Save EBP
		push	esi
		push	edi

ifdef MASM
		mov	ebp, _m6809Base		; Put base into EBP
endif

		LoadStackS			; Load stack pointer S into AX

ifdef MASM
		mov	dx, _m6809pc		; Put PC into DX
endif
		PushStackWord			; Push it on the stack

ifdef MASM
		mov	dx, _m6809u		; Put U into DX
endif
		PushStackWord			; Push it on the stack

ifdef MASM
		mov	dx, _m6809y		; Put Y into DX
endif
		PushStackWord			; Push it on the stack

ifdef MASM
		mov	dx, _m6809x		; Put X into DX
endif
		PushStackWord			; Push it on the stack

ifdef MASM
		mov	dl, _m6809dp		; Put DP into DL
endif
		PushStackByte			; Push it on the stack

ifdef MASM
		mov	dl, _m6809b		; Put B into DL
endif
		PushStackByte			; Push it on the stack

ifdef MASM
		mov	dl, _m6809a		; Put A into DL
endif
		PushStackByte			; Push it on the stack

ifdef MASM
		mov	dl, _m6809cc		; Put CC into DL
endif
		PushStackByte			; Push it on the stack

		UnloadStackS			; Unload stack pointer from AX

		mov	ax, 0fffch		; The address of the IRQ vector
		ReadMemoryWord			; Read it

ifdef MASM
		mov	_m6809pc, ax		; Store virtual address
endif

ifdef MASM
		mov	dh, _m6809cc		; Put CC into DH
endif

		or	dh, (ENTIRE or FIRQ or IRQ)	; Set entire and irq flags

ifdef MASM
		mov	_m6809cc, dh		; Put DH into CC
endif

		xor	eax, eax		; Indicate that we took the IRQ

		pop	edi
		pop	esi
		pop	ebp
		pop	edx

		ret

nmiNotTaken:
		mov	eax, -1			; Indicate that we didn't take the NMI
		ret

ifdef MASM
m6809nmi_	endp
endif

;************************************************************************
;									*
; Name	: m6809int_							*
;									*
; Entry : Nothing							*
;									*
; Exit	: 0=IRQ was taken, Non-zero=already in interrupt		*
;									*
; Description:								*
;									*
; This routine simulates a virtual IRQ to the emulated 6809. No code	*
; execution actually takes place - only the PC is reset and the current *
; PC gets pushed to the stack.						*
;									*
;************************************************************************

Alignment

ifdef MASM
m6809int_	proc
_m6809int::
endif

		xor	eax, eax		; Zero EAX

ifdef MASM
		cmp	byte ptr _inNmi, al	; Are we in an nmi?
endif

		jne	irqNotTaken		; Yep, we are.  Bail out...

ifdef MASM
		mov	al, _m6809cc		; Get our flags
endif
		test	al, IRQ			; Is the IRQ flag set?

		jnz	irqNotTaken		; Yes it is...  We're outta here.

		Clocktime	19		; Chew up 19 cycles...

		push	edx			; Save EDX
		push	ebp			; Save EBP
		push	edi
		push	esi

ifdef MASM
		mov	ebp, _m6809Base		; Put base into EBP
endif

		LoadStackS			; Load stack pointer S into AX

ifdef MASM
		mov	dx, _m6809pc		; Put PC into DX
endif
		PushStackWord			; Push it on the stack

ifdef MASM
		mov	dx, _m6809u		; Put U into DX
endif
		PushStackWord			; Push it on the stack

ifdef MASM
		mov	dx, _m6809y		; Put Y into DX
endif
		PushStackWord			; Push it on the stack

ifdef MASM
		mov	dx, _m6809x		; Put X into DX
endif
		PushStackWord			; Push it on the stack

ifdef MASM
		mov	dl, _m6809dp		; Put DP into DL
endif
		PushStackByte			; Push it on the stack

ifdef MASM
		mov	dl, _m6809b		; Put B into DL
endif
		PushStackByte			; Push it on the stack

ifdef MASM
		mov	dl, _m6809a		; Put A into DL
endif
		PushStackByte			; Push it on the stack

ifdef MASM
		mov	dl, _m6809cc		; Put CC into DL
endif
		PushStackByte			; Push it on the stack

		UnloadStackS			; Unload stack pointer from AX

		mov	ax, 0fff8h		; The address of the IRQ vector
		ReadMemoryWord			; Read it

ifdef MASM
		mov	_m6809pc, ax		; Store virtual address
endif

ifdef MASM
		mov	dh, _m6809cc		; Put CC into DH
endif

		or	dh, (ENTIRE or FIRQ or IRQ)	; Set entire and irq flags

ifdef MASM
		mov	_m6809cc, dh		; Put DH into CC
endif
		xor	eax, eax		; Indicate that we took the IRQ

		pop	esi
		pop	edi
		pop	ebp
		pop	edx

		ret

irqNotTaken:
		mov	eax, -1			; Indicate that we didn't take the IRQ
		ret

ifdef MASM
m6809int_	endp
endif

;************************************************************************
;									*
; Name	: m6809firq_							*
;									*
; Entry : Nothing							*
;									*
; Exit	: 0=FIRQ was taken, Non-zero=already in interrupt		*
;									*
; Description:								*
;									*
; This routine simulates a fast IRQ to the emulated 6809. No code	*
; execution actually takes place - only the PC is reset and the current *
; PC gets pushed to the stack.						*
;									*
;************************************************************************

Alignment

ifdef MASM
m6809firq_	proc
_m6809firq::
endif

		xor	eax, eax		; Zero EAX

ifdef MASM
		cmp	byte ptr _inNmi, al	; Are we in an nmi?
endif

		jne	firqNotTaken		; Yep, we are.  Bail out...

		test	dh, FIRQ		; Is the firq flag set?

		jnz	firqNotTaken		; Yes it is...  We're outta here.

		Clocktime	10		; Chew up 10 cycles...

		push	edx
		push	ebp			; Save EBP

ifdef MASM
		mov	ebp, _m6809Base		; Put base into EBP
endif

		LoadStackS			; Load stack pointer S into AX

ifdef MASM
		mov	dx, _m6809pc		; Put PC into DX
endif
		PushStackWord			; Push it on the stack

ifdef MASM
		mov	dl, _m6809cc		; Put CC into DL
endif
		PushStackByte			; Push it on the stack

		UnloadStackS			; Unload stack pointer from AX

		mov	ax, 0fff6h		; The address of the IRQ vector
		ReadMemoryWord			; Read it

ifdef MASM
		mov	_m6809pc, ax		; Store virtual address
endif

ifdef MASM
		mov	dh, _m6809cc		; Put CC into DH
endif

		and	dh, 07fh		; Zero out entire flag
		or	dh, (FIRQ or IRQ)	; Set entire and irq flags

ifdef MASM
		mov	_m6809cc, dh		; Put DH into CC
		mov	_m6809cc, dh		; Put DH into CC
endif

		xor	eax, eax		; Indicate that we took the IRQ

		pop	edx
		pop	ebp

		ret

firqNotTaken:
		mov	eax, -1			; Indicate that we didn't take the IRQ
		ret

ifdef MASM
m6809firq_	endp
endif

;************************************************************************
;									*
; Name	: m6809GetContextSize						*
;									*
; Entry : Nothing							*
;									*
; Exit	: Size of context array (in bytes)				*
;									*
; Description:								*
;									*
; This routine returns the # of bytes needed for a context switch for	*
; multi-CPU operation.							*
;									*
;************************************************************************

ifdef MULTI_CPU

Alignment

ifdef MASM
m6809GetContextSize_	proc
_m6809GetContextSize::
endif

		mov	eax, offset ContextEnd - offset ContextBegin
		ret

ifdef MASM
m6809GetContextSize_	endp
endif

endif

;************************************************************************
;									*
; Name	: _m6809setContext						*
;									*
; Entry : Address of context in EAX					*
;									*
; Exit	: Nothing							*
;									*
; Description:								*
;									*
; This routine will set a context. On input, a source address for the	*
; context is given.							*
;									*
;************************************************************************

ifdef MULTI_CPU

Alignment

m6809SetContext_ proc
_m6809SetContext::

ifdef USE_STACK
		mov	eax, [esp+4]	; Get our address of our context
endif

		push	esi		; Save registers we use
		push	edi
		push	ecx

		mov	ecx, offset ContextEnd - offset ContextBegin
		mov	esi, eax	; Source address in ESI
		mov	edi, offset ContextBegin
		rep	movsb		; Move it as fast as we can!

		pop	ecx
		pop	edi
		pop	esi

		ret			; No return code

m6809SetContext_ endp

endif	; MULTI_CPU

;************************************************************************
;									*
; Name	: m6809GetContext						*
;									*
; Entry : Address of context in EAX					*
;									*
; Exit	: Nothing							*
;									*
; Description:								*
;									*
; This routine will get a context. On input, a dest. address for the	*
; context is given.							*
;									*
;************************************************************************

ifdef MULTI_CPU

Alignment

ifdef MASM
m6809GetContext_	proc
_m6809GetContext::
endif

ifdef USE_STACK
		mov	eax, [esp+4]	; Get our address of our context
endif

		push	esi		; Save registers we use
		push	edi
		push	ecx

		mov	edi, eax	; Destination address in EDI

		mov	ecx, offset ContextEnd - offset ContextBegin
		mov	esi, offset ContextBegin
		rep	movsb		; Move it as fast as we can!

		pop	ecx
		pop	edi
		pop	esi

		ret			; No return code

ifdef MASM
m6809GetContext_	endp
endif

endif ; MULTI_CPU

;************************************************************************
;									*
; Name	: m6809exec_(instructions)					*
;									*
; Entry : # Of 6809 instructions to execute.				*
;									*
; Exit	: 010000h If Execution was valid				*
;	  010001h If attempted access outside of 64K area.		*
;	 <010000h If invalid instruction - address of invalid instr.	*
;									*
; Description:								*
;									*
; This routine will cause the virtual 6809 code to be emulated. Read the*
; header notes at the beginning of this file for programming		*
; information on setting up the 6809 for execution.			*
;									*
;************************************************************************

Alignment

ifdef MASM
m6809exec_	proc
_m6809exec::
	
endif

ifdef USE_STACK
		mov	eax, [esp+4]	; Get # of cycles to run
endif
		push	ebp
		push	ebx			; Save all registers we use
		push	ecx
		push	edx
		push	esi
		push	edi

ifdef MASM
		mov	_cyclesRemaining, eax	 ; Store # of cycles to exec
		mov	[_dwElapsedTicks], 0
endif

		cld				; Go forward!

		xor	eax, eax		; Zero EAX 'cause we use it!
		xor	ebx, ebx		; Zero EBX, too
		xor	ecx, ecx		; Zero ECX
		xor	edx, edx		; And EDX
		xor	edi, edi		; Zero EDI as well
		xor	esi, esi		; Zero our source address

ifdef MASM
		mov	bh, _m6809a		; Get our A register
		mov	bl, _m6809b		; Get our B register
		mov	cx, _m6809x		; Get our X register
		mov	dh, _m6809cc		; Get our flags
		mov	dl, _m6809dp		; Get our DP register
		mov	si, _m6809pc		; Get our program counter
		mov	ebp, _m6809Base		; Get the base address
endif

		add	esi, ebp		; Add in our base address

		FetchInstructionByte		; Get our first instruction
		jmp	dword ptr m6809zero[eax*4] ; Jump to our instruction!

; We get to invalidInsByte if it's a single byte invalid opcode

ifdef MASM
invalidInsByte::
endif

		dec	esi			; Back up one instruction...
		mov	eax, esi		; Get our address in EAX
		sub	eax, ebp		; And subtract our base for
						; an invalid instruction

		push	eax			; Save our return code

		inc	esi	; WRONG!!!!

		jmp	short emulateEnd

; We get to invalidPageInsByte if it's a double byte invalid opcode

ifdef MASM
invalidPageInsByte::
endif

		sub	esi, 02h		; Back up two instructions...
		mov	eax, esi		; Get our address in EAX
		sub	eax, ebp		; And subtract our base for
						; an invalid instruction

		push	eax			; Save our return code

		jmp	short emulateEnd

ifdef MASM
instReturn::
endif

		mov	eax, 080000000h		; Indicate successful exec

		push	eax			; Save our return code

; Now let's tuck away the virtual registers for next time

emulateEnd:

ifdef MASM
		mov	_m6809a, bh		; Store B
		mov	_m6809b, bl		; Store A
		mov	_m6809x, cx		; Store X
		mov	_m6809cc, dh		; Store CC
		mov	_m6809dp, dl		; Store DP
		sub	esi, _m6809Base		; Knock off physical address
		mov	_m6809pc, si		; And store virtual address
endif

		pop	eax			; Restore our return code

		pop	edi			; Restore registers
		pop	esi
		pop	edx
		pop	ecx
		pop	ebx
		pop	ebp

		ret

ifdef MASM
OutOfRange::
endif

		mov	eax, 80000001h		; Indicates a boundary
						; violation (outside 64K)

		push	eax			; Save our return code

		jmp	short emulateEnd

ifdef MASM
m6809exec_	endp
endif

_TEXT		ends
		end
